/** 
    This file defines pragmas for BIAS WIN32 compilation.

    It is used as a central place to disable some warnings for pedantic W4 compilation 
    instead of spreading pragmas over all source files directly
    (e.g. minpack and other dirty code)
    Please include it *only* in CPP files directyl to avoid side effects as
    #include <Base/Common/BIASpragma.hh>

    If you need it in *header* files, use: 
      #include <Base/Common/BIASpragmaStart.hh>
      //... the bad code/ bad foreign headers etc...
      #include <Base/Common/BIASpragmaEnd.hh>
    to avoid interfernec with client include programs.

    You can switch on/off each individual warning messages globally, here.
    as long a s "dirty code" exists in BIAS/MIP.
    This way you can compile your own project with W4
    without taking care of possible internal problems of BIAS or third party libraries including warning noise from STL.

    And if you want to improve BIAs, just enable warning one by one and fix them in the code.

    @author Jan Woetzel 08/2005
*/

#ifdef WIN32

/* suppress STL warning noise (Jan Woetzel 2005)
 see:  http://www.codeproject.com/vcpp/stl/stl_without_warnings.asp?df=100&forumid=2917&exp=0&select=114698&tid=80903
 warning numbers get enabled in yvals.h: */
/* #include <yvals.h>  */

#define _CRT_SECURE_NO_DEPRECATE

#pragma warning (disable: 4005)
/* #pragma warning (disable: 4018) */ /* signed/unsigned mismatch */
#pragma warning (disable: 4099) /* .pdb not found, no debugging information (not catchable in source) */
#pragma warning (disable: 4100) /* unreferenced formal parameter */
#pragma warning (disable: 4127)
#pragma warning (disable: 4131)
#pragma warning (disable: 4204)
#pragma warning (disable: 4210)
#pragma warning (disable: 4217)
#pragma warning (disable: 4221) /* Linker warning: no public symbols found, members will be inaccessible */
#pragma warning (disable: 4231)
#pragma warning (disable: 4238)
#pragma warning (disable: 4244)
/* #pragma warning (disable: 4245) */ /* conversion from 'type1' to 'type2', signed/unsigned mismatch */
#pragma warning (disable: 4251)
#pragma warning (disable: 4275)
#pragma warning (disable: 4290)
#pragma warning (disable: 4305)
#pragma warning (disable: 4310)
#pragma warning (disable: 4389)
#pragma warning (disable: 4505)
#pragma warning (disable: 4511)
#pragma warning (disable: 4512) /* 'class' : assignment operator could not be generated */
#pragma warning (disable: 4530)
/* #pragma warning (disable: 4663) */  /* C++ language change: to explicitly specialize class template 'vector' */
#pragma warning (disable: 4701)
#pragma warning (disable: 4702) /* unreachable code */
#pragma warning (disable: 4706)
/* #pragma warning (disable: 4710) */ /* function not inlined */
/* #pragma warning (disable: 4786) */ /* identifier was truncated to 'number' characters in the debug information */

/* #pragma warning (disable: 4798) */ /* linker tool confict (debug/release, threadde/single etc) (not catchable in source) */
/* #pragma warning (disable: 4799) */ /* .pdb not found, no debugging information (not catchable in source) */
#pragma warning (disable: 4996) /* deprecated warning (for VS2005) */
#endif 

