// must be first: 
#include "precompiled.h"

#define _CRT_SECURE_NO_DEPRECATE 
#include <stdio.h>

// Jan Woetzel 03/2005

#include "FileHandling.hh"

// STD
#include <iostream>
#include <fstream>

// BIAS
#include "Base/Debug/Error.hh"


// for transfrom:
#include <algorithm>
#include <cctype>


using namespace std;
//using namespace BIAS:



// JW
bool BIAS::FileReadable( const std::string & s)
{
	ifstream ifs( s.c_str() );
	// ok, if file could be opended and is redable
	bool ok = ifs.good();
	ifs.close();
	return ok;
}


// JW
void BIAS::SplitName(const std::string & fullname,
					 string & dir,
					 string & base,
					 string & suffix )
{ 
	// find from right 
	// expr: dir/subdir/base.ext
	string::size_type posL = fullname.rfind('/'); // separator for directory
	string::size_type posR = fullname.rfind('.'); // separator for suffix

	// which separators were found ?
	bool foundDir = (posL!=std::string::npos);
	bool foundSuf = (posR!=std::string::npos);

	if ( foundDir && foundSuf ) {
		// found path and suffix. base is inner part
		dir    = fullname.substr(0, posL+1 ); // start, length
		base   = fullname.substr(posL+1, (posR-posL-1)  );
		suffix = fullname.substr(posR, (fullname.size()-posR)  );    

	} else if ( foundDir && !foundSuf) {
		dir    = fullname.substr(0, posL+1 ); // start, length
		base   = fullname.substr(posL+1, (fullname.size()-posL) );
		suffix = ""; // no suffix found.

	} else if ( !foundDir && foundSuf) {
		dir    = "";
		base   = fullname.substr(0, posR);
		suffix = fullname.substr(posR, (fullname.size()-posR) );

	} else {    
		// is already a basename. 
		// TODO: cerr msg seems to cause a LEAK in string !!! (jw)
		// BIASERR("could not split path. delimiters not found. ");   
		// cerr<<"blah"; // produces a valgrind memory leak/exception
		dir="";
		base=fullname;
		suffix="";
	};
}


//JW
std::string BIAS::Directory(const std::string & fullname){
	string dir, base, suf;
	BIAS::SplitName(fullname,
		dir, base, suf );
	return dir;
}


//JW
std::string BIAS::Basename(const std::string & fullname){
	string dir, base, suf;
	BIAS::SplitName(fullname,
		dir, base, suf );
	return base;
}


//JW
std::string BIAS::Suffix(const std::string & fullname){
	string dir, base, suf;
	BIAS::SplitName(fullname,
		dir, base, suf );
	return suf;
}


std::string  
BIAS::Extension(const std::string & fullname)
{
  string suf = Suffix(fullname);
  string ext=suf;

  // remove the dot to get extension from suffix
  string::size_type pos = suf.rfind('.'); // separator
  const bool found = (pos!=std::string::npos);

  // cut the dot if found, kepp full ext otherwise
  if (found) 
    ext = suf.substr(pos+1, (suf.size()-pos-1) );

  return ext;
}


//JW
void BIAS::LogCommandLine( int argc,char** argv, std::ostream & os){
	for (int i=0; i<argc; i++){
		os<<argv[i]<<" "<<endl;
	}
	os<<endl;
	os<<"argc="<<argc<<endl;
	for (int i=0; i<argc; i++){
		os<<"argv["<<i<<"]="<<argv[i]<<endl;
	}
	os<<flush;
}


void BIAS::LogCommandLine( int argc,char** argv, const std::string & filename){
	ofstream ofs;
	ofs.open(filename.c_str()
		//, ios::app 
		);
	if (!ofs.good()) {
		BIASERR("could not open "<<filename<<" for LogCommandLine.");
		LogCommandLine(argc, argv, std::cout);
	} else {
		LogCommandLine(argc, argv, ofs);    
	};
	ofs.close();
}



// return leading zero std::string of number 
// not elegantly but works.  JW 05/2003
std::string 
BIAS::LeadingZeroString( const int & n, 
						const unsigned int & digits ) 
{ 
	char bufC[10]; //  to convert the integer into a string
	// s#thsi switch is not pretty but works... 
	switch (digits) {
  case 1:
	  sprintf( bufC,"%01i", n);
	  break;
  case 2:
	  sprintf( bufC,"%02i", n);
	  break;
  case 3:
	  sprintf( bufC,"%03i", n);
	  break;
  case 4:
	  sprintf( bufC,"%04i", n);
	  break;
  case 5:
	  sprintf( bufC,"%05i", n);
	  break;
  case 6 :
	  sprintf( bufC,"%06i", n);
	  break;
  case 7:
	  sprintf( bufC,"%07i", n);
	  break;
  case 8:
	  sprintf( bufC,"%08i", n);
	  break;
  case 9:
	  sprintf( bufC,"%09i", n);
	  break;
  case 10:
	  sprintf( bufC,"%010i", n);
	  break;
  case 11:
	  sprintf( bufC,"%011i", n);
	  break;
  case 12:
	  sprintf( bufC,"%012i", n);
	  break;
  case 13:
	  sprintf( bufC,"%013i", n);
	  break;
  case 14:
	  sprintf( bufC,"%014i", n);
	  break;
  case 15:
	  sprintf( bufC,"%015i", n);
	  break;
  case 16:
	  sprintf( bufC,"%016i", n);
	  break;


  default:
	  BIASERR("error: overflow for digits="<<digits<<" using 10 digits as fallback,now");
	  sprintf( bufC,"%010i", n);
	};
	std::string str;
	str = bufC;
	return str;
}


/// type casting helper function for Suffix/ExtensName JW
char BIAS::ToLower_c(char c) {
	return char(tolower(c));
}

char BIAS::ToUpper_c(char c) {
	return char(toupper(c));
}


/// @return lower case string of input JW
std::string BIAS::LowerCase(const std::string & input)
{
	string s(input);
	// convert each char to lower case
	transform(s.begin(), s.end(), s.begin(), ToLower_c); 
	return s;
}


/// @return lower case string of input JW
std::string BIAS::UpperCase(const std::string & input)
{
	string s(input);
	// convert each char to lower case
	transform(s.begin(), s.end(), s.begin(), ToUpper_c); 
	return s;
}

