#ifndef __FileHandling_hh__
#define __FileHandling_hh__

/** File handling tools
@todo: correct handling of Win32 SAMBA paths including \\machine\....
@author Jan Woetzel 02/2005
*/

#include <bias_config.h>

#include <string>

#define DEFAULT_LEADING_ZEROS 5



namespace BIAS {

  /* @return true if the file exists can be pendedis readable on file system */

  bool BIASCommon_EXPORT 
    FileReadable( const std::string & s);


  /**  split full path into:
  - directory
  - basename
  - .suffix (inlcluding dot) 
  @todo: Extract machine name or IP for SMB network shares
  for WIN32, e.g. \\machine\share and fix backslashes on  Win32.
  @author Jan Woetzel 03/2005 */
  // TODO: \todo WIN: fix to make \, \\ and / work on directories (jw)
  void BIASCommon_EXPORT
    SplitName(const std::string & fullname
    ,std::string & dir
    ,std::string & base
    ,std::string & suffix );

  /** get Directory from fullname path JW */
  std::string BIASCommon_EXPORT
    Directory(const std::string & fullname);

  /**  get Basename of file, no suffix, no direcory. */
  std::string  BIASCommon_EXPORT
    Basename(const std::string & fullname);

  /**  get .suffix of file including the ".dot"
  No basename, no direcory. */
  std::string  BIASCommon_EXPORT
    Suffix(const std::string & fullname);

  /**  get Extension of file without the "dot"
  No basename, no direcory. */
  std::string  BIASCommon_EXPORT
    Extension(const std::string & fullname);


  /// type correct helper wrapper for tolower required for stl transformations JW
  char ToLower_c(char c);
  /// type correct helper wrapper for toupper required for stl transformations JW
  char ToUpper_c(char c);


  /// @return lower case string of input JW
  std::string BIASCommon_EXPORT LowerCase(const std::string & input);

  /// @return upper case string of input JW
  std::string BIASCommon_EXPORT UpperCase(const std::string & input);

  /** write compilation time/date and all command line arguments 
  to a log file/stream JW */
  void BIASCommon_EXPORT 
    LogCommandLine( int argc,char** argv, std::ostream & os);

  void BIASCommon_EXPORT 
    LogCommandLine( int argc,char** argv, const std::string & filename);

  /** Creates astring with leading zeroes JW */
  std::string BIASCommon_EXPORT
    LeadingZeroString( const int & n, 
    const unsigned int & digits=DEFAULT_LEADING_ZEROS ) ;



} // namespace BIAS
#endif // do not add code after this line 
