/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Base/Geometry/RMatrixBase.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[]){
  if (argc!=4){
    cerr << argv[0] << ": PhiX PhiY PhiZ"<<endl;
    cerr << "  specify the three rotation angles"<<endl;
    return -1;
  }
  double X = atof(argv[1]), Y = atof(argv[2]), Z = atof(argv[3]);

  RMatrixBase Rxyz; 
  Rxyz.SetXYZ(X, Y, Z);
  cout << "R.SetXYZ("<<X<<", "<<Y<<", "<<Z<<") = "<<Rxyz<<endl;
  RMatrixBase Rzyx; 
  Rzyx.SetZYX(X, Y, Z);
  cout << "R.SetZYX("<<X<<", "<<Y<<", "<<Z<<") = "<<Rzyx<<endl;

  double x, y, z;

  if (Rxyz.GetRotationAnglesXYZ(x, y, z)==0){
    cout <<"Rxyz was composed from the following angles: "<<x<<" "<<y<<" "
         <<z<<endl;
  } else {
    cout <<"unable to determine angles" <<endl;
  }

  if (Rzyx.GetRotationAnglesZYX(x, y, z)==0){
    cout <<"Rzyx was composed from the following angles: "<<x<<" "<<y<<" "
         <<z<<endl;
  } else {
    cout <<"unable to determine angles" <<endl;
  }

  //return 0;

  Vector3<double> xyz_axis, zyx_axis;
  double xyz_angle, zyx_angle;

  Rxyz.GetRotationAxisAngle(xyz_axis, xyz_angle);
  cerr << "axis: "<< xyz_axis <<"\tangle: "<< xyz_angle<<endl;

  Rzyx.GetRotationAxisAngle(zyx_axis, zyx_angle);
  cerr << "axis: "<< zyx_axis << "\tangle: "<< zyx_angle<<endl;

  RMatrixBase Rxyz2; Rxyz2.Set(xyz_axis, xyz_angle);
  RMatrixBase Rzyx2; Rzyx2.Set(zyx_axis, zyx_angle);

  cerr << "Rxyz: "<< Rxyz << "Rxyz2 (from axis and angle): " << Rxyz2 << endl;
  cerr << "Rxyz * Rxyz2': "<< Rxyz * Rxyz2.Transpose() << endl;

  cerr << "Rzyx: "<< Rzyx << "Rzyx2 (from axis and angle): " << Rzyx2 << endl;
  cerr << "Rzyx * Rzyx2': "<< Rzyx * Rzyx2.Transpose() << endl;

  return 0;
} 
