/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



//
// Check for valid x and y coordinates (if BIAS_DEBUG is defined)
// In case of error display the functionname
// 
#ifdef BIAS_DEBUG
#define CheckCoordinates(x,y,functionname) \
  if (x < 0 || x >= GetWidth()) { \
    BIASERR("x-coordinate out of range in "<<functionname<<" "<<x); \
  } \
  if (y < 0 || y >= GetHeight()) { \
    BIASERR("y-coordinate out of range in "<<functionname<<" "<<y); \
  }
#else
#define CheckCoordinates(x,y,functionname)
#endif


// Check for valid channel nr. in Debug mode. Do nothing in Release
// In case of error display the functionname (JW 09/2003)
#ifdef BIAS_DEBUG
#define CheckChannel(channel,functionname) \
  if (channel >= GetChannelCount()) { \
    BIASERR("channel"<<channel<<" out of range in "<<functionname); \
  }
#else
#define CheckChannel(channel,functionname)
#endif



inline bool ImageBase::SamePixelAndChannelCount(const ImageBase& Image) const {
  return ( (GetPixelCount() == Image.GetPixelCount()) && 
           (GetChannelCount() == Image.GetChannelCount()) ); 
}

inline void ImageBase::SetZero()
{
  if (!IsEmpty()){
    memset(GetImageData(), 0, GetWidthStep()*GetHeight());
  }
}

inline void ImageBase::ReleaseImageDataPointer()
{       
  delete[] (char**)ImageDataArray_;
  delete[] (char*)ImageData_;
  ImageDataArray_=NULL;
  ImageData_=NULL;
}

inline unsigned int 
ImageBase::GetPixelPosition(unsigned int x, unsigned int y, 
                            unsigned short int channel) const
{ 
  CheckCoordinates(x, y, "GetPixelPosition");

  if (IsPlanar()) 
    return y*Width_+x+channel*PixelCount_;
  else  
    return y*Width_*ChannelCount_ +x*(ChannelCount_)+channel; 
}

bool ImageBase::IsPositionInImage(const int& x,const int& y) const{
  return (x>=0 && x<(int)Width_ && y>=0 && y<(int)Height_);
}

int  ImageBase::MoveToClosestPositionInImage(int& x, int& y) const {
#ifdef BIAS_DEBUG
  if (!((Width_>0)&&(Height_>0))) {
    BIASERR("Image is not initialiozed correctly: "<<Width_<<"x"<<Height_);
    abort();
  }
#endif 
  int ret = 0;
  if (x<0) {
    x = 0;
    ret = 1;
  } else if (x>=(int)Width_) {
    x = Width_-1;
    ret = 2;
  }
  if (y<0) {
    y = 0;
    ret = 3;
  } else if (y>=(int)Height_) {
    y = Height_-1;
    ret = 4;
  }
  return ret;
}
