/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "bias_config.h"

#include "Image.cpp"
#include "ImageConvert.cpp"
#include "Operators.cpp"
#include "ImageDraw.cpp"
#include "ImageConvertForeign.cpp"

using namespace BIAS;
using namespace std;

#define INSTANCE(type)\
template class BIASImageBase_EXPORT Image<type>;\
template class BIASImageBase_EXPORT ImageDraw<type>;\
template BIASImageBase_EXPORT int ImageConvert::IP_ToGrey(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::ToGrey(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToRGB(Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToHSV(Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToHSL(Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::TohsL(Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleaved(Image<type>& source,\
                                         Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedRGB(Image<type>& red, \
                                            Image<type>& green,\
                                            Image<type>& blue,\
                                            Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToPlanar(Image<type>& source,\
                                    Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleaved(Image<type>& source,\
                                    Image<type>& Image1, Image<type>& Image2,\
                                    Image<type>& Image3);\
template BIASImageBase_EXPORT int ImageConvert::IP_RGBToGrey_(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::IP_YUV422ToGrey_(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::BGRToRGB_(Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::YUV422ToRGB_(Image<type>& source,\
                                        Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::GreyToRGB_(Image<type>& source,\
                                      Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::RGBToGrey_(const Image<type>& source,\
                                      Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::HSLToGrey_(const Image<type>& source,\
                                      Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::YUV422ToGrey_(const Image<type> &source,\
                                         Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToPlanarYUYV422_(Image<type>& source,\
                                            Image<type>& dest);    \
template BIASImageBase_EXPORT int ImageConvert::ToPlanarRGB_(Image<type>& source,\
                                        Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedYUYV422_(Image<type>& source,\
                                                 Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedRGB_(Image<type>& source,\
                                             Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleavedYUYV422_(Image<type>& source,\
                                            Image<type>& Y, \
                                            Image<type>& U,\
                                            Image<type>& V);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleavedRGB_(Image<type>& source,\
                                        Image<type>& R, \
                                        Image<type>& G,\
                                        Image<type>& B);\
template BIASImageBase_EXPORT int ImageConvert::BayerToGrey_(const Image<type>& source,\
                                       Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::BayerToRGB_(Image<type>& source,\
                                       Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBSlow_(Image<type>& source,\
                                       Image<type>& dest);

#ifdef BIAS_HAVE_DV
#define INSTDV(dvtype)\
template BIASImageBase_EXPORT int ImageConvert::DVToRGB_(const Image<dvtype>& source,\
                                    Image<dvtype>& dest,unsigned char quality);
#else
#define INSTDV(dvtype)
#endif


#ifdef BIAS_HAVE_OPENCV
#define INSTOPENCV(cvt)\
template BIASImageBase_EXPORT int ImageConvert::BIAS2ipl(const Image<cvt>& source, IplImage* &dest);\
template BIASImageBase_EXPORT int ImageConvert::Ipl2BIAS(IplImage* source, Image<cvt>& dest);
#else
#define INSTOPENCV(cvt)
#endif // HAVE_OPENCV

#define INSTI(atype)   INSTANCE(atype) INSTOPENCV(atype) INSTDV(atype)

INSTI(unsigned char)
INSTI(float)

#ifdef BUILD_IMAGE_INT
INSTI(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INSTI(char)
//INSTI(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INSTI(short)
#endif

//JW: 
#if defined(BUILD_IMAGE_USHORT) || defined(WIN32) 
INSTI(unsigned short)
#endif

#ifdef BUILD_IMAGE_DOUBLE
INSTI(double)
#endif
#ifdef BUILD_IMAGE_UINT
INSTI(unsigned int)
#endif

