##
## Try to find CMU 1394 Camera driver library  
## available from:
## www-2.cs.cmu.edu/~iwan/1394/
##
## Once run this will define: 
##
## 1394CameraCMU_FOUND
## 1394CameraCMU_INCLUDE_DIR
## 1394CameraCMU_LIBRARIES
## 1394CameraCMU_HEADER  This is a hack to allow command completion in MSVS
##
##  (1394CameraCMU_LINK_DIRECTORIES: not yet...)
##
## Jan Woetzel 02/2004.
## www.mip.informatik.uni-kiel.de/~jw
## 
## tested with version 6.2, 6.3 (on Windows XP)
## --------------------------------

IF (NOT WIN32)
  MESSAGE("Find1394CameraCMU.cmake: This library supports only WIN32. skipping.")
  SET(1394CameraCMU_FOUND FALSE)
ELSE (NOT WIN32)

FIND_PATH(1394CameraCMU_INCLUDE_DIR 1394Camera.h
  $ENV{1394CameraCMU_HOME}/include
  C:/source/1394camera/include
  D:/source/1394camera/include
  C:/source/1394camera62/include
  D:/source/1394camera62/include
  C:/source/1394camera63/include
  D:/source/1394camera63/include
  
)
#MESSAGE("DBG 1394CameraCMU_INCLUDE_DIR=${1394CameraCMU_INCLUDE_DIR}")  
  
FIND_LIBRARY(1394CameraCMU_LIBRARY
  NAMES 1394camera 1394Camera 1394camerad 1394Camerad
  PATHS 
  $ENV{1394CameraCMU_HOME}/lib
  C:/source/1394camera/lib
  D:/source/1394camera/lib
  C:/source/1394camera62/lib
  D:/source/1394camera62/lib
  C:/source/1394camera63/lib
  D:/source/1394camera63/lib 
)
#MESSAGE("DBG 1394CameraCMU_LIBRARY=${1394CameraCMU_LIBRARY}")
  
## --------------------------------

IF(1394CameraCMU_LIBRARY)
  SET(1394CameraCMU_LIBRARIES ${1394CameraCMU_LIBRARY})
ELSE(1394CameraCMU_LIBRARY)
  MESSAGE(SEND_ERROR "1394CameraCMU library not found.")
ENDIF(1394CameraCMU_LIBRARY)

IF(NOT 1394CameraCMU_INCLUDE_DIR)
  MESSAGE(SEND_ERROR "1394CameraCMU include dir not found.")
ENDIF(NOT 1394CameraCMU_INCLUDE_DIR)

IF(1394CameraCMU_LIBRARIES AND 1394CameraCMU_INCLUDE_DIR)
  SET(1394CameraCMU_FOUND TRUE)
  
  
  SET( 1394CameraCMU_HEADER 
	"${1394CameraCMU_INCLUDE_DIR}/1394Camera.h"
	"${1394CameraCMU_INCLUDE_DIR}/1394CameraControl.h"
	"${1394CameraCMU_INCLUDE_DIR}/1394CameraControlSize.h"
	"${1394CameraCMU_INCLUDE_DIR}/1394CameraControlTrigger.h"
	"${1394CameraCMU_INCLUDE_DIR}/1394camapi.h"
	"${1394CameraCMU_INCLUDE_DIR}/1394common.h"
  )
  #MESSAGE("DBG 1394CameraCMU_HEADER=${1394CameraCMU_HEADER}")
  SOURCE_GROUP(CMU1394 FILES ${1394CameraCMU_HEADER})
  
ELSE(1394CameraCMU_LIBRARIES AND 1394CameraCMU_INCLUDE_DIR)
  SET(1394CameraCMU_FOUND FALSE)
ENDIF(1394CameraCMU_LIBRARIES AND 1394CameraCMU_INCLUDE_DIR)

MARK_AS_ADVANCED(
  1394CameraCMU_INCLUDE_DIR
  1394CameraCMU_LIBRARY
  1394CameraCMU_LIBRARIES
)
ENDIF (NOT WIN32)
