/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __CornerDetectorKLT_hh__
#define __CornerDetectorKLT_hh__

#include "CornerDetectorGradient.hh"


namespace BIAS {

 
  
  /** @class CornerDetectorKLT
      @ingroup g_feature
      @brief computes the cornerness as the smaller eigenvalue of the 
             structure tensor matrix
      @author woelk 09/2004 */

  template <class StorageType, class CalculationType>
  class BIASFeatureDetector_EXPORT CornerDetectorKLT 
    : public CornerDetectorGradient<StorageType, CalculationType>
  {
  public:
    CornerDetectorKLT();

    CornerDetectorKLT(Param& para, const std::string& postfix="");

    CornerDetectorKLT(Param& para, const int grp, const std::string& postfix);

    virtual ~CornerDetectorKLT();

  protected:    
    /** @brief Computes the smaller eigenvalue of the structure tensor _sgxx, 
        _sgyy, _sgxy over the image and stores it in _Cornerness. 
        Also fills up _FeatList.
        @author woelk 09/2004 */
    virtual int _ComputeCornerness(Image<CalculationType>& mineig);

  }; // class

} // namespace

#endif // __CornerDetectorKLT_hh__
