/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __DeInterlace_hh__
#define __DeInterlace_hh__

#include "Convolution.hh"

#include <Base/Common/BIASpragmaStart.hh>

namespace BIAS 
{
  /** @class Mean
      @ingroup g_filter
      @brief deinterlacer filter
      
      @author ischiller 01/2005 */

  enum BIAS_DEINTERLACE_TYPE{
    BIAS_DI_121, //calculate new pixel from old neighbouring pixels with  new = (old1+2*old2+old3)/4
    BIAS_DI_22  // calculate new pixel from old neighbouring pixels with  new = (old1+old2)/2
  };


  template <class InputStorageType, class OutputStorageType>
  class BIASFilter_EXPORT DeInterlace : public Convolution<InputStorageType, OutputStorageType>
  {
  public:
    DeInterlace();

    DeInterlace(Param& para, const std::string& postfix="");
    
    DeInterlace(Param& para, const int group_id, const std::string& postfix);
    
    DeInterlace(const DeInterlace<InputStorageType, OutputStorageType>& m) {};
    
    ~DeInterlace();
    
    /** @brief averages over a region with constant weights
        @author woelk 09/2004 */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& dst);

    /** @brief only calls filter, for consistency of params */
    virtual int FilterInt(const Image<InputStorageType>& src, 
                          Image<OutputStorageType>& dst); 
    
    /** @brief only calls filter, for consistency of params */
    virtual int FilterFloat(const Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);

    /** Set if the  even or odd lines in the images should be used  */
    void SetUseEvenLines(bool bEvenLines=true);

    /** Set the Type of deinterlacing e.g. mean of 2 pixel or weighting with 121... */
    void SetDeInterlaceType(BIAS_DEINTERLACE_TYPE type);

    /** Set the downsampling factor while filter */
    void SetDownSamplingByTwo(bool bDoDownSamplingByTwo = true );
  protected:
   
    void _AddParam(Param& para, const int group_id, 
                   const std::string& postfix);

    bool bUseEvenLines_;
    bool bDoDownSamplingByTwo_;
    BIAS_DEINTERLACE_TYPE eDeInterlaceType_;

    //using Convolution<InputStorageType, OutputStorageType>::_fm;
  }; // class




} // namespace

#include <Base/Common/BIASpragmaEnd.hh>

#endif // __Mean_hh__
