/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __2DFFT_TILES_hh__
#define __2DFFT_TILES_hh__


#include "FFT2D.hh"

namespace BIAS {

  template <class InputStorageType, class OutputStorageType>
  class BIASFilter_EXPORT FFT2D_Tiles 
    : public FFT2D<InputStorageType, OutputStorageType>
  {
    /** wrapper to the fftw3 library
    use fast as follows:
    - call Init() with appropriate size
    - call Filter()

    Jan Woetzel 2005: This class is practically *NOT* implemented !
    \todo remove FFT2D_Tiles or really implement it. (JW)    

    @author evers 2004/01 */
  public:
    FFT2D_Tiles();
    ~FFT2D_Tiles();

    virtual void Init(int /*imgsizex*/, int /*imgsizey*/) 
    {
      BIASERR("Can not use FFT2D_Tiles::Init(width,height)");
    }
    /** @brief Initialize FFT to transform image tile- (patch-)wise
    */
    virtual void Init(int width, int height,
      int tilewidth, int tileheight);

    /** cal this before a second call to Init() */
    void Release();    

    /** dst.GetChannelCount()==2*src.GetCHannelCount() 
    The result is not normalized! */
    virtual int Filter(const Image<InputStorageType>& src, 
      Image<OutputStorageType>& dst);

    /** dstX.GetChannelCount()==src.GetCHannelCount() 
    The result is not normalized! */
    virtual int Filter(const Image<InputStorageType>& src, 
      Image<OutputStorageType>& dst1, 
      Image<OutputStorageType>& dst2);

    /** Transform forward and get absolute value from complex result
    The result is not normalized! */
    virtual int TransformAbs(const Image<InputStorageType>& src, 
      Image<OutputStorageType>& dst);

    /** Transform reverse, src _must_ be of _SizeX,_OutSizeY,2
    */
    virtual int TransformReverse(const Image<OutputStorageType>& src,
      Image<InputStorageType>& dst);



  protected:

    virtual int Forward_(const Image<InputStorageType>& src);

    // assumes src.GetChannelCount() == 2
    virtual int Reverse_(const Image<OutputStorageType>& src);

    int _TileSizeX, _TileSizeY;
    int _NumTilesX, _NumTilesY; 
  };

} // namespace

#endif // __FFT2D_Tiles_hh__
