/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __HMatrixEstimation_hh__
#define __HMatrixEstimation_hh__
#include "bias_config.h" 

#include <Geometry/HMatrix.hh>
#include <Base/Geometry/Normalization.hh>

namespace BIAS {

  /** @class HMatrixEstimation
      @ingroup g_geometry
      @brief estimate 3x3 Matrix (homography, affine transformation, ...)

      code was originally in HMatrix, moved and extended by koeser 12/2005
  */
  class BIASGeometry_EXPORT HMatrixEstimation
  {
  public:
    
    explicit HMatrixEstimation(bool normalizehartley=true) {
      normalizeHartley_ = normalizehartley; }

    /** calculatinges H using the basic linear transformation algorithm
        (see Hartley, Zisserman "Multiple View Geometry", p.73)
        p'=Hp -> (homg. coo) [p']x Hp = 0 
        3 row linear system, but rank 2 -> take only first two rows
        p=1...numpoints -> 2*numpoints equations
        solve A*h=0 with SVD

        @author Woelk/Frahm */
    int Compute(const std::vector<BIAS::HomgPoint2D>& frompoints, 
                const std::vector<BIAS::HomgPoint2D>& topoints, 
                HMatrix& H) const;

    /** @brief compute affine transformation (not using DLT !)
        @author koeser 12/2005 */
    int ComputeAffine(const std::vector<BIAS::HomgPoint2D>& frompoints, 
                      const std::vector<BIAS::HomgPoint2D>& topoints, 
                      HMatrix& H) const;
  
  protected:
    /// flag indicating whether we should normalize the given points before
		/// and denormalize the fmatrix after computation, so that it refers to
		/// the original points
		bool normalizeHartley_;
	
		/// the normalization object
		Normalization normalization_;
  };


}// namespace

#endif // __HMatrixEstimation_hh__
