/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _BIAS_RMatrix_hh_
#define _BIAS_RMatrix_hh_

#include "bias_config.h" 

#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector4.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Geometry/RMatrixBase.hh>

namespace BIAS {

/** @class RMatrix 
    @ingroup g_geometry
    @brief 3D rotation matrix 
    
    @author JMF
    @status finished (07/05/2002)
**/

  class BIASGeometry_EXPORT RMatrix : public RMatrixBase {

  public:

    inline RMatrix()
      : RMatrixBase(){};

    explicit RMatrix(const MatrixInitType& i)
      : RMatrixBase(i){};

    /// initialization from rotation axis w and angle phi (Rodriguez)
    RMatrix(Vector3<ROTATION_MATRIX_TYPE> w, ROTATION_MATRIX_TYPE phi):
      RMatrixBase(w,phi) {};

    virtual ~RMatrix();

    inline RMatrix(Matrix3x3<ROTATION_MATRIX_TYPE> mat)
      : RMatrixBase(mat) {};

    RMatrix(const TNT::Matrix<ROTATION_MATRIX_TYPE>& mat);
    /** Enforces scale ==1.0 anw right-handed rotation
	@author koeser
     */
    int EnforceRConstraints();
  };
} // namespace BIAS

#endif
