
#include "CameraViewController.hh"
#include "glstuff.h"

using namespace std;
using namespace BIAS;


// JW
CameraViewController::~CameraViewController(){}


CameraViewController::CameraViewController(){
    InitMembers();
}


void CameraViewController::SetAspect(){
    if (height_==0) {
        aspect_=1.0;
    } else  {
        aspect_= float(width_)/float(height_);
    }
}

void CameraViewController::InitMembers(){
    mode_ = InspectionMoveCAbsolute;

    x0_=0;
    y0_=0;
    width_ = 1024; // size is HACk for now and can be handled by Reshape
    height_= 768;
    fovY_=36.0; // degree
    SetAspect();
    //zNear_=1.0;
    zNear_=0.1;
    zFar_ =1000.0;

    // some default C (e.g. froma nice calibration)
    //C_.Set(0, 0, 36.0);
    C_.Set(0.1926, 0.85514, 29.8252);
    C_ += Vector3<double>(0, 0, 0.1);

    //viewAt_.SetZero();
    //viewAt_.Set(0,0,28.0);
    viewAt_ = C_ + Vector3<double>(0.0,0.0, -10.0);
    up_.Set(0.0, 1.0, 0.0);


    // update steps:
    CStep_.Set(0.5, 0.5, 0.1);
    //CStep_.Set(2.0, 2.0, 2.0);
    //viewAtStep_=CStep_;
    viewAtStep_.Set(1.0, 1.0, 1.0);
}


void CameraViewController::DisplayGL(){
    DisplayGLInternal();
    DisplayGLExternal();
}

void CameraViewController::DisplayGLInternal(){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // dbg
    SetAspect();

    glViewport(
         (GLint)x0_
        ,(GLint)y0_
        ,(GLsizei)width_
        ,(GLsizei)height_ 
        );
    gluPerspective(
        fovY_,
        aspect_,
        zNear_, zFar_
        );

    glMatrixMode(GL_MODELVIEW);
}


void CameraViewController::DisplayGLExternal(){
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(
        (GLdouble)C_[0],
        (GLdouble)C_[1],
        (GLdouble)C_[2],
        (GLdouble)viewAt_[0],
        (GLdouble)viewAt_[1],
        (GLdouble)viewAt_[2],
        (GLdouble)up_[0],
        (GLdouble)up_[1],
        (GLdouble)up_[2] 
        );
}


void CameraViewController::Update(const unsigned int axisNo
                                  ,const double & delta
                                  ,const bool useCurrentMode
                                  ,const int alternativeMode )
{
    int useMode=alternativeMode;
    if (useCurrentMode) useMode = mode_;
    switch(useMode){
        case InspectionMoveCAbsolute:
            BIASASSERT(axisNo<3);
            C_[axisNo] += delta* CStep_[axisNo];
            break;
        case InspectionMoveViewAtAbsolute:
            BIASASSERT(axisNo<3);
            viewAt_[axisNo] += delta* viewAtStep_[axisNo];
            break;
        default:
            BIASERR("unknown mode ");
    };
}


void CameraViewController::DisplayViewAt(const float & size){
    {glPushMatrix();
    //glEnable(GL_DEPTH_TEST);
    glTranslated(viewAt_[0], viewAt_[1], viewAt_[2] );
    glColor3f(0.0, 0.0, 0.75);
    glutWireCube(size);
    glPopMatrix();}
}


void CameraViewController::Set(BIAS::PMatrix & P,
                               const unsigned int w,
                               const unsigned int h,
                               const float & zNear,
                               const float & zFar
                               )
{
    C_ = P.GetC();
    viewAt_ = C_ + 100.0 * P.GetA().GetNormalized();
    up_ = P.GetV().GetNormalized();

    zNear_  = zNear;
    zFar_   = zFar;
    width_  = w;
    height_ = h;
    fovY_ = P.GetFieldOfViewY(width_  , height_);
    fovY_ *= 180.0 /M_PI ;
 
    SetAspect();
}


void CameraViewController::Print(std::ostream & os ){
  os<<"CameraViewController : "<<endl
    <<"fovY_   "<<fovY_<<endl
    <<"aspect_ "<<aspect_<<endl
    <<"zNear_  "<<zNear_<<endl
    <<"zFar_   "<<zFar_<<endl

    <<"x0_     "<<x0_<<endl
    <<"y0_     "<<y0_<<endl
    <<"width_  "<<width_<<endl
    <<"height_ "<<height_<<endl

    <<"C_      "<<C_<<endl
    <<"CStep_  "<<CStep_<<endl
    <<"viewAt_ "<<viewAt_<<endl
    <<"viewAtStep_ "<<viewAtStep_<<endl
    <<"up_     "<<up_<<endl
    <<"mode_   "<<mode_<<endl

    <<endl;
}

