#ifndef __CameraViewController_h__
#define __CameraViewController_h__

// ----------------------------------------------------------
// @author Jan Woetzel (c)  09/2003
// all rights reserved
#include <bias_config.h>
#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use this lib. 
#endif // BIAS_HAVE_OPENGL

#ifndef BIAS_HAVE_GLUT
#  error Please recompile BIAS with USE_GLUT to use this lib. 
#endif // BIAS_HAVE_GLUT

// must be first
#include <Gui/glstuff.h>

// STD
#include <iostream>

// BIAS
#include <Base/Debug/Error.hh>
#include <Base/Debug/DebugSimple.hh>
#include <Base/Math/Vector3.hh>
#include <Geometry/PMatrix.hh>


namespace BIAS {

	enum Navigationmode {
		InspectionMoveCAbsolute=0
		,InspectionMoveViewAtAbsolute
    //,ExamineTrackballmode
		//,InspectionModeRelative
		//,FlyMode
		//,PMatrixMode
	};


	/** \brief base for (interactively) manipulating a (virtual) camera view. 
	Typically used by OpenGL as a glulookat Update. 
	Navigator would be a good name, too.
	\author Jan Woetzel */
	class CameraViewController {
	public:
		virtual ~CameraViewController(); ///< destructor

		// ctors
		CameraViewController();

		/// any constructor should call this function 
		/// to initialize all data members (e.g. pointers to NULL)
		virtual void InitMembers();

		virtual void DisplayGL();
		virtual void DisplayGLInternal();
		virtual void DisplayGLExternal();

		/** map update of one axis (mouse wheel, delta, key...) 
		maps delta (e.g. mouse dragging motion or wheel) to pose update 
		dependeing on current mode JW */
		virtual void Update(const unsigned int axisNo
			,const double & delta
			,const bool useCurrentMode=true
			,const int alternativeMode = -1
			);

		virtual void DisplayViewAt(const float & size=1.0f);

		virtual void Set(BIAS::PMatrix & P,
			const unsigned int w,
			const unsigned int h,
			const float & zNear,
			const float & zFar );

    void SetAspect();

    void Print(std::ostream & os=std::cout );


  public:
    // internal params
		// projection frustum
		double fovY_;
		double aspect_;
		double zNear_;
		double zFar_;

		// viewport
		int x0_;
		int y0_;
		unsigned int width_;
		unsigned int height_;

		// external params : pose
		BIAS::Vector3<double> C_;
		BIAS::Vector3<double> CStep_;

		BIAS::Vector3<double> viewAt_;
		BIAS::Vector3<double> viewAtStep_;

		BIAS::Vector3<double> up_;

		Navigationmode mode_;

	}; // CameraViewController
} // namespace BIAS
#endif
