/**	Simple example for BIAS::ImageCanvasGLBase(Base) usage.
@author: Jan Woetzel */
#include <bias_config.h>

#include <Base/Common/LeakChecking.h>

#include "ExampleImageCanvasGLBase.hh"

// BIAS
#include <Gui/ConsoleRedirectIO.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>

// WX
#include <wx/sizer.h>
#include <wx/log.h>




using namespace std;

// required for wx RTTI dynamic construction: 
IMPLEMENT_APP(ExampleImageCanvasGLBaseApp);

// --------------------------------------------------------

bool ExampleImageCanvasGLBaseApp::OnInit()
{
	// utility: create a Redirection console for cout, cerr etc.
	ConsoleRedirectIO();
	// testing output: 
	cout<<"Hello cout"<<endl;
	cout<<"Hello cerr"<<endl;

	MyFrame *frame = new MyFrame( "ExampleImageCanvasGLBase", wxPoint(0,0), wxSize(640,480) );
	frame->Show(TRUE);
	SetTopWindow(frame);

	return TRUE;
}


// --------------------------------------------------------
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(ID_QUIT, MyFrame::OnQuit)
EVT_MENU(ID_ABOUT, MyFrame::OnAbout)
EVT_MENU(ID_LOADIMAGE, MyFrame::OnLoadImage)
END_EVENT_TABLE()
// --------------------------------------------------------


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
,p_canvas1(NULL)
,p_canvas2(NULL)
{
	// Create a simple top Menu
	wxMenu *menuFile = new wxMenu;  
	menuFile->Append( ID_ABOUT, "&About..." );

	menuFile->Append( ID_LOADIMAGE, "&Load Image...\tCtrl-L" );

	menuFile->AppendSeparator();
	menuFile->Append( ID_QUIT, "E&xit" );



	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append(menuFile, "&File");
	SetMenuBar(menuBar);

	// Create a simple bottom status bar 
	CreateStatusBar(1);

	// Canvas to test: 
	p_canvas1=new BIAS::ImageCanvasGLBase(this, -1);
	// create aseond canvas that shares the GL context: 
	p_canvas2=new BIAS::ImageCanvasGLBase(this, p_canvas1->GetContext(),-1 );

	// put them into sizers: 
	wxBoxSizer * sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add( p_canvas1 ,1,wxEXPAND|wxALL);
#if wxCHECK_VERSION(2, 6, 2)
	sizer->AddSpacer(4); // available in 2.6.2 but not 2.4.2 (JW)
#endif
	sizer->Add(p_canvas2 ,1,wxEXPAND|wxALL);

	// limit min. size:
	sizer->SetItemMinSize(p_canvas1, 200,100);
	sizer->SetItemMinSize(p_canvas2, 200,100);
	sizer->SetSizeHints(this);

	// set sizer and triggers Layout()
	SetSizerAndFit(sizer);

	// now work with the canvases: 
	// enable Mipmapping just for one of them to see the effect: 
	p_canvas1->EnableMipMapping(true);


	// HACK> context i screate dtoo laet on Linux
#ifdef WIN32
  this->LoadImages();
#endif // WIN32
	
	// set the image to display:
	//p_canvas1->Set(img);
	//p_canvas2->Set(img); // allocates GPU texture meory twice
	// TODO: share texobjs "id" instead: See wglShareLists (JW)
	//p_canvas2->Set(p_canvas1->GetTexobj() );
}


void MyFrame::OnQuit(wxCommandEvent& ){ 
	Close(TRUE); 
}


void MyFrame::OnAbout(wxCommandEvent& )
{
	wxString title("About");
	wxString msg; // message
	msg<<"About - "<<FUNCNAME<<"\n";
	msg<<"author: Jan Woetzel (c) 2003-2005\n";
	msg<<"build: "<<__DATE__<<" "<<__TIME__<<"\n";
	msg<<"from "<<__FILE__<<"\n";
	msg<<"GUI using: "<<wxVERSION_STRING<<"\n";
	msg<<"\n";
	COUT(msg);
	wxMessageBox(msg, title, wxICON_INFORMATION | wxOK );
}

void MyFrame::LoadImages()
{
  CALLINFO;
  // set the image to display:
  // an image we want to display using OpenGL:
  BIAS::Image<unsigned char> img;
  string filename(BIAS_TESTS_DATA "r4.jpg");
  if (0!=BIAS::ImageIO::Load( filename,img)){
    BIASERR("could not load image"<<filename);
    return;
  };
  cout<<"loaded "<<filename<<" for display "<<endl;

  // now work with the canvases: 
  if (p_canvas1!=NULL) {
    // set the image to display:
    p_canvas1->Set(img);       
  } else {
    BIASERR("p_canvas1 is NULL");
  }
  
  if (p_canvas2!=NULL) {
    // set the image to display:
    p_canvas2->Set(img);       
  } else { 
    BIASERR("p_canvas2 is NULL");
  }
  
  COUT(FUNCNAME<<" done."<<endl);
}



void MyFrame::OnLoadImage(wxCommandEvent& )
{
  //CALLINFO;
  LoadImages();
}

