/**	Simple example for BIAS::OpenGLCanvas(Base) usage.
@author: Jan Woetzel */
#include <bias_config.h>

#include <Base/Common/LeakChecking.h>

#include "ExampleOpenGLCanvas.hh"

// BIAS
#include <Gui/ConsoleRedirectIO.hh>
#include <Gui/GeometryGL.hh>

using namespace std;

// required for wx RTTI dynamic construction: 
IMPLEMENT_APP(ExampleOpenGLCanvasApp);

// --------------------------------------------------------
#ifndef BIAS_HAVE_GLUT
#  error Please recompile BIAS with USE_GLUT to compile this example. 
#endif // BIAS_HAVE_GLUT
#define GLUT_BUILDING_LIB
#include <GL/glut.h> 


///
/// your own implemnetation of the scenes GL calls: 
///

MyGLCanvas::MyGLCanvas(wxFrame* parent)
: 
BIAS::OpenGLCanvas( parent, -1,wxDefaultPosition, wxDefaultSize,0,"GL" ,(int*)(&attribsRGBASD) )
{}




void MyGLCanvas::DisplayCalls()
{
  //CALLINFO;
  CHECK_GL_ERROR;

  DisplayClear();
  //DisplayRenderMode();

  glMatrixMode(GL_PROJECTION);
  //glLoadIdentity();
  gluPerspective(67.5 /*fovy*/,
    GetAspectRatio(),
    0.1 /*zNear*/ ,
    100.0 /*zFar*/ );

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(1,1,3, /*eye position*/
    0,0,0, /* centered view on */
    0,1,0 /*up*/ );

  // animated scene
  BIAS::GeometryGL::DisplayCallsSampleDisplayListScene(this->time);

}


// --------------------------------------------------------

bool ExampleOpenGLCanvasApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  ConsoleRedirectIO();
  // testing output: 
  cout<<"Hello cout"<<endl;
  cout<<"Hello cerr"<<endl;

  MyFrame *frame = new MyFrame( "ExampleOpenGLCanvas", wxPoint(0,0), wxSize(640,480) );
  frame->Show(TRUE);
  SetTopWindow(frame);

  return TRUE;
}


// --------------------------------------------------------
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(ID_QUIT, MyFrame::OnQuit)
EVT_MENU(ID_ABOUT, MyFrame::OnAbout)
EVT_MENU(ID_TESTSHOT, MyFrame::OnTestShot)
END_EVENT_TABLE()
// --------------------------------------------------------


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
,p_canvas(NULL)
//,p_2nd_canvas(NULL)
{
  // Create a simple top Menu
  wxMenu *menuFile = new wxMenu;  
  menuFile->Append( ID_ABOUT, "&About..." );
  menuFile->AppendSeparator();
  menuFile->Append( ID_TESTSHOT, "&Shot...\tCtrl-S" );
  menuFile->AppendSeparator();
  menuFile->Append( ID_QUIT, "E&xit" );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, "&File");
  SetMenuBar(menuBar);

  // Create a simple bottom status bar 
  CreateStatusBar(2);

  // Canvas to test: 
  //p_canvas=new BIAS::OpenGLCanvas(this, -1);
  p_canvas=new MyGLCanvas(this);

  //p_2nd_canvas = new MyGLCanvas(this);
}


void MyFrame::OnTestShot(wxCommandEvent& /*event*/)
{
  p_canvas->ScreenShot("out_RGB.jpg", true, GL_RGB);

  p_canvas->ScreenShot("out_RGB.mip", true, GL_RGB);
  p_canvas->ScreenShot("out_RGBA.mip", true, GL_RGBA);
  p_canvas->ScreenShot("out_DEPTH.mip", true, GL_DEPTH_COMPONENT);
  p_canvas->ScreenShot("out_DEPTH.jpg", true, GL_DEPTH_COMPONENT);
  p_canvas->ScreenShot("out_ALPHA.mip", true, GL_ALPHA);
  p_canvas->ScreenShot("out_ALPHA.jpg", true, GL_ALPHA);
  p_canvas->ScreenShot("out_LUMINANCE.mip", true, GL_LUMINANCE);
  p_canvas->ScreenShot("out_STENCIL.mip", true, GL_STENCIL_INDEX);
}


void MyFrame::OnQuit(wxCommandEvent& ){ 
  Close(TRUE); 
}


void MyFrame::OnAbout(wxCommandEvent& )
{
  wxString title("About");
  wxString msg; // message
  msg<<"About - "<<FUNCNAME<<"\n";
  msg<<"author: Jan Woetzel (c) 2003-2005\n";
  msg<<"build: "<<__DATE__<<" "<<__TIME__<<"\n";
  msg<<"from "<<__FILE__<<"\n";
  msg<<"GUI using: "<<wxVERSION_STRING<<"\n";
  msg<<"\n";
  COUT(msg);
  wxMessageBox(msg, title, wxICON_INFORMATION | wxOK );
}

