/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <unistd.h>
#include <pthread.h>
#include <sstream>
#include "GuiGTK.hh"

using namespace BIAS;
using namespace std;

bool GuiGTK::ThreadsInitialized_=false;
bool GuiGTK::MainThreadRunning_=false;
bool GuiGTK::GtkInitialized_=false;
unsigned short GuiGTK::WindowCount_=0;
pthread_t GuiGTK::MainThread_;
pthread_mutex_t GuiGTK::MainThreadRunningMutex_;
pthread_mutex_t GuiGTK::GtkInitializedMutex_;
pthread_mutex_t GuiGTK::WindowCountMutex_;
pthread_mutex_t GuiGTK::BufferMutex_;

/*
#undef BIAS_MUTEX_LOCK
#define BIAS_MUTEX_LOCK(arg)\
      cerr <<__FILE__<<":"<<__LINE__<<" attempting to lock mutex .. "<<#arg;\
      cerr.flush();\
      pthread_mutex_lock(arg);\
      cerr <<" .. got lock\n";

#undef BIAS_MUTEX_UNLOCK
#define BIAS_MUTEX_UNLOCK(arg)\
      cerr <<__FILE__<<":"<<__LINE__<<" unlocking mutex .. "<<#arg;\
      cerr.flush();\
      pthread_mutex_unlock(arg);\
      cerr <<" .. unlocked\n";
*/


GuiGTK::GuiGTK(bool ShowMousePos) : GuiBase(ShowMousePos)
{  
  Init_(ShowMousePos);
}

GuiGTK::GuiGTK(const string& Title,bool ShowMousePos) : GuiBase(ShowMousePos)
{  
  Init_(ShowMousePos);
  Title_=Title;
}

GuiGTK::GuiGTK(const GuiGTK& gui) : GuiBase(gui)
{}

void GuiGTK::Init_(bool ShowMousePos)
{
  GuiBase::Init_(ShowMousePos);
  
  if (!ThreadsInitialized_){
    pthread_mutex_init(&MainThreadRunningMutex_, NULL);
    pthread_mutex_init(&GtkInitializedMutex_, NULL);
    pthread_mutex_init(&WindowCountMutex_, NULL);
    pthread_mutex_init(&BufferMutex_, NULL);
    ThreadsInitialized_=true;
  }
  Buffer_=NULL;
  Window_=NULL;
  DrawingArea_=NULL;
  Label_=NULL;
  Vbox_=NULL;
  //  SetDebugLevel(GetDebugLevel() | D_GUI_GDK);
  //AddDebugLevel(D_GUI_THREAD);
}




GuiGTK::~GuiGTK()
{
    BIASDOUT(D_GUI_DESTRUCT, "GuiGTK destructor started");
    if (WindowCreated_){
      DestroyWindow();
    }
    BIASDOUT(D_GUI_DESTRUCT, "GuiGTK destructor finished");
    // gtk_widget_destroy(Window_);
}

void GuiGTK::InitGtk_()
{
  int argc1=2;
  char **argv1;
  argv1=new char*[3];
  argv1[0]="Window Title";
  argv1[1]="--sync";
  argv1[2]=NULL;
    
  InitGtk(&argc1, &argv1);
  delete[] argv1;
}

void GuiGTK::InitGtk(int *argc, char **argv[])
{
  BIASCDOUT(D_GUI_THREAD, "InitGtk()"<<endl);
  /* init threads */	
  g_thread_init(NULL);
#if GTK_CHECK_VERSION(2,0,0)
  gdk_threads_init();
  //cerr << "using gtk2\n"<<GTK_MAJOR_VERSION<<endl;
#endif
  // ?
  gtk_init(argc, argv);
  BIASCDOUT(D_GUI_GDK, "gdk_rgb_init ... "); BIASFLUSHDOUT;
  gdk_rgb_init();
  BIASCDOUT(D_GUI_GDK, " finished\n");
  BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&GtkInitializedMutex_);
  BIASDOUT(D_GUI_MUTEX," .. got mutex");
  GtkInitialized_=true;
  BIAS_MUTEX_UNLOCK(&GtkInitializedMutex_);
}




void GuiGTK::SetTitle(string const& Title)
{
  BIASCDOUT(D_GUI_THREAD, "SetTitle()"<<endl);
  gdk_threads_enter();
  if (WindowCreated_)
    gtk_window_set_title(GTK_WINDOW(Window_), Title.c_str());
  Title_=Title;
  gdk_threads_leave();
  return;
}


int GuiGTK::AllocateBuffer_(Image<unsigned char>& image)
{
  BIASCDOUT(D_GUI_THREAD, "AllocateBuffer_()"<<endl);
    Width_ = image.GetWidth();
    Height_ = image.GetHeight();
    nChannels_ = image.GetChannelCount();
    BIAS_MUTEX_LOCK(&BufferMutex_);
    Buffer_ = new guchar[Width_ * Height_ * nChannels_];
    BIAS_MUTEX_UNLOCK(&BufferMutex_);
    return 0;
}

int GuiGTK::FreeBuffer_()
{
  BIASCDOUT(D_GUI_THREAD, "FreeBuffer_()"<<endl);
    Width_=0;
    Height_=0;
    nChannels_=0;
    BIAS_MUTEX_LOCK(&BufferMutex_);
    delete[] Buffer_;
    Buffer_=NULL;
    BIAS_MUTEX_UNLOCK(&BufferMutex_);
    return 0;
}



int GuiGTK::CreateWindow_(Image<unsigned char> &image)
{
  BIASCDOUT(D_GUI_THREAD, "CreateWindow_()"<<endl);
  BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&GtkInitializedMutex_);
  BIASDOUT(D_GUI_MUTEX," .. got mutex");
  if (!GtkInitialized_){
    BIAS_MUTEX_UNLOCK(&GtkInitializedMutex_);
    InitGtk_(); }
  else
    BIAS_MUTEX_UNLOCK(&GtkInitializedMutex_);
  AllocateBuffer_(image);

  Window_ = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  Vbox_ = gtk_vbox_new (FALSE, 5);

  DrawingArea_ = gtk_drawing_area_new();

  gtk_drawing_area_size (GTK_DRAWING_AREA (DrawingArea_), 
			 Width_, Height_);

  Label_ = gtk_label_new("no mouse pointer information yet");

  gtk_signal_connect (GTK_OBJECT (Window_), "delete_event",
		      GTK_SIGNAL_FUNC (DeleteEvent_), this);
    
  gtk_signal_connect (GTK_OBJECT (Window_), "destroy",
		      GTK_SIGNAL_FUNC (Destroy_), this);

  gtk_signal_connect(GTK_OBJECT (DrawingArea_), "expose-event",
		     GTK_SIGNAL_FUNC(DrawingAreaExposeCallback_), this);
  
#if !GTK_CHECK_VERSION(0,0,0) 
  gtk_signal_connect(GTK_OBJECT (Label_), "expose-event",
		     GTK_SIGNAL_FUNC(LabelExposeCallback_), this);
#endif

  gtk_signal_connect(GTK_OBJECT(Window_), "key_press_event",
		     GTK_SIGNAL_FUNC(KeyPressEventCallback_), this);

  gtk_signal_connect(GTK_OBJECT(DrawingArea_), "motion_notify_event",
		     GTK_SIGNAL_FUNC(MouseMotionCallback_), this);	

  gtk_signal_connect(GTK_OBJECT(DrawingArea_), "button_press_event",
		     GTK_SIGNAL_FUNC(MouseButtonPressCallback_), this);

  gtk_signal_connect(GTK_OBJECT(DrawingArea_), "button_release_event",
		     GTK_SIGNAL_FUNC(MouseButtonReleaseCallback_), this);

  gtk_widget_add_events(Window_, GDK_KEY_RELEASE_MASK);	

  gtk_widget_set_events (DrawingArea_, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_HINT_MASK
			 | GDK_POINTER_MOTION_MASK);

  gtk_container_add (GTK_CONTAINER (Window_), Vbox_);

  gtk_box_pack_start(GTK_BOX (Vbox_), DrawingArea_, FALSE, FALSE, 0);
  if (ShowMouse_){
    gtk_box_pack_start(GTK_BOX (Vbox_), Label_, TRUE, FALSE, 5);
    //cerr << "adding label\n";
  }

  gtk_widget_show_all (Window_);

  BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&MainThreadRunningMutex_);
  BIASDOUT(D_GUI_MUTEX," .. got mutex");
  if (!MainThreadRunning_){
    MainThreadRunning_=true;
    BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
    pthread_create(&MainThread_ , NULL, GtkMain, this);
  } else
    BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);

  WindowCreated_=true;
  BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&WindowCountMutex_);
  BIASDOUT(D_GUI_MUTEX," .. got mutex");
  WindowCount_++;
  BIAS_MUTEX_UNLOCK(&WindowCountMutex_);

  // cerr << "Create WindowCount_: " << WindowCount_ << endl;

  SetTitle(Title_);

  return 0;
} 


int GuiGTK::DestroyWindow()
{   
  BIASCDOUT(D_GUI_THREAD, "DestroyWindow()"<<endl);
  if (WindowCreated_){
    WindowCreated_=false;
    BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
    BIAS_MUTEX_LOCK(&WindowCountMutex_);
    BIASDOUT(D_GUI_MUTEX," .. got mutex");
    WindowCount_--;
    // cerr << "Destroy WindowCount_: " << WindowCount_ << endl;
    if (WindowCount_ == 0){
      BIASDOUT(D_GUI_SIGNAL, "no more windows, MainThreadRunning_=false");
      //gdk_threads_enter();
      //gtk_main_quit();
      //gdk_threads_leave();
      BIASDOUT(D_GUI_MUTEX,"waiting for mutex..."); 
      BIAS_MUTEX_LOCK(&MainThreadRunningMutex_);
      BIASDOUT(D_GUI_MUTEX," .. got mutex");
      if (MainThreadRunning_){
        MainThreadRunning_=false;
        BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
        int res;
        if ((res=pthread_join(MainThread_, NULL))!=0){
          BIASERR("error joining thread "<<res);
        } 
      } else {
        BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
      }
    }
    BIAS_MUTEX_UNLOCK(&WindowCountMutex_);

    if (DrawingArea_) { gtk_widget_destroy(DrawingArea_); DrawingArea_=NULL; }
    if (Label_) { gtk_widget_destroy(Label_); Label_=NULL; }
    if (Vbox_) { gtk_widget_destroy(Vbox_); Vbox_=NULL; }
    if (Window_) { gtk_widget_destroy(Window_); Window_=NULL; }
    //    gtk_widget_hide_all(Window_);
  }

  if (IsAllocated_())
    FreeBuffer_();

  return 0;
}

int GuiGTK::ShowConvertedImage_(Image<unsigned char> &image)
{
  BIASCDOUT(D_GUI_THREAD, "ShowConvertedImage_()"<<endl);
    if (!WindowCreated_)
      CreateWindow_(image);
    if (!TestImage_(image)){     
      ReInitWindow_(image);
    }
    gdk_threads_enter();
    BIAS_MUTEX_LOCK(&BufferMutex_);
    memcpy(Buffer_, image.GetImageData(), image.GetSize());
    BIAS_MUTEX_UNLOCK(&BufferMutex_);
    Redraw_();
    gdk_threads_leave();
    return 0;
}


int GuiGTK::Redraw_()
{
  BIASCDOUT(D_GUI_THREAD, "Redraw_()"<<endl);
  switch (nChannels_){
	case 1:
    BIAS_MUTEX_LOCK(&BufferMutex_);
    BIASCDOUT(D_GUI_GDK, "gray: "<<Width_<<"x"<<Height_); BIASFLUSHDOUT;
    gdk_draw_gray_image (DrawingArea_->window, 
                         Window_->style->fg_gc[GTK_STATE_NORMAL],
                         0, 0, Width_, Height_,
                         GDK_RGB_DITHER_MAX, Buffer_, Width_ );
    BIASCDOUT(D_GUI_GDK, "   finished\n");
    BIAS_MUTEX_UNLOCK(&BufferMutex_);
    break;
	case 3:
    BIAS_MUTEX_LOCK(&BufferMutex_);
    BIASCDOUT(D_GUI_GDK, "rgb: "<<Width_<<"x"<<Height_); BIASFLUSHDOUT;
    gdk_draw_rgb_image (DrawingArea_->window, 
                        Window_->style->fg_gc[GTK_STATE_NORMAL],
                        0, 0, Width_, Height_,
                        GDK_RGB_DITHER_MAX, Buffer_, Width_*3);
    BIASCDOUT(D_GUI_GDK, "   finished\n");
    BIAS_MUTEX_UNLOCK(&BufferMutex_);
    break;
	default:
    BIASERR(nChannels_ 
            << " channels not supported. Use grey or RGB image");
    //exit (-1);
    break;
	}
  return 0;
}


int GuiGTK::ReInitWindow_(Image<unsigned char>& image)
{
  BIASCDOUT(D_GUI_THREAD, "ReInitWindow_()"<<endl);
  int res = 0;
  // cout << "reinitializing\n";
  if (IsAllocated_())
    FreeBuffer_();
  res = AllocateBuffer_(image);
  gtk_drawing_area_size (GTK_DRAWING_AREA (DrawingArea_), 
			 Width_, Height_);
  // gdk_window_resize(Window_, Width_, Height_);
  //  gtk_widget_show_all (Window_);
  return res;
}


void* GuiGTK::GtkMain(void* arg)
{  
  //cout << "thread id main: " << pthread_self() << endl;
#ifdef BIAS_DEBUG
  GuiGTK *obj=(GuiGTK *)arg;
#endif
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "GtkMain()");
  bool Running_=true;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
                  "waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&MainThreadRunningMutex_);
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
		 " .. got mutex");
  MainThreadRunning_=true;
  BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
    
//      gdk_threads_enter();
//      gtk_main();
//      gdk_threads_leave();
  while (Running_){
    gdk_threads_enter();
    gtk_main_iteration_do(FALSE);
    gdk_threads_leave();
    usleep(1000);
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MAIN, 
		   "GtkMain still lives");
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
		   "waiting for mutex..."); 
    BIAS_MUTEX_LOCK(&MainThreadRunningMutex_);
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
		   " .. got mutex");
    Running_=MainThreadRunning_;
    BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
  }
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream,D_GUI_MAIN, 
		 "gtk_main finished");
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
		 "waiting for mutex..."); 
  BIAS_MUTEX_LOCK(&MainThreadRunningMutex_);
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MUTEX,
		 " .. got mutex");
  MainThreadRunning_=false;
  BIAS_MUTEX_UNLOCK(&MainThreadRunningMutex_);
  return (void *)NULL;
}

gint GuiGTK::KeyPressEventCallback_(GtkWidget *Widget, 
				      GdkEventKey *Event, 
				      gpointer user_data)
{

  GuiGTK *obj=(GuiGTK *)user_data;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "KeyPressEventCallback_()"<<endl);
  obj->KeyBuffer_=Event->keyval;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream,D_GUI_KEY, 
                  "key  pressed: "<<obj->KeyBuffer_<<endl);
  return TRUE;
}

gint GuiGTK::MouseButtonPressCallback_(GtkWidget *Widget, 
				       GdkEventButton *Event,
				       gpointer user_data)

{
  GuiGTK *obj=(GuiGTK *)user_data;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "MouseButtonPressCallback_()");
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MOUSE_BUTTON,
		 "mouse button "<<(int)Event->button 
		 << " pressed at (" << (int)Event->x 
		 << ", "<<(int)Event->y <<")");
  pthread_mutex_lock(&MouseButtonMutex_);
  obj->MouseButtonPressed_ = (int)Event->button;
  obj->MouseButtonPressX_ = (int)Event->x;
  obj->MouseButtonPressY_ = (int)Event->y;
  pthread_mutex_unlock(&MouseButtonMutex_);
  return TRUE;
}

gint GuiGTK::MouseButtonReleaseCallback_(GtkWidget *Widget, 
					 GdkEventButton *Event,
					 gpointer user_data)
{
  GuiGTK *obj=(GuiGTK *)user_data;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "MouseButtonReleaseCallback_()");
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_MOUSE_BUTTON, 
		 "mouse button "<<(int)Event->button 
		 << " released at (" << (int)Event->x 
		 << ", "<<(int)Event->y <<")");
  pthread_mutex_lock(&MouseButtonMutex_);
  obj->MouseButtonReleased_ = (int)Event->button;
  obj->MouseButtonReleaseX_ = (int)Event->x;
  obj->MouseButtonReleaseY_ = (int)Event->y;
  pthread_mutex_unlock(&MouseButtonMutex_);
  return TRUE;
}

gint GuiGTK::MouseMotionCallback_(GtkWidget *Widget,
                                  GdkEventMotion* Event,
                                  gpointer user_data)
{
  ostringstream label;
  GdkModifierType state;
  GuiGTK *obj=(GuiGTK *)user_data;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "MouseMotionCallback_()");

  //cout << "GuiGTK::MouseMotionCallback_"<<endl;
  if (Event->is_hint){
    BIASDOUT_THREAD(obj->GetDebugLevel(), obj->GetDebugStream(), 
                    D_GUI_GDK, "gdk_window_get_pointer ...");
    gdk_window_get_pointer (Event->window, &(obj->MousePosX_), 
                            &(obj->MousePosY_), &state);
    BIASDOUT_THREAD(obj->GetDebugLevel(), obj->GetDebugStream(), 
                    D_GUI_GDK, " finished");
  } else {
    obj->MousePosX_ = (int)Event->x;
    obj->MousePosY_ = (int)Event->y;
  }
  
  //cout << "IsPyramid_: "<<obj->IsPyramid_<<"\tPyramid0Width_: "<<obj->Pyramid0Width_<<"\tPyramidLevels_: "<<obj->PyramidLevels_<<endl;
  if (obj->IsPyramid_>0 &&  obj->MousePosX_> (int)obj->Pyramid0Width_){
    obj->MousePosX_ = obj->MousePosX_ - obj->Pyramid0Width_ - PYRAMID_BORDER;
    unsigned i=1;
    while ((obj->MousePosY_ > int(obj->Pyramid0Height_>>i)+PYRAMID_BORDER)&&
           (i<obj->PyramidLevels_)){
      obj->MousePosY_ -= (obj->Pyramid0Height_>>i)+PYRAMID_BORDER;
      i++;
    }
  } 

  if ((obj->MousePosX_ < (int)obj->Width_) && 
      (obj->MousePosY_ < (int)obj->Height_) &&
      (obj->MousePosX_ >= 0) && (obj->MousePosY_ >= 0) &&
      (obj->Factor_ >= 1)){
    label << " (x, y) = ("<< obj->MousePosX_/obj->Factor_+obj->XOffset_ 
          <<", " << obj->MousePosY_/obj->Factor_+obj->YOffset_
          <<")          (R G B) = (";
    if (obj->OrigPicture_.GetChannelCount() == 1) {
      BIAS_MUTEX_LOCK(&BufferMutex_);
      switch (obj->OrigPicture_.GetStorageType()) {
      case ImageBase::ST_unsignedchar:
        label << (int)((unsigned char*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->Width_+obj->MousePosX_]<< " ";
        break;
      case ImageBase::ST_float:
        label << ((float*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->Width_+obj->MousePosX_]<< " ";
        break;
      case ImageBase::ST_unsignedshortint:
        label << (int)((unsigned short int*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->Width_+obj->MousePosX_]<< " ";
        break;
      case ImageBase::ST_int:
        label << (int)((int*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->Width_+obj->MousePosX_]<< " ";
        break;
      default:
        BIASERR("storage type not yet implemented");
        break;
      }
      BIAS_MUTEX_UNLOCK(&BufferMutex_);
    } else {
      
      BIAS_MUTEX_LOCK(&BufferMutex_);
      for (unsigned int i=0; i<obj->OrigPicture_.GetChannelCount(); i++){
        switch (obj->OrigPicture_.GetStorageType()) {
        case ImageBase::ST_unsignedchar:
          label << (int)((unsigned char*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->OrigPicture_.GetWidth()*obj->OrigPicture_.GetChannelCount()+obj->MousePosX_*obj->OrigPicture_.GetChannelCount()+i]<< " ";
          break;
        case ImageBase::ST_float:
          label << ((float*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->OrigPicture_.GetWidth()*obj->OrigPicture_.GetChannelCount()+obj->MousePosX_*obj->OrigPicture_.GetChannelCount()+i]<< " ";
          break;
        case ImageBase::ST_int:
          label << (int)((int*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->OrigPicture_.GetWidth()*obj->OrigPicture_.GetChannelCount()+obj->MousePosX_*obj->OrigPicture_.GetChannelCount()+i]<< " ";
          break;
        case ImageBase::ST_char:
          label << (int)((signed char*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->OrigPicture_.GetWidth()*obj->OrigPicture_.GetChannelCount()+obj->MousePosX_*obj->OrigPicture_.GetChannelCount()+i]<< " ";
          break;
        case ImageBase::ST_unsignedshortint:
          label << (unsigned short)((unsigned short*)(obj->OrigPicture_.GetImageData()))[obj->MousePosY_*obj->OrigPicture_.GetWidth()*obj->OrigPicture_.GetChannelCount()+obj->MousePosX_*obj->OrigPicture_.GetChannelCount()+i]<< " ";
          break;default:
          BIASERR("storage type not yet implemented");
          break;
        }
      
//         label << (int)obj->Buffer_[obj->MousePosY_*obj->Width_*
//                                    obj->nChannels_
//                                    +obj->MousePosX_*obj->nChannels_+i] <<" ";
      }
      BIAS_MUTEX_UNLOCK(&BufferMutex_);
    }
    label << ")";
      
    if (obj->ShowMouse_){
      //cerr << "now setting label: "<<label.str() << endl;
      gtk_label_set_text(GTK_LABEL(obj->Label_), (gchar *)label.str().c_str());
      //       gtk_signal_emit(GTK_OBJECT(obj->Label_), 
      // 		      gtk_signal_lookup("expose-event", 
      // 					GTK_TYPE_LABEL));
      //const gchar *l = gtk_label_get_text(GTK_LABEL(obj->Label_));
      //cout << "label has text: "<<l<<endl;
    } 
  }
  return TRUE;	
}

gboolean GuiGTK::DrawingAreaExposeCallback_(GtkWidget *widget, 
                                         GdkEventExpose *Event, 
                                         gpointer user_data)
{
    GuiGTK *obj=(GuiGTK *)user_data;
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_SIGNAL, 
		"DrawingAreaExposeCallback_()");
    obj->Redraw_();
    return TRUE;
}

gboolean GuiGTK::LabelExposeCallback_(GtkWidget *widget, 
				   GdkEventExpose *Event, 
				   gpointer user_data)
{
#ifdef BIAS_DEBUG
    GuiGTK *obj=(GuiGTK *)user_data;
#endif
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "LabelExposeCallback_()");
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_SIGNAL, 
		   "LabelExposeCallback_");
    // cerr << "LabelExposeCallback_" << endl;
    return TRUE;
}

gint GuiGTK::DeleteEvent_( GtkWidget *widget, GdkEvent  *event,
				      gpointer user_data )
{
     /* Change TRUE to FALSE and the main window will be destroyed with
        a "delete_event". */
#ifdef BIAS_DEBUG
    GuiGTK *obj=(GuiGTK *)user_data;
#endif
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, 
                  "DeleteEvent_()");
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream,
                    D_GUI_SIGNAL, "DeleteEvent_");
    return(FALSE);
}



void GuiGTK::Destroy_( GtkWidget *widget, gpointer user_data)
{
    GuiGTK *obj=(GuiGTK *)user_data;
  BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, D_GUI_THREAD, "Destroy_()");
    BIASDOUT_THREAD(obj->_liDebugLevel, obj->_zDebugStream, 
                    D_GUI_SIGNAL, "Destroy_");
    obj->DestroyWindow();
}













