/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** For our wxWindows-GUI we need to define some more classes than GuiWX.
    The other three classes WxMainThread, ImgCanvas and TopFrame are needed
    for creating windows (and its content) in wxWindows and to split off
    the wxWindows-main-loop, which is run in a seperate thread to ensure
    that we have only one main-loop for each instance of GuiWX.
*/

#ifndef __GuiWX_hh__
#define __GuiWX_hh__

#error This class (BIAS::GuiWX) is deprecated. It will be removed in a few days

#include "bias_config.h" 

#include <fstream>
#include <string>

#include <wx/wx.h>
#include <wx/thread.h>

#include "GuiBase.hh"
#include "ImageCanvas.hh"
//BIAS libraries
#include <Base/Math/Math.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

namespace BIAS{


// forward declaration:
class TopFrame;
class GuiWX;

/** @class WxMainThread
    To use threads in wxWindows we must create our own class, which is 
    derived from wxThread. 
    In this case this is our main thread class in which the main loop 
    will be run
*/
class WxMainThread : public wxThread
{
public:
  WxMainThread();
  ~WxMainThread();

  WxMainThread(GuiWX *gui);

  virtual void *Entry();
  virtual void OnExit();
protected:
  GuiWX *MyGui_;
};



/** This is the fundamental frame in which the canvas (displaying the 
    image) is positioned.
    We also define the status bar here. 
*/
class TopFrame : public wxFrame
{
public:
  TopFrame( GuiWX *gui, Image<unsigned char> &image );
  ~TopFrame();

  // event handler:
  void OnChar( wxKeyEvent& event );
  void OnClose( wxCloseEvent& event );

  ImageCanvas *Canvas_;

protected:
  DECLARE_EVENT_TABLE()
  GuiWX  *MyGui_;
};



/** @class GuiWX
    @ingroup g_gui
    @brief a sophisticated more complex windowing environment

    Features:
    - GLcanvas
    - Menu support

    Usage:
    - look at the example or ask Jan Friso Evers or Jan Woetzel 
*/
class BIASGui_EXPORT GuiWX : public GuiBase
{
  friend class TopFrame;
  friend class ImgCanvas;  
  friend class WxMainThread;
public:
  GuiWX(bool ShowMousePos = true);
  /// as standard constructor, also sets Title_ 
  GuiWX(std::string const& Title, bool ShowMousePos);
  GuiWX(std::string const& Title);
  
  virtual ~GuiWX();
  
  /// self explaining
  void SetTitle(std::string const& Title);

 
  /// stops the wx-main loop and closes the window
  int DestroyWindow();


protected:
  unsigned char *Buffer_;

  // Booleans:
  static bool ThreadInitialized_;
  static bool MainThreadRunning_;
  static bool WxInitialized;
  
  // Counter for open windows
  static unsigned short int WindowCounter_;

  static wxApp *AppApp_;

  // Locks
  static wxMutex GuiWxMutex_;

  // WX-Widgets
  TopFrame *Frame_;
   
  // Main-Thread
  wxThread *WxMainLoop_;
  
  wxImage *TmpWxImg_;
  /// Init is called from the constructor
  void Init_(bool ShowMousePos);
  
  int CreateWindow_(Image<unsigned char> &image);
  
  int ShowConvertedImage_(Image<unsigned char> &image);
};

} // namepsace BIAS
#endif // __GuiWX_hh__
