#ifndef __ImgObjGL_h__
#define __ImgObjGL_h__

#include <bias_config.h>
#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use ImgObjGL
#endif // BIAS_HAVE_OPENGL

#include "glstuff.h"

// BIAS
#include <Base/Image/ImageBase.hh>
#include <Base/Image/Image.hh>


/** data holder describing OpenGL texture objects in addition to its unique texID.
@author Jan Woetzel  */
namespace BIAS {
  class ImgObjGL {
  public: 
    ~ImgObjGL();

    ImgObjGL();

    //* ctor initializing members JW */
    ImgObjGL(
      const GLuint & id,
      const unsigned int & widthBrutto,
      const unsigned int & heightBrutto,
      const GLenum & target=GL_TEXTURE_2D
      );

    //* set memebrs to defaults */
    void InitMembers();

    /// frees the pointer members if we allocated them
    void Release();

    /// deletes the texture memory and frees texture in GL 
    void  DeleteGLTexture();

    /// @return true ibn RECT instead of 2D pow2 object
    bool IsRectangleTexture() const;

    /**  creates an OpenGL texture from p_valueImage JW
    if ImmediateCopyData is false teh memory is just allocated and not copied from c_img.
    JW */
    void CreateGLTexture(
      const BIAS::ImageBase & c_img, /*zeropad etc is currently missing*/
      const bool & forcePow2tex=true,
      const bool & flipY=true,
      const bool ImmediateCopyData=true );


    /** set the textur retarget to rectangular or pow 2 texture JW */
    void SetTargetPOW2(){ SetTarget2D(); };
    void SetTarget2D(){ target=GL_TEXTURE_2D; };
    void SetTargetRECT(){ target=GL_TEXTURE_RECTANGLE_NV; };

    /** get the GLenum pixel storage type for the corresponding BIAS enum @author Jan Woetzel */
    static GLenum GetGLStorageType(const BIAS::ImageBase::EStorageType & biasST,
      const unsigned int & ByteDepth );

    /** get the GLenum pixel src format for the corresponding BIAS enum ColroModel @author Jan Woetzel */
    static GLenum GetGLPixelFormat(const BIAS::ImageBase::EColorModel & biasCM,
      const unsigned int & channelcount );

    /// JW determin teh GPU internal format teh src data should be mapped to @author Jan Woetzel 
    static GLint GetGLInternalFormat(
      const BIAS::ImageBase::EStorageType & biasST,
      const BIAS::ImageBase::EColorModel & biasCM,
      const unsigned int & channelcount,
      const unsigned int & ByteDepth,
      const unsigned int & BitDepth );

    ///  interface for teh above @author Jan Woetzel 
    static GLint GetGLInternalFormat(const BIAS::ImageBase & img);



  public:
    // OpenGL texture 
    GLuint id; // unique OpengL texture id
    GLenum target; // GL_TEXTURE_2D or GL_TEXTURE_RECTANGLE_NV

    // true if we "created" the above and need to free it in destructor. 
    bool p_id_SelfAllocated;

    GLenum minificationMode;
    GLenum magnificationMode;

    // original feed in size
    unsigned int widthOrig; // should match p_valueImage size if exists
    unsigned int heightOrig;

    // padded to next pow2 size:
    unsigned int widthBrutto;
    unsigned int heightBrutto;

    // image containing the original values
    BIAS::ImageBase * p_valueImage;

    // true if we "newed" the above and have to free it in destructor. 
    // false if we got a pointer which we do not need to clean up.
    bool p_valueImage_SelfAllocated;

  };
} // namespace BIAS

#endif

