
#include <Base/Common/BIASpragma.hh>

#include "OpenGLCanvas.hh"

#include <Base/Image/ImageIO.hh>
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector4.hh>

#include "GeometryGL.hh"

// -----------------------------------------
using namespace std;
using namespace BIAS;
// -----------------------------------------

// override parent classes event handling table behavior
BEGIN_EVENT_TABLE (OpenGLCanvas, OpenGLCanvasBase)
// --- parent EVT handlers ---
//EVT_PAINT     (OpenGLCanvasBase::OnPaint)
//EVT_SIZE      (OpenGLCanvasBase::OnSize)
//EVT_ERASE_BACKGROUND (OpenGLCanvasBase::OnEraseBackground)
// --- additional handlers ---
EVT_MOTION(OpenGLCanvas::OnMotion)
#if OPENGLCANVAS_CONNECT_ONIDLE
EVT_IDLE   (OpenGLCanvas::OnIdle)
#endif

//EVT_LEFT_DCLICK(OpenGLCanvas::OnLeftDoubleClick)
//EVT_RIGHT_DCLICK(OpenGLCanvas::OnRightDoubleClick)
//EVT_MIDDLE_DCLICK(OpenGLCanvas::OnMiddleDoubleClick)
//EVT_MOUSEWHEEL(OpenGLCanvas::OnMouseWheel)
END_EVENT_TABLE();

// -----------------------------------------
bool		 OpenGLCanvas::idleRepaintEnabled = OPENGLCANVAS_DEFAULT_idleRepaintEnabled;
bool	   OpenGLCanvas::realtimeAnimation = OPENGLCANVAS_DEFAULT_realtimeAnimation;
// -----------------------------------------

/// destructor
OpenGLCanvas::~OpenGLCanvas(){};


//// constructors: 
OpenGLCanvas::OpenGLCanvas(
                           wxFrame* parent
                           ,wxWindowID id
                           ,const wxPoint& pos
                           ,const wxSize& size
                           ,long style
                           ,const wxString& name
                           ,int* attribList 
                           ,const wxPalette& palette
                           )                           
                           : OpenGLCanvasBase(parent,id,pos,size,style,name,attribList,palette)
{
  //CALLINFO;
  InitMembers();
}


OpenGLCanvas::OpenGLCanvas(
                           wxFrame* parent
                           ,wxGLContext* sharedContext
                           ,wxWindowID id
                           ,const wxPoint& pos
                           ,const wxSize& size
                           ,long style
                           ,const wxString& name
                           ,int* attribList 
                           ,const wxPalette& palette 
                           )
                           : OpenGLCanvasBase(parent,sharedContext,id,pos,size,style,name,attribList,palette)
{
  //CALLINFO;
  InitMembers();
}


void OpenGLCanvas::InitMembers()
{
  //CALLINFO;

  OpenGLCanvasBase::InitMembers();
  // additional members: 
  _polygonmodeGL=GL_FILL;
  //_pos=wxPoint(0,0);
}


void OpenGLCanvas::DisplayCameraView()
{
  DisplayCameraViewInternalParams();
  DisplayCameraViewExternalParams();
}


void OpenGLCanvas::DisplayCameraViewInternalParams()
{
  viewCtrl_.DisplayGLInternal();
}

void OpenGLCanvas::DisplayCameraViewExternalParams(){
  viewCtrl_.DisplayGLExternal();
}




void OpenGLCanvas::OnMotion( wxMouseEvent &event )
{
#ifdef WIN32
  wxDC dc;
#else
  // HACK
  wxPaintDC dc(this); 
#endif

  // mouse position
  wxPoint pos= event.GetPosition();
  ShowValue(pos);	
}





// JW onidle redisplay like in GLUT
// useful for animations
void OpenGLCanvas::OnIdle(wxIdleEvent& event)	
{
  //CALLINFO;
  // TODO set global OpenGLCanvasBase::time based on system time?
  if (!(this->idleRepaintEnabled)) {
    return; // nothing to do, in particular do not request more OnIdle events.
  };

  // for animation
  UpdateTime();

  // paint again, timestamp may have changed, useful for animations: 
  Display();
  ShowFPS();

  // usually only one Idle event is generated on when we become idle
  // but we want to run this continuously *while* we stay idle.
  event.RequestMore();
}


void OpenGLCanvas::UpdateTime()
{
  if (animationUpdateEnabled){
    // next time step: increase a bit
    if (realtimeAnimation){
      BIAS::OpenGLCanvasBase::UpdateTime();
    } else {
      // fallback that depends on frame no. 
      // should be a fixed update
      // assume scaling for TARGET_FPS fps display...
      time +=1.0/TARGET_FPS; 
    }
    //cout<<"OpenGLCanvasBase::time="<<OpenGLCanvasBase::time <<endl; // DBG  
  }
}

/// return GL state info
void OpenGLCanvas::GetInfoGL(std::ostream & os)
{
  OpenGLCanvasBase::GetInfoGL(os);

  // display (possibly used) CamerViewcontroller member status:
  this->viewCtrl_.Print(os);
}
