#ifndef __OpenGLCanvas_hh__
#define __OpenGLCanvas_hh__

/** \author Jan Woetzel 01/2003 - 12/2004 */
#include <bias_config.h>
#ifndef BIAS_HAVE_WXWINDOWS
#  error Please recompile BIAS with USE_WXWINDOWS to use this lib. 
#endif // BIAS_HAVE_WXWINDOWS

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use this lib. 
#endif // BIAS_HAVE_OPENGL

#ifndef BIAS_HAVE_GLUT
#  error Please recompile BIAS with USE_GLUT (gl utility toolkit) to use this lib. 
#endif // BIAS_HAVE_GLUT



// must be defined to connect the OnIdle handler at all:
#ifndef OPENGLCANVAS_CONNECT_ONIDLE
#  define OPENGLCANVAS_CONNECT_ONIDLE 1
#endif //OPENGLCANVAS_CONNECT_ONIDLE

// behave like glut with continuous rendering using OnIdle function
// useful only for animations, otherwise CPU time is consumed although the same
// view is rendered again and again. (JW)
// true: repaint in OnIdle event handler
// false: do not repaint bad for animations, but good for CPU if there is no aniamtion. 
#ifndef OPENGLCANVAS_DEFAULT_idleRepaintEnabled
#  define OPENGLCANVAS_DEFAULT_idleRepaintEnabled true
#endif 

// true:  use realtime wall clock, good for animation taht should not depend on GPU performance/resolution
// false: use fixed incremental counter on each rendering, good for feedback of visual performance of animations.
#ifndef  OPENGLCANVAS_DEFAULT_realtimeAnimation
#  define OPENGLCANVAS_DEFAULT_realtimeAnimation true;
#endif

// HACK: expected fps, only for similar speed in realtime and free running mode
//#define TARGET_FPS 500 
#define TARGET_FPS 200 


// TODO: integrate trackball/virtual camera clas, e.g. based on GLhelper interactor. (JW)

// BIAS
#include <Gui/OpenGLCanvasBase.hh>
#include <Gui/CameraViewController.hh>


namespace BIAS {
	/** 
	@class OpenGLCanvas
	@ingroup g_gui
	@brief OpenGL canvases with additinal event handling (JW)

	In addition to OpenGLCanvasbase "automatioc" event handling is integrated,
	e.g. 
	-Display value under mouse in status bar,
	-internal storage of camera view,
	-navigation through mouse and keys
	And it is optionally adding
	-automatic repaint by Onidle function (like in glut)
	-automatic update of time for animations.
	\author Jan Woetzel */
	class BIASGui_EXPORT OpenGLCanvas : public OpenGLCanvasBase 
	{	
	public:
		virtual ~OpenGLCanvas(); ///< destructor

		// ctors
		OpenGLCanvas(
			wxFrame* parent
			,wxWindowID id = -1
			,const wxPoint& pos = wxDefaultPosition
			,const wxSize& size = wxDefaultSize
			,long style = 0
			,const wxString& name = "OpenGLCanvas"
			,int* attribList = NULL
			,const wxPalette& palette = wxNullPalette
			);

		/// 
		/// constructor using a shared context 
		OpenGLCanvas(
			wxFrame* parent
			,wxGLContext* sharedContext = NULL
			,wxWindowID id = -1
			,const wxPoint& pos = wxDefaultPosition
			,const wxSize& size = wxDefaultSize
			,long style=0
			,const wxString& name="OpenGLCanvas"
			,int* attribList = 0
			,const wxPalette& palette = wxNullPalette
			);

		virtual void InitMembers();

    virtual void GetInfoGL(std::ostream & os=std::cout);

		/// display call to be overridden by child classes
		//virtual void Display();

		virtual void DisplayCameraView();
		virtual void DisplayCameraViewInternalParams();
		virtual void DisplayCameraViewExternalParams();

    /// sets the internal time param, by default based on realtime wall clock
    virtual void UpdateTime();



		void OnMotion( wxMouseEvent &event );
		void OnIdle(wxIdleEvent& event);

		//void OnLeftDoubleClick( wxMouseEvent &event );
		//void OnRightDoubleClick( wxMouseEvent &event );
		//void OnMiddleDoubleClick( wxMouseEvent &event );
		//void OnMouseWheel( wxMouseEvent &event );

	public:
		// true to repaint in OnIdle
		static bool idleRepaintEnabled;
		// auto update time?
		static bool realtimeAnimation;

	public:
		// for (interactive) view control, holds parameters of virtual camera 
		CameraViewController viewCtrl_;

	protected: 
		//wxPoint _pos; // mouse position
		GLenum _polygonmodeGL; // GL_POINT GL_LINE GL_FILL 


		// override parent classes event handling table: 
		DECLARE_EVENT_TABLE();

	private:
	}; // OpenGLCanvas

} // namespace BIAS
#endif
