#ifndef __RenderModeGL_h__
#define __RenderModeGL_h__

#include <bias_config.h>
#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use ImgObjGL
#endif // BIAS_HAVE_OPENGL

#include "glstuff.h"

#include <Base/Math/Vector4.hh>

/** 
@ingroup g_gui
@brief GLRenderMode describes mode for rendering.

In particular 
#culling, 
#pointsize, 
#linesize, 
#buffer mode (wireframe, points..) 
#stereo mod e(e.g. page flipping) for display.
# etc...
@author Jan Woetzel  */
namespace BIAS {

	enum  DrawbufferMode {
		MONO=0 /* one display but possibly double buffered */
		,STEREO_UNTOUCHED /* do not touch selcting the DrawBuffer, user is doing it himself */
		,STEREO_CONSUMER_NV /* consumer 3D stereo using NV driver. works only in fullscreen */
		,STEREO_QUADBUFFERED /* page flipping quad buffered stereo, e.g. with consumer shutter glasses like Elsa Revelator */
		,STEREO_LEFT_RIGHT /* stereo without sdouble buffering */
		,STEREO_TWOSCREENS /* two monitors, e.g. HMD or polarized glasses */
		//,STEREO_ANAGLYPH_RG /* anaglyph using left:red right: green (with one screen) */
		//,STEREO_ANAGLYPH_GR /* anaglyph using green-red */
		//,STEREO_ANAGLYPH_RB /* anaglyph using red-blue */
		//,STEREO_ANAGLYPH_BR /* anaglyph using blue-red*/
	};

	class RenderModeGL {
	public: 
		RenderModeGL()
		{
			polygonmodeFRONT=GL_FILL;
			polygonmodeBACK=GL_FILL;
			shademodel=GL_SMOOTH;

			enable_cullface=false;
			cullface=GL_BACK;

			pointsize=1.0f;
			linewidth=1.0f;

			// handled extra by Display function 
			// because it has to match SwapBuffers
			drawbuffermode=BIAS::STEREO_UNTOUCHED;
			//drawbuffermode=BIAS::STEREO_CONSUMER_NV;
			vsync=false;
		};

		/** no 3D streo enabled, single viewpoint only? */
		bool IsMono() const {return (drawbuffermode==BIAS::MONO);};

		/** page flipping quad buffered stereo?
		E.g. with consumer shutter glasses like Elsa Revelator  */
		bool IsStereoQuadbuffered() const {return (drawbuffermode==BIAS::STEREO_QUADBUFFERED);};

		// members:

		// GL_FILL, GL_FLAT, GL_POINT
		GLenum polygonmodeFRONT; ///< glPoygonMode(GL_FRONT...
		GLenum polygonmodeBACK;  ///< glPoygonMode(GL_BACK...

		// GL_SMOOTH, GL_FLAT
		GLenum shademodel; ///< glShadeModel(...

		bool enable_cullface;
		GLenum cullface; ///< glCullFace: GL_FRONT, GL_BACK, and GL_FRONT_AND_BACK 

		float pointsize;
		float linewidth;

		// 3D display using shutter glasses or multiple monitors with polarizers?
		BIAS::DrawbufferMode drawbuffermode; // enable page swap stereo, e.g. with shutetr glasses

		bool vsync; // true to enable vertical sync
	};
} // namespace BIAS

#endif

