#ifndef _glstuff_h_
#define _glstuff_h_

/** 
wrapper to include all gl, glu, glut, glew, cg etc. needed
- keep choice of using glew/gl/glut/...  flexible in client apps
  while asserting glew is included first!
- disable GL warnings in a central place.
- allows precompiled headers
- Please note that the app. ha sto call glewinit liek this:
\verbatim
  if (!(GLEW_VERSION_1_1)) {
    // init and try again:
    GLenum result = glewInit();
    if (result != GLEW_OK){
      BIASERR("glewInit failed.");
    };
  }
\endverbatim
author: Jan Woetzel
*/

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use OpenGL.
#endif // BIAS_HAVE_OPENGL 

#ifndef BIAS_HAVE_GLEW
#  error Please recompile BIAS with USE_GLEW to use GL extension handler GLEW. 
#endif // BIAS_HAVE_GLEW

/// USE glew:
// must define kind of lib/src for glew extension handler lib
// (linker errors otherwise) !!!
#ifdef WIN32
#  include <windows.h> // required *before* glew due to LEAN AND MEAN
#endif

#define GLEW_STATIC
#  include <GL/glew.h> // includes gl.h and glu.h
#ifdef WIN32
#  include <GL/wglew.h>
#else
#  include <GL/glxew.h>
#  define __glx_h__
#endif

#if defined(BIAS_HAVE_WXWINDOWS) || defined(MIP_HAVE_WXWINDOWS)
// order is mandatory: include glew first to avoid redefinitions !!! (JW)
// there *must not* be a gl.h inclusion before glew !!!
#  include <wx/glcanvas.h> 
//#else
//#  error Please recompile BIAS with USE_WXWINDOWS to use wxGL. 
#endif


#ifndef BIAS_HAVE_GLUT
//#  error Please recompile BIAS with USE_GLUT to use GLUT. 
#else
// avoid some entry calls /4505 warnings with this define on WIN32: 
#  define GLUT_BUILDING_LIB
#  include <GL/glut.h> 
#endif // BIAS_HAVE_GLUT


// --- define special token extension values ---
//#define GL_CRYCBY_422_NVX   0x600A
//#define GL_YCRYCB_422_NVX   0x600B 

// BIAS ERR stream: 
#include <Base/Debug/Error.hh>

// STD for output of macros:   
#include <string>
#include <sstream>
#include <iostream>


// ---------------------------------------------------
///use this macro in your code to check for OpenGL errors with "click+jump" 
///@author Jan Woetzel
#if defined(BIAS_DEBUG) && defined(WIN32)

#define CHECK_GL_ERROR \
{\
  GLenum lastErr = glGetError(); \
  if (lastErr!=GL_NO_ERROR) { \
    std::stringstream ss; \
    ss<<std::endl<<__FILE__<<"("<<__LINE__<<") : "<<FUNCNAME<<std::endl \
      <<"\tOpenGL Error: "<<(char *)gluErrorString(lastErr)<<std::endl; \
    ERROR_STREAM<<ss.str(); \
    OutputDebugString(ss.str().c_str()); \
  } \
} 

#elif defined(BIAS_DEBUG) 

#define CHECK_GL_ERROR \
{\
  GLenum lastErr = glGetError(); \
  if (lastErr!=GL_NO_ERROR) { \
    std::stringstream ss; \
    ss<<std::endl<<__FILE__<<":"<<__LINE__<<" : "<<FUNCNAME<<std::endl \
      <<"\tOpenGL Error: "<<(char *)gluErrorString(lastErr)<<std::endl; \
    ERROR_STREAM<<ss.str(); \
  } \
} 
#else
// release: no output 
#  define CHECK_GL_ERROR 
#endif




// ----------------------------------------------------
// DEPRECATED JW
//namespace BIAS {
//  /// wrapper for glGenTextures to generate a fresh texture object (id)
//  //static GLuint BIASGui_EXPORT 
//  //static inline 
//  GLuint GenTextureID();
//
//} // end namespace BIAS -------------------------------


#endif // _glstuff_h_ 
// DO NOT ADD CODE AFTER THIS LINE

