
#include <bias_config.h>

#ifndef BIAS_HAVE_XML2
#  error You need XML2 to compile this code. Please enable USE_XML2 and recompile BIAS. 
#endif // BIAS_HAVE_XML2


#include <Image/CameraParam.hh>
#include <Image/UndistortionMapping.hh>
#include <Image/MatlabTBUndistortion.hh>
#include <Image/SphericalUndistortion.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <iostream>

using namespace BIAS;
using namespace std;



int main(int argc, char** argv )
{
  MatlabTBUndistortion mu;
  SphericalUndistortion su;
  UndistortionMapping<unsigned char, unsigned char> ums;

  Image<unsigned char>* src;
  Image<unsigned char> dst;
  src = new Image<unsigned char> [argc-1];

  // double xDst, yDst;
  string camfileSphere = "PtScorpion_1600x1200C_IBAK.xml";
  string camfilePersp = "PtGrey_Dragonfly_640x480C_4mm.xml";

  mu.Init(camfilePersp);
  su.Init(camfileSphere);

  if (su.IsInitialized() && su.IsSpherical()) {
    ums.SetCameraParam(su);
    cout << camfileSphere << " used for initialization." << endl;
  } else {
    cout << "Error: could not read camera data file or no perspective lens!" 
      << endl;
    abort();
  }
  
  if (argc>1) {
    
    int i, j;
    for (i=1; i<argc; i++) {
      if (ImageIO::Load(argv[i], src[i-1])!=0){
        BIASERR("error loading image "<<argv[i]);
        return -1;
      } else {
        ImageConvert::ToGrey(src[i-1], src[i-1]);
      }
    }

    double phi, theta;
    double cx, cy;

    dst.Init(su.GetWidth(),su.GetHeight());
    int ct = argc-1;
    for (i=0; i<ct/3+1; i++) {
      for (j=((ct-i*3)<3 ? ct-i*3 : 3)-1 ; j>=0; j--) {
        phi = i*2*M_PI/(ct/3+1)-M_PI;
        theta = (j*M_PI/2)/3;
        cout << "drawing image phi:" << phi << "  theta:" << theta << endl;
        su.TransfCoordSphere2Image(theta, phi, cx, cy);
        ums.MapPerspToSphere(src[i], dst, mu.GetKMatrix(), cx, cy, 
                             SphericalUndistortion::PERSPECTIVE);
      }
    }
  } else {
    cout << "No images given!" 
         << endl;
    abort();
  }
  
  ImageIO::Write("Sphere", dst);
  cout << " created: "<<"Sphere.mip"<<endl;
  
  
  return 0;  
}
