#ifndef __RIG_PARAMETER_
#define __RIG_PARAMETER_

#include <bias_config.h>

//BIAS
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Geometry/KMatrix.hh>
#include <Geometry/RMatrix.hh>
#include <Geometry/PMatrix.hh>
#include <Base/Math/Matrix4x4.hh>

#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <string>

#include "CameraParam.hh"

#define RIG_PARAMETER_CURRENT_XML_VERSION 1.0

namespace BIAS {

  class BIASImage_EXPORT RigParameter { 
  public:
    RigParameter() { DownSamplingFactor_=1;};

    RigParameter(const RigParameter& RigParam) {
      Camera_ = RigParam.Camera_;
      Center_ = RigParam.Center_;
      Rotation_ = RigParam.Rotation_;
      Calibration_ = RigParam.Calibration_;
      SensorCenter_ = RigParam.SensorCenter_;
      SensorRotation_ = RigParam.SensorRotation_;
      DownSamplingFactor_ = RigParam.DownSamplingFactor_;
    };

    inline void GetDownSamplingFactor(double &Down) const{
      Down = DownSamplingFactor_;
    };
    ////////////////////////////////////////////////////////////////////////////
    inline bool SetCameraParam(CameraParam param,unsigned int index){
      if(index < Camera_.size()){Camera_[index]=param; return true;}
      else return false;
    };
    inline void GetCameraParams(std::vector<CameraParam> &CameraParam) const{
      CameraParam = Camera_;
    };
    inline bool GetCameraParam(CameraParam &CParam, unsigned int index) const{
      if(index < Camera_.size()){CParam = Camera_[index];return true;}
      else return false;
    };
    inline void DeleteCameraParams(){
      Camera_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////
    inline bool SetCameraCenter(BIAS::Vector3<double> center,unsigned int index){
      if(index < Center_.size()){Center_[index]=center; return true;}
      else return false;
    };
    inline void SetCameraCenters(std::vector< BIAS::Vector3<double> > center){
      Center_ = center;
    };
    inline void GetCameraCenters(std::vector<BIAS::Vector3<double> > &CameraCenters) const{
      CameraCenters = Center_;
    };
    inline bool GetCameraCenter(BIAS::Vector3<double> &CCenter, unsigned int index) const{
      if(index < Center_.size()){	CCenter = Center_[index];return true;}
      else return false;
    };
    inline void DeleteCameraCenters(){
      Center_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////
    inline bool SetRotation(BIAS::RMatrix rMat,unsigned int index){
      if(index < Rotation_.size()){Rotation_[index]=rMat; return true;}
      else return false;
    };

    inline void GetRotations( std::vector< BIAS::RMatrix >  &Rotations) const{
      Rotations = Rotation_;
    };
    inline bool GetRotation(BIAS::RMatrix &Rotation, unsigned int index) const{
      if(index < Rotation_.size()){Rotation = Rotation_[index];return true;}
      else return false;
    };
    inline void DeleteRotations(){
      Rotation_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////
    inline bool SetCalibration(BIAS::KMatrix kMat,unsigned int index){
      if(index < Calibration_.size())	{Calibration_[index]=kMat; return true;}
      else return false;
    };
    inline void GetCalibrations( std::vector< BIAS::KMatrix >  &Calibration) const{
      Calibration = Calibration_;
    };
    inline void DeleteCalibrations(){
      Calibration_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////
    inline bool SetSensorRotation(BIAS::RMatrix rSMat,unsigned int index){
      if(index < SensorRotation_.size()){SensorRotation_[index]=rSMat; return true;}
      else return false;
    };
    inline void GetSensorRotations( std::vector< BIAS::RMatrix >  &SRotation) const{
      SRotation = SensorRotation_;
    };
    inline bool GetSensorRotation(BIAS::RMatrix &SRotation, unsigned int index) const{
      if(index < SensorRotation_.size()){SRotation=SensorRotation_[index];return true;}
      else return false;
    };
    inline void DeleteSensorRotations(){
      SensorRotation_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////
    inline bool SetSensorCenter(BIAS::Vector3<double> vSC,unsigned int index){
      if(index < SensorCenter_.size())
      {SensorCenter_[index]=vSC; return true;}
      else return false;
    };
    inline void GetSensorCenters(std::vector<BIAS::Vector3<double> > &SensorCenters) const{
      SensorCenters = SensorCenter_;
    };

    inline bool GetSensorCenter(BIAS::Vector3<double> &SensorCenter, unsigned int index) const{
      if(index < SensorCenter_.size()){SensorCenter=SensorCenter_[index];return true;}
      else return false;
    };
    inline void DeleteSensorCenters(){
      SensorCenter_.clear();
    };
    /////////////////////////////////////////////////////////////////////////////


    ~RigParameter() {};

    inline unsigned int getNumberOfCameras() const{
      return Camera_.size(); };

      inline unsigned int getRigSize() const {
        return Center_.size(); };

        bool ReadFromCMP(const std::string &filename);

#ifdef BIAS_HAVE_XML2
        /** @brief Write all data camera data and camera positions in the rig
        to a rig parameter file in XML-notation
        @param Filename Name of the XML-rig-data-file, 
        document tag will be to be "Rig"
        @author streckel 06/2004 */
        int WriteRigData(const std::string &filename);
        /** @brief Read a rig parameter file and store the data in (*this)
        the Camera-, Center-, RMatrix-, and KMatrix-Vectors are 
        reset from the file.
        @param Filename Name of the XML-camera-data-file, 
        document tag has to be "Rig"
        @author streckel 06/2004 */
        int ReadRigData(const std::string &filename);
#endif //BIAS_HAVE_XML2

        void SetLadybugCenters(double value = 1.0);


        /** @brief compute mean rigid transformations of virtual camera into 
        rigcameras given a number of rigs
        @param Pvec contains n rigs of size Rigsize, first all cameras of rig1
        cameras must be in world coordinate system
        @param Rigsize size of each rig
        @param WeightOfCurrentParams if not zero, the current parameters
        are averaged with the new ones (weight 1.0)
        @author koeser 03/2004 */
        bool Compute(std::vector<BIAS::PMatrix>& Pvec,
          const unsigned int &Rigsize,
          const double& WeightOfCurrentParams = 0.0);

        /** @brief returns the K-Matrices in the vector
        the downsampling factor is applied to the matrices
        caution: the vector is erased befor refilled
        @ author schiller 07/05
        */
        void GetKMatrices(std::vector<BIAS::KMatrix>& vKmatrix);


        bool GetKMatrix(BIAS::KMatrix& matrix,int index) const;

        /** @brief Set the downsampling factor
        1 means no downsampling, 2 means downsampling by 2 ...
        0.5 means upsample!
        @author schiller
        */
        void SetDownsampling(const double factor);


        void SetData_DummyRig();

        /** @brief This sets a Camera and sets everything else to identity */ 
        void AddCameraOnly(CameraParam &Camera){
          //Prepare data
          double fl = Camera.GetFocalLength(); //focal length
          BIAS::Vector3<double> vec;
          vec[0]=0;vec[1]=0;vec[2]=0;
          BIAS::RMatrix rDummy;
          rDummy.SetIdentity();
          BIAS::KMatrix kDummy;
          kDummy.SetIdentity();
          //set kmatrix to correct values
          kDummy[0][0]= fl;
          kDummy[1][1]= fl;
          kDummy[0][2]= Camera.GetPrincipalX();
          kDummy[1][2]= Camera.GetPrincipalY();
          //and set it
          Camera_.push_back(Camera);
          Center_.push_back(vec);
          Rotation_.push_back(rDummy);
          Calibration_.push_back(kDummy);
          SensorRotation_.push_back(rDummy);
          SensorCenter_.push_back(vec);
        }

        void AddCamera(CameraParam &Camera){
          Camera_.push_back(Camera);
        };
        void AddCalibration(BIAS::KMatrix &kMatrix){
          Calibration_.push_back(kMatrix);
        };
        void AddRotation(BIAS::RMatrix &rMatrix){
          Rotation_.push_back(rMatrix);
        };
        void AddCameraCenter(BIAS::Vector3<double> &center){
          Center_.push_back(center);
        };
        void AddSensorRotation(BIAS::RMatrix &sRotation){
          SensorRotation_.push_back(sRotation);
        };
        void AddSensorCenter(BIAS::Vector3<double > &sCenter){
          SensorCenter_.push_back(sCenter);
        };

  protected:

    // DLL JW
    BIAS_EXPORT_STL_VECTOR( BIASImage_EXPORT, BIAS::CameraParam)
    BIAS_EXPORT_STL_VECTOR( BIASImage_EXPORT, BIAS::Vector3<double> )
    BIAS_EXPORT_STL_VECTOR( BIASImage_EXPORT, BIAS::RMatrix )
    BIAS_EXPORT_STL_VECTOR( BIASImage_EXPORT, BIAS::KMatrix )


    std::vector< BIAS::CameraParam >        Camera_;
    std::vector< BIAS::Vector3<double> >    Center_;
    std::vector< BIAS::RMatrix >            Rotation_;
    std::vector< BIAS::KMatrix >            Calibration_;
    std::vector< BIAS::RMatrix >            SensorRotation_;
    std::vector< BIAS::Vector3<double > >   SensorCenter_;

    double DownSamplingFactor_;

    int ParseView(const std::string &block);

    // read old rig datafile version
    int ReadRigDataV0_8(const std::string &filename);

    /** @brief adapts rotation representation in Axis/Angle such that averaging
    against RefAxis and RefAngle is possible without wrap-around problems 
    @author koeser 03/2004 */
    void AdaptAxisAngleRepresentation(BIAS::Vector3<double>& Axis,
      double& Angle,
      const BIAS::Vector3<double>& RefAxis,
      const double& RefAngle);

  };
  std::ostream& operator<<(std::ostream &os, BIAS::RigParameter &r);
}




#endif
