/** Jan Woetzel 2002 */

#include <MathAlgo/SVD.hh>
#include <MathAlgo/SVD3x3.hh>
#include <Base/Math/Random.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Base/Debug/TimeMeasure.hh>


using namespace BIAS;
using namespace std;

void Print(ostream& os, SVD& svd, SVD3x3&svd3x3)
{
  os << "U: SVD: "   <<svd.GetU()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetU()<<endl;
  os << "   diff: "<<svd.GetU()-svd3x3.GetU()<<endl;
      
  os << "S: SVD: "   <<svd.GetS()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetS()<<endl;
  os << "   diff: "<<svd.GetS()-svd3x3.GetS()<<endl;
      
  os << "V: SVD: "   <<svd.GetV()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetV()<<endl;
  os << "   diff: "<<svd.GetV()-svd3x3.GetV()<<endl;
}

//int main(int argc, char *arg[])
int main()
{
  Random rand;
  Matrix3x3<double> M;
  SVD svd;
  SVD3x3 svd3x3;
  TimeMeasure tsvd, tsvd3;
  
  int num=90000;
  for (int k=0; k<num; k++){
//     if (k==0){
//       M.SetZero();
//       M[0][0]=16384;
//       M[1][1]=M[2][2]=-16384;
//     } else {
      for (int i=0; i<9; i++){
        M.GetData()[i]=rand.GetUniformDistributed(-10.0, 10.0);
      }
      //    }
    
    tsvd3.Start();
    svd3x3.Compute(M);
    tsvd3.Stop();
    tsvd.Start();
    svd.Compute(M);
    tsvd.Stop();
    
    if (k==0){
      Print(cerr, svd, svd3x3);
    } /*        this does not work, wince svd is not unique

      else {
      if (svd.GetU()!=svd3x3.GetU()){
        BIASERR("differnt U in "<<k<<"th try");
        Print(cerr, svd, svd3x3); 
        break;
      }
      if (svd.GetS()!=svd3x3.GetS()){
        BIASERR("differnt S in "<<k<<"th try");
        Print(cerr, svd, svd3x3);   
        break;
      }
      if (svd.GetV()!=svd3x3.GetV()){
        BIASERR("differnt V in "<<k<<"th try");
        Print(cerr, svd, svd3x3);   
        break;
      }
      } */
    
    Matrix<double> id, res, D, inv;
    Vector<double> d;
    id.newsize(3,3);
    res.newsize(3,3);
    D.newsize(3,3);
    inv.newsize(3,3);
    d.newsize(3);
    id.SetIdentity();
    inv=svd.Invert();
    res=M*inv-id;
    if (!res.IsZero(1e-8)){
      BIASERR("svd different decomposition in "<<k<<"th try");
      cerr << "res :"<<res<<endl;
      cerr << "M :"<<M<<endl;
      Print(cerr, svd, svd3x3);  
      cerr << "svd:   M : " << M << endl;
      d=svd.GetS();
      D.SetZero();
      for (int i=0; i<3; i++) D[i][i]=d[i];
      cerr << "     USV': "<<svd.GetU()*D*svd.GetVT()<<endl;
      break;
    }
    inv=svd3x3.Invert();
    res=M*inv-id;
    if (!res.IsZero(1e-8)){
      BIASERR("svd3x3 different decomposition in "<<k<<"th try");
      cerr << "res :"<<res<<endl;
      cerr << "M :"<<M<<endl;
      Print(cerr, svd, svd3x3);  
      cerr << "svd3x3:   M : " << M << endl;
      d=svd3x3.GetS();
      D.SetZero();
      for (int i=0; i<3; i++) D[i][i]=d[i];
      cerr << "        USV': "<<svd3x3.GetU()*D*svd3x3.GetVT()<<endl;
       break;
   }
  }

  cerr << "svd: ";
  tsvd.Print();
  cerr << "user: "<<(double)tsvd.GetUserTime()/(double)num << " ms / svd\n";
  cerr << "real: "<<(double)tsvd.GetRealTime()/(double)num << " us / svd\n\n";
  cerr << "svd3x3: ";
  tsvd3.Print();
  cerr << "user: "<<(double)tsvd3.GetUserTime()/(double)num << " ms / svd\n";
  cerr << "real: "<<(double)tsvd3.GetRealTime()/(double)num << " us / svd\n\n";
  

  return 0;
}


