/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Median1D.hh"

using namespace BIAS;
using namespace std;

/////////////////////////////////////////////////////////////////////////
// implementation
/////////////////////////////////////////////////////////////////////////

template<class DataType>
Median1D<DataType>::Median1D()
{}

template<class DataType>
Median1D<DataType>::Median1D(vector<DataType>& vec)
{
  Compute(vec);
}

template<class DataType>
Median1D<DataType>::Median1D(const vector<DataType>& vec)
{
  Compute(vec);
}

template<class DataType>
Median1D<DataType>::~Median1D()
{}

template<class DataType>
void Median1D<DataType>::Compute(const vector<DataType>& vec)
{
  vector<DataType> tmp=vec;
  Compute(tmp);
}

/** This Quickselect routine is based on the algorithm described in
    "Numerical recipies in C", Second Edition,
    Cambridge University Press, 1992, Section 8.5, ISBN 0-521-43108-5 */

#define BIAS_SWAP(a,b) { t=(a); (a)=(b); (b)=t; }

template<class DataType>
void Median1D<DataType>::Compute(vector<DataType>& vec)
{
  int low, high;
  int med;
  int mid, ll, hh;
  int n=vec.size();
  register DataType t;
    
  low = 0 ; high = n-1 ; med = (low + high) / 2;
  for (;;) {
    if (high <= low) { /* One element only */
      _Median = vec[med];
      return;
    }

    if (high == low + 1) {  /* Two elements only */
      if (vec[low] > vec[high])
        BIAS_SWAP(vec[low], vec[high]) ;
      _Median = vec[med];
      return;
    }

    /* Find med of low, mid and high items; swap into position low */
    mid = (low + high) / 2;
    if (vec[mid] > vec[high]) BIAS_SWAP(vec[mid], vec[high]) ;
    if (vec[low] > vec[high]) BIAS_SWAP(vec[low], vec[high]) ;
    if (vec[mid] > vec[low])  BIAS_SWAP(vec[mid], vec[low]) ;

    /* Swap low item (now in position mid) into position (low+1) */
    BIAS_SWAP(vec[mid], vec[low+1]) ;

    /* Nibble from each end towards mid, swapping items when stuck */
    ll = low + 1;
    hh = high;
    for (;;) {
      do ll++; while (vec[low] > vec[ll]) ;
      do hh--; while (vec[hh]  > vec[low]) ;

      if (hh < ll)
        break;

      BIAS_SWAP(vec[ll], vec[hh]) ;
    }

    /* Swap mid item (in position low) back into correct position */
    BIAS_SWAP(vec[low], vec[hh]) ;

    /* Re-set active partition */
    if (hh <= med)
      low = ll;
    if (hh >= med)
      high = hh - 1;
  }
}

#undef BIAS_SWAP

/////////////////////////////////////////////////////////////////////////
// instantiation
/////////////////////////////////////////////////////////////////////////

template class BIAS::Median1D<double>;
template class BIAS::Median1D<float>;
template class BIAS::Median1D<int>;
