/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __Median1D_hh__
#define __Median1D_hh__

#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Debug/Debug.hh>

#include <vector>

namespace BIAS {

  /** @class Median1D
      @ingroup g_mathalgo
      @brief computes the median of a vector
      @author woelk 06/2005 */
  template <class DataType>
  class BIASMathAlgo_EXPORT Median1D : public Debug
  {
  public:
    Median1D();

    Median1D(std::vector<DataType>& vec);

    Median1D(const std::vector<DataType>& vec);

    ~Median1D();

    /** computation of the median using a quicksort approach
        @author woelk 06/2005 */
    void Compute(std::vector<DataType>& vec);

    void Compute(const std::vector<DataType>& vec);

    inline DataType GetMedian()
    { return _Median; }

  protected:
    DataType _Median;

  }; // class 


} // namespace


#include <Base/Common/BIASpragmaEnd.hh>

#endif // __Median1D_hh__

