// must be first:
#include <Base/Common/LeakChecking.h> 

#include <Base/Image/ImageIO.hh>
#include "Base/Image/ImageConvert.hh"
#include "Utils/Param.hh"
#include <stdio.h>
#include <iostream>

using namespace BIAS;
using namespace std;



/** @class biasdiff
    @ingroup g_tools
    @brief tool for computing intensity difference of images
    @author woelk */

int main(int argc, char *argv[]){
  // Dart Testing (JW)
  if (argc>=2) if (strcmp(argv[1],"-darttest")==0){
    std::cout<<"started dart test: "<<argv[0]<<std::endl; 
    return 0;
  }

    
  Param param;
  param.AddParamInt("border", "sets ROI", 0, 0,INT_MAX,'b');
  param.AddParamBool("absdiff","if set calculates abs difference else im1-im2",false, 'a');
  param.AddParamBool("help","",false, 'h');
  int next = param.ParseCommandLine(argc, argv);

  param.DisableDestructorWarning();
  
  if (argc<3 || *param.GetParamBool("help")){
    param.Usage();
    return -1;
  }

  ImageBase im1, im2, diff;
  if (ImageIO::Load(argv[next], im1)!=0){
    BIASERR("error loading "<<argv[next]);
    return -1;
  }
  if (ImageIO::Load(argv[next+1], im2)!=0){
    BIASERR("error loading "<<argv[next+1]);
    return -1;
  }


  int border = *param.GetParamInt("border");

  string newfilename;
  string fn1,fn2;
  fn1 = argv[next];
  fn2 = argv[next+1];
  fn1 = fn1.substr(0,fn1.rfind('.'));
  fn2 = fn2.substr(0,fn2.rfind('.'));
  newfilename = fn1+'_'+fn2+"_diff";

  im1.SetROI(border, border, im1.GetWidth()-border, im1.GetHeight()-border);
  im2.SetROI(border, border, im2.GetWidth()-border, im2.GetHeight()-border);
  BIASASSERT(im1.GetWidth()==im2.GetWidth());
  BIASASSERT(im1.GetHeight()==im2.GetHeight());

  Image<float> fim1, fim2, fdiff;
  Image<unsigned char> ucim1, ucim2, ucdiff;
  float fmin, fmax;
  unsigned char ucmin, ucmax;

  switch (im1.GetStorageType()){
  case ImageBase::ST_unsignedchar:
    ucim1=im1;
    ucim2=im2;
    if(*param.GetParamBool("absdiff"))
      ucdiff.AbsDiff(ucim1, ucim2);
    else
      {
	ImageConvert::ConvertST(ucim1,fim1, ImageBase::ST_float);
	ImageConvert::ConvertST(ucim2,fim2, ImageBase::ST_float);
	fim1 -= fim2;
	fim1.ScaleShiftBetween(0.0,255.0);
	ImageConvert::ConvertST(fim1,ucdiff, ImageBase::ST_unsignedchar);
      }
    
    if (ImageIO::Write(newfilename, ucdiff)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    ucdiff.GetMinMaxPixelValue(ucmin, ucmax);
    cerr << "differences between "<<(int)ucmin<<" and "<<(int)ucmax<<endl;
    if (ucdiff.ScaleShiftBetween(0, 255)!=0){
      BIASERR("error scaling image");
    }
    if (ImageIO::Write(newfilename+"-scaled", ucdiff)!=0){
      cerr << "error writing diff-scaled.mip "<<endl;
    }
    break;
  case ImageBase::ST_float:
    fim1=im1;
    fim2=im2;
    if(*param.GetParamBool("absdiff"))
      fdiff.AbsDiff(fim1, fim2);
    else
      {
	fdiff = fim1 - fim2;
      }

    if (ImageIO::Write(newfilename, fdiff)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    fdiff.GetMinMaxPixelValue(fmin, fmax);
    cerr << "differences between "<<fmin<<" and "<<fmax<<endl;
    break;
  default:
    cerr << "unknown storage type\n";
    break;
  }

  return 0;
}
