/**
\todo implement command line switch to terminate biasviewwx automatically 
with useful return code for dart ADD_TEST to be able to test biasvviewwx 
run in nightly cmake Dart dashboard (JW)
*/

#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/dnd.h>
#include <wx/filename.h>

#include <string>
#include <sstream>

#include <Gui/ImageCanvas.hh>
#include <Gui/ConsoleRedirectIO.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>


using namespace std;
using namespace BIAS;

#undef WXWIN_COMPATIBILITY

#if !wxUSE_DRAG_AND_DROP
#  error The dndFileTarget class requires drag and drop support in the WX library. Please recompile wxWidgets. __FILE__: __LINE__
#endif

/*
===============================================================================
MainFrame
===============================================================================
*/

enum
{
  // menu item IDs
  ID_Load,
  ID_Info,
  ID_MetaData,
  ID_Exit,
  ID_RunStop,
  ID_IncDelay,
  ID_DecDelay,
  ID_About,

  // toolbar item IDs
  ID_Next,
  ID_Prev,
  ID_Run,
  ID_Stop,
  ID_Fit,

  // timer ID
  ID_Timer
};

class MainFrame : public wxFrame
{

public:
  friend class DragAndDropTarget;

  MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void SetFileList(const vector<string>& fileNames);
  void LoadImages(const wxArrayString &paths);

protected:
  void LoadImageFromList(int index);

  ImageCanvas*        _ImageCanvasUC;
  ScaledImageCanvas*  _ImageCanvasFloat;
  vector<string>      _FileNames;
  int                 _CurrentImage;
  string              _MetaData;
  wxTimer             _Timer;
  wxStaticText*       _DelayLabel;
  int                 _Delay;
  bool                _BlockAnimation;
  wxCheckBox*         _FitCheckBox;
  bool                _NeedReload;

  // for file open dialog
  wxString            _DefaultFile;
  wxString            _DefaultDir;

  // event handlers
  void OnLoad(wxCommandEvent& event);
  void OnInfo(wxCommandEvent& event);
  void OnMetaData(wxCommandEvent& event);
  void OnExit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnRunStop(wxCommandEvent& event);
  void OnIncDelay(wxCommandEvent& event);
  void OnDecDelay(wxCommandEvent& event);

  void OnNext(wxCommandEvent& event);
  void OnPrev(wxCommandEvent& event);
  void OnRun(wxCommandEvent& event);
  void OnStop(wxCommandEvent& event);
  void OnDelay(wxCommandEvent& event);
  void OnFit(wxCommandEvent& event);

  void OnTimer(wxTimerEvent& event);
  void OnIdle(wxIdleEvent& event);
  void OnSize(wxSizeEvent& event);

  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
EVT_MENU    (ID_Load,     MainFrame::OnLoad)
EVT_MENU    (ID_Info,     MainFrame::OnInfo)
EVT_MENU    (ID_MetaData, MainFrame::OnMetaData)
EVT_MENU    (ID_Exit,     MainFrame::OnExit)
EVT_MENU    (ID_Next,     MainFrame::OnNext)
EVT_MENU    (ID_Prev,     MainFrame::OnPrev)
EVT_MENU    (ID_RunStop,  MainFrame::OnRunStop)
EVT_MENU    (ID_IncDelay, MainFrame::OnIncDelay)
EVT_MENU    (ID_DecDelay, MainFrame::OnDecDelay)
EVT_MENU    (ID_About,    MainFrame::OnAbout)
EVT_MENU    (ID_Fit,      MainFrame::OnFit)

EVT_BUTTON  (ID_Next,  MainFrame::OnNext)
EVT_BUTTON  (ID_Prev,  MainFrame::OnPrev)
EVT_BUTTON  (ID_Run,   MainFrame::OnRun)
EVT_BUTTON  (ID_Stop,  MainFrame::OnStop)
EVT_CHECKBOX(ID_Fit,   MainFrame::OnFit)

EVT_TIMER   (ID_Timer, MainFrame::OnTimer)
EVT_IDLE    (MainFrame::OnIdle)
EVT_SIZE    (MainFrame::OnSize)
END_EVENT_TABLE()


// ctor 
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame*)NULL, -1, title, pos, size)
,_ImageCanvasUC(NULL)
,_ImageCanvasFloat(NULL)
,_FitCheckBox(NULL)
{
  _CurrentImage     = 0;
  _DefaultFile      = "ucim.pgm";
  _DefaultDir       = "";
  _BlockAnimation   = true;

  // init timer
  _Timer.SetOwner(this, ID_Timer);

  // create file menu
  wxMenu* fileMenu = new wxMenu;
  fileMenu->Append(ID_Load, "&Open/Load...\tCtrl-O", "Open a file from disk for loading");
  fileMenu->Append(ID_Info, "Image Info...\tCtrl-I");
  fileMenu->Append(ID_MetaData, "Meta Data...\tCtrl-M");
  fileMenu->AppendSeparator();
  fileMenu->Append(ID_Exit, "E&xit", "Exit this application");

  // create help menu
  wxMenu* helpMenu = new wxMenu;
  helpMenu->Append(ID_About, "&About...\tF1");

  // create menu bar
  wxMenuBar* menuBar = new wxMenuBar;
  menuBar->Append(fileMenu, "&File");
  menuBar->Append(helpMenu, "&Help");
  SetMenuBar(menuBar);

  // create status bar
  const int numFields = 2;
  CreateStatusBar(numFields);
  int widths[] = { -3, -1 }; // sizes of fields in array, negative means relative *variable* size JW
  SetStatusWidths(numFields, widths);
  SetStatusText("Welcome to BiasViewWX", 0);

  // create tool bar
  wxToolBar* toolBar = CreateToolBar();
  toolBar->AddControl(new wxButton(toolBar, ID_Next, "Next"));
  toolBar->AddControl(new wxButton(toolBar, ID_Prev, "Previous"));
  toolBar->AddControl(new wxButton(toolBar, ID_Run,  "Run"));
  toolBar->AddControl(new wxButton(toolBar, ID_Stop, "Stop"));
  toolBar->AddSeparator();

  toolBar->AddControl(new wxStaticText(toolBar, -1, "Delay: "));
  _Delay = 100;
  _DelayLabel = new wxStaticText(toolBar, -1, "100", wxDefaultPosition,
    wxDefaultSize, wxST_NO_AUTORESIZE);
  toolBar->AddControl(_DelayLabel);

  toolBar->AddSeparator();
  _FitCheckBox = new wxCheckBox(toolBar, ID_Fit, "Fit to window");
  toolBar->AddControl(_FitCheckBox);
  toolBar->Realize();

  // set the shortcuts
  wxAcceleratorEntry entries[7];
  entries[0].Set(wxACCEL_NORMAL, WXK_NUMPAD_ADD,      ID_IncDelay);
  entries[1].Set(wxACCEL_NORMAL, '+',                 ID_IncDelay);
  entries[2].Set(wxACCEL_NORMAL, WXK_NUMPAD_SUBTRACT, ID_DecDelay);
  entries[3].Set(wxACCEL_NORMAL, '-',                 ID_DecDelay);
  entries[4].Set(wxACCEL_NORMAL, WXK_SPACE,           ID_Next);
  entries[5].Set(wxACCEL_NORMAL, WXK_BACK,            ID_Prev);
  entries[6].Set(wxACCEL_NORMAL, 's',                 ID_RunStop);
  wxAcceleratorTable accel(7, entries);
  SetAcceleratorTable(accel);
}

void MainFrame::SetFileList(const vector<string>& fileNames)
{
  _FileNames = fileNames;
  if (_FileNames.size() > 0)
  {
    LoadImageFromList(0);
  }
}

void MainFrame::LoadImageFromList(int index)
{
  if (_FileNames.size() == 0)
  {
    return;
  }

  _CurrentImage = index;
  const string& fileName = _FileNames[index];

  // load image
  ImageBase image;
  if (ImageIO::Load(fileName.c_str(), image)!=0)
  {
    stringstream msg;
    msg << "Error loading " << fileName;
    BIASERR(msg.str());
    SetStatusText(msg.str().c_str());
    return;
  }

  // save meta data for image info dialog
  stringstream info;
  switch (image.GetStorageType())
  {
  case ImageBase::ST_unsignedchar:
    {
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();     
      camera.PrintAppData(info);
    } break;

  case ImageBase::ST_float:
    {
      Camera<float> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;

  default:
    break;
  }
  _MetaData = info.str();

  // create appropriate image canvas based on storage type
  if (image.GetStorageType() == ImageBase::ST_unsignedchar)
  {
    // delete float image canvas
    if (_ImageCanvasFloat != NULL)
    {
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    // create uc image canvas
    if (_ImageCanvasUC == NULL)
    {
      _ImageCanvasUC = new ImageCanvas(this, GetStatusBar(), 1);
    }

    // convert image
    Image<unsigned char> ucImage;
    ucImage.StealImage(image);
    ucImage.SetUID(image.GetUID());

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasUC->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasUC->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasUC->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasUC->Show(ucImage, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasUC->SetScrollbars(0, 0, 0, 0);
    }
  }
  else
  {
    // delete uc image canvas
    if (_ImageCanvasUC != NULL)
    {
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }

    // create float image canvas
    if (_ImageCanvasFloat == NULL)
    {
      _ImageCanvasFloat = new ScaledImageCanvas(this, GetStatusBar(), 1);
    }

    // set appropriate scale and offset range
    switch (image.GetStorageType())
    {
    case ImageBase::ST_unsignedshortint:
      _ImageCanvasFloat->SetScaleRange(1.0/16.0, 1.0);
      _ImageCanvasFloat->SetOffsetRange(-4095.0, 0.0);
      break;

    case ImageBase::ST_char:
      _ImageCanvasFloat->SetScaleRange(0.5, 5.0);
      _ImageCanvasFloat->SetOffsetRange(-128.0, 128.0);
      break;

    case ImageBase::ST_float:
      _ImageCanvasFloat->SetScaleRange(0.5, 50.0);
      _ImageCanvasFloat->SetOffsetRange(-100.0, 100.0);
      break;

    default:
      cout << "Warning: Unknown storage type" << endl;
      // TODO: set default ranges? don't show image at all?
      break;
    }

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);
      width = max(1, width - 10); // little fix so the image is really fully visible
      height = max(1, height - 40); // larger fix because of sliders in float canvas

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasFloat->GetImageCanvas()->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasFloat->Show(image, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasFloat->GetImageCanvas()->SetScrollbars(0, 0, 0, 0);
    }
  }

  stringstream msg;
  msg << "File '" << fileName << "' (" << image.GetWidth() << "x" << image.GetHeight() << ") loaded";
  SetStatusText(msg.str().c_str());

  // make the shortcuts work without the need to click the canvas manually (on linux)
  if (_ImageCanvasUC) _ImageCanvasUC->SetFocus();
  if (_ImageCanvasFloat) _ImageCanvasFloat->SetFocus();

  // canvas needs to be refitted on windows after loading (not sure why)
#ifdef WIN32
  int w, h;
  GetClientSize(&w, &h);

  if (_ImageCanvasUC) _ImageCanvasUC->SetSize(0, 0, w, h);
  if (_ImageCanvasFloat) _ImageCanvasFloat->SetSize(0, 0, w, h);
#endif
}


void MainFrame::LoadImages(const wxArrayString &paths)
{
  // add file names to internal std::string list
  bool ok=true;
  for (int i = 0; i < (int)paths.GetCount(); i++) {
    if (wxFileName(paths[i]).IsDir()) 
      ok=false;
    _FileNames.push_back( string(paths[i]) );
  }

  // load last file and remember file name and directory
  if (!_FileNames.empty()) {
    LoadImageFromList(_FileNames.size() - 1);
    _DefaultFile = _FileNames[_FileNames.size() - 1].c_str();
  }

  if (!ok) {
    wxMessageBox("Dropping directories is unsupported. Please drop only filenames!",
      "Drag&Drop Error", wxOK | wxICON_EXCLAMATION);
  }
}


void MainFrame::OnLoad(wxCommandEvent& /*event*/)
{
  wxFileDialog fileDialog(
    this,                     // parent
    "Choose file(s) to load", // message
    _DefaultDir,              // default directory
    _DefaultFile,             // default file
    "*.*",                    // wildcard
    wxOPEN | wxMULTIPLE);     // style

  if (fileDialog.ShowModal() == wxID_OK)
  {
    // get file names
    wxArrayString paths;
    fileDialog.GetPaths(paths);
    // load them
    LoadImages(paths);
  } else {
    SetStatusText("Loading canceled");
    cout << "Canceled loading image" << endl;
  }

  _DefaultDir = fileDialog.GetDirectory();
}

void MainFrame::OnInfo(wxCommandEvent& /*event*/)
{
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL)
  {
    image = _ImageCanvasUC->GetOrigImage();
  }
  else if (_ImageCanvasFloat != NULL)
  {
    image = &_ImageCanvasFloat->GetOrigimg();
  }
  if (image == NULL)
  {
    wxMessageBox("No image loaded", "Image Info", wxOK | wxICON_ERROR, this);
    return;
  }

  // collect information
  stringstream info;
  info << "Width: " << image->GetWidth() << endl;
  info << "Height: " <<  image->GetHeight() << endl;
  info << "Depth: " << image->GetDepth() << endl;
  info << "BitDepth: " << image->GetBitDepth() << endl;
  info << "Size: " << image->GetSize() << " bytes" << endl;
  info << "Pixel Count: " << image->GetPixelCount() << endl;
  info << "Channels: " << image->GetChannelCount() << endl;
  info << "ColorModel: "  << image->GetColorModel()  << endl;
  info << "StorageType: " << image->GetStorageType() << endl;
  info << "IsPlanar: " << image->IsPlanar() << endl;
  info << "IsInterleaved: " << image->IsInterleaved() << endl;
  info << "UUID: " << image->GetUID() << endl;
  info << "Version: " << image->GetVersionNumber() << endl;

  // show information in message box
  wxMessageBox(info.str().c_str(), "Image Info", wxOK, this);
}

void MainFrame::OnMetaData(wxCommandEvent& /*event*/)
{
  wxFont font(10, wxMODERN, wxNORMAL, wxNORMAL, false, "Courier");
  wxDialog* dlg = new wxDialog(this, -1, "Meta Data", wxDefaultPosition, wxDefaultSize,
    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  wxTextCtrl* text = new wxTextCtrl(dlg, -1, "",
    wxDefaultPosition, wxSize(650, 400),
    wxTE_MULTILINE | wxTE_READONLY | wxTE_DONTWRAP);
#ifdef WIN32
  text->SetFont(font);
#endif
  text->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, font));
  text->AppendText(_MetaData.c_str());
  text->SetInsertionPoint(0);
  dlg->Fit();
  dlg->ShowModal();
  dlg->Destroy();
}

void MainFrame::OnExit(wxCommandEvent& /*event*/)
{
  Destroy();
}

void MainFrame::OnAbout(wxCommandEvent& /*event*/)
{
  stringstream msg;
  string version = "$Revision: 2.66 $";
  version = version.substr(9, version.length()-10);

  msg << "The BiasViewWX Image Viewer\n\n"
    << "http://www.mip.informatik.uni-kiel.de\n\n"
    << "build on " << __DATE__ << " " << __TIME__ << "\n"
    << "from file " << __FILE__ << "\n"
    << "with cvs file version " << version << "\n\n"
    << "GUI using: " << wxVERSION_STRING << "\n"
    << "\n";

  wxMessageBox(msg.str().c_str(), "About biasviewwx",
    wxOK | wxICON_INFORMATION, this);
}

void MainFrame::OnRunStop(wxCommandEvent& /*event*/)
{
  if (_Timer.IsRunning())
  {
    _Timer.Stop();
  }
  else
  {
    _Timer.Start(_Delay);
  }
}

void MainFrame::OnIncDelay(wxCommandEvent& /*event*/)
{
  _Delay += 10;

  wxString label;
  label.Printf("%d", _Delay);
  _DelayLabel->SetLabel(label);

  if (_Timer.IsRunning())
  {
    _Timer.Start(_Delay);
  }
}

void MainFrame::OnDecDelay(wxCommandEvent& /*event*/)
{
  _Delay -= 10;
  if (_Delay < 10)
  {
    _Delay = 10;
  }

  wxString label;
  label.Printf("%d", _Delay);
  _DelayLabel->SetLabel(label);

  if (_Timer.IsRunning())
  {
    _Timer.Start(_Delay);
  }
}

void MainFrame::OnNext(wxCommandEvent& /*event*/)
{
  _CurrentImage++;
  if (_CurrentImage >= (int)_FileNames.size())
  {
    _CurrentImage = 0;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnPrev(wxCommandEvent& /*event*/)
{
  _CurrentImage--;
  if (_CurrentImage < 0)
  {
    _CurrentImage = _FileNames.size() - 1;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnRun(wxCommandEvent& /*event*/)
{
  _Timer.Start(_Delay);
}

void MainFrame::OnStop(wxCommandEvent& /*event*/)
{
  _Timer.Stop();
}

void MainFrame::OnFit(wxCommandEvent& /*event*/)
{
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnTimer(wxTimerEvent& /*event*/)
{
  if (!_BlockAnimation)
  {
    // block animation until at least one idle event occurs,
    // so the viewer doesn't get stuck if delay is too low.
    _BlockAnimation = true;
    wxCommandEvent dummyEvent;
    OnNext(dummyEvent);
  }
}

void MainFrame::OnIdle(wxIdleEvent& /*event*/)
{
  _BlockAnimation = false;
}

void MainFrame::OnSize(wxSizeEvent& event)
{
  if (_FitCheckBox != NULL)
  {
    if (_FitCheckBox->IsChecked())
    {
      LoadImageFromList(_CurrentImage);
    }
  }
  event.Skip();
}

/*
===============================================================================
DragAndDropTarget
===============================================================================
*/

class DragAndDropTarget : public wxFileDropTarget
{
public:
  DragAndDropTarget(MainFrame* frame);

  virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
protected:
  MainFrame* _Frame;
};

DragAndDropTarget::DragAndDropTarget(MainFrame* frame)
{
  _Frame = frame;
}

bool DragAndDropTarget::OnDropFiles(wxCoord /*x*/, wxCoord /*y*/, const wxArrayString& filenames)
{
  BIASASSERT( _Frame != NULL);
  _Frame->LoadImages(filenames);

  return true;
}

/*
===============================================================================
BiasViewApp
===============================================================================
*/

class BiasViewApp : public wxApp
{
public:
  virtual bool OnInit();
};

bool BiasViewApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  ConsoleRedirectIO();

  MainFrame* frame = new MainFrame("BiasViewWX", wxPoint(50, 50), wxSize(700, 500));

  // read file list from command line
  std::vector<string> fileNames;
  for (int i = 1; i < argc; i++)
  {
    fileNames.push_back(string(argv[i]));
  }
  frame->SetFileList(fileNames);

  // create drag and drop target
  DragAndDropTarget* dndTarget = new DragAndDropTarget(frame);
  frame->SetDropTarget(dndTarget);

  // show frame
  frame->Show(TRUE);
  SetTopWindow(frame);

  return TRUE;
}

IMPLEMENT_APP(BiasViewApp)

