/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "GenSynthMatches.hh"
#include <Base/Image/ImageDraw.hh>
#include <Geometry/PMatrix.hh>

using namespace BIAS;
using namespace std;




ostream& BIAS::operator<<(ostream& os, const GenSynthMatches& m)
{
  
  HomgPoint3D p3d(0.0, 0.0, 0.0, 0.0);
  os << "true matches (0-"<<*m._NumStaticPoints-1<<"), outliers ("
     <<*m._NumStaticPoints<<"-"<<*m._NumStaticPoints+*m._NumOutliers-1
     <<") and moving objs ("<<*m._NumStaticPoints+*m._NumOutliers<<"-"
     <<*m._NumStaticPoints+*m._NumOutliers+
        *m._NumMovingPoints* *m._NumMovingObjects-1<<"): \n";
  for (int i=0; i<*m._NumStaticPoints; i++){
    os << setw(3)<<i<<" : "<< m._StaticPoints[i] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_Points[k][i]<<"  ";
    }
    os << endl;
  }  
  for (int i=*m._NumStaticPoints; i<*m._NumStaticPoints+*m._NumOutliers; i++){
       os << setw(3) << i << " : " << p3d << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_Points[k][i]<<"  ";
    }
    os << endl;
  } 
  for (int i=*m._NumStaticPoints+*m._NumOutliers; 
       i<*m._NumStaticPoints+*m._NumOutliers+
         *m._NumMovingPoints * *m._NumMovingObjects; i++){
    os << setw(3)<<i<<" : "<< 
      m._MovingPoints[i-*m._NumStaticPoints-*m._NumOutliers] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_Points[k][i]<<"  ";
    }
    os << endl;
  } 

  //////////////////////////////////////////////////

  os << "noisy matches (0-"<<*m._NumStaticPoints-1<<"), outliers ("
     <<*m._NumStaticPoints<<"-"<<*m._NumStaticPoints+*m._NumOutliers-1
     <<") and moving objs ("<<*m._NumStaticPoints+*m._NumOutliers<<"-"
     <<*m._NumStaticPoints+*m._NumOutliers+
       *m._NumMovingPoints * *m._NumMovingObjects-1<<"): \n";
  for (int i=0; i<*m._NumStaticPoints; i++){
    os << setw(3)<<i<<" : "<< m._StaticPoints[i] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._Points[k][i]<<"  ";
    }
    os << endl;
  }  
  for (int i=*m._NumStaticPoints; i<*m._NumStaticPoints+*m._NumOutliers; i++){
       os << setw(3) << i << " : " << p3d << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._Points[k][i]<<"  ";
    }
    os << endl;
  } 
  for (int i=*m._NumStaticPoints+*m._NumOutliers; 
       i<*m._NumStaticPoints+*m._NumOutliers+
         *m._NumMovingPoints * *m._NumMovingObjects; i++){
    os << setw(3)<<i<<" : "<< 
      m._MovingPoints[i-*m._NumStaticPoints-*m._NumOutliers] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._Points[k][i]<<"  ";
    }
    os << endl;
  } 

  /*
  //////////////////////////////////////////////////

  os << "normalized true matches (0-"<<*m._NumStaticPoints-1<<"), outliers ("
     <<*m._NumStaticPoints<<"-"<<*m._NumStaticPoints+*m._NumOutliers-1
     <<") and moving objs ("<<*m._NumStaticPoints+*m._NumOutliers<<"-"
     <<*m._NumStaticPoints+*m._NumOutliers+
    *m._NumMovingPoints * *m._NumMovingObjects-1<<"): \n";
  for (int i=0; i<*m._NumStaticPoints; i++){
    os << setw(3)<<i<<" : "<< m._StaticPoints[i] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  }  
  for (int i=*m._NumStaticPoints; i<*m._NumStaticPoints+*m._NumOutliers; i++){
       os << setw(3) << i << " : " << p3d << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  } 
  for (int i=*m._NumStaticPoints+*m._NumOutliers; 
       i<*m._NumStaticPoints+*m._NumOutliers+
         *m._NumMovingPoints * *m._NumMovingObjects; i++){
    os << setw(3)<<i<<" : "<< 
      m._MovingPoints[i-*m._NumStaticPoints-*m._NumOutliers] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._GT_NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  } 

  //////////////////////////////////////////////////

  os << "normalized noisy matches (0-"<<*m._NumStaticPoints-1<<"), outliers ("
     <<*m._NumStaticPoints<<"-"<<*m._NumStaticPoints+*m._NumOutliers-1
     <<") and moving objs ("<<*m._NumStaticPoints+*m._NumOutliers<<"-"
     <<*m._NumStaticPoints+*m._NumOutliers+
    *m._NumMovingPoints * *m._NumMovingObjects-1<<"): \n";
  for (int i=0; i<*m._NumStaticPoints; i++){
    os << setw(3)<<i<<" : "<< m._StaticPoints[i] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  } 
  for (int i=*m._NumStaticPoints; i<*m._NumStaticPoints+*m._NumOutliers; i++){
       os << setw(3) << i << " : " << p3d << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  } 
  for (int i=*m._NumStaticPoints+*m._NumOutliers; 
       i<*m._NumStaticPoints+*m._NumOutliers+*m._NumMovingPoints * 
         *m._NumMovingObjects; i++){
    os << setw(3)<<i<<" : "<< 
      m._MovingPoints[i-*m._NumStaticPoints-*m._NumOutliers] << " : ";
    for (int k=0; k<*m._NumImages; k++){
      os << k<<":"<<m._NormalizedPoints[k][i]<<"  ";
    }
    os << endl;
  } 
  */
  return os;
}

//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////


GenSynthMatches::GenSynthMatches(Param &para) 
  : MatchDataBase(para)
{
  _para=&para;

  _NumStaticPoints=_NumOutliers=_ImWidth=_ImHeight=NULL;
  _NumMovingPoints=_NumMovingObjects=NULL;
  _PrinciplePointX=_PrinciplePointY=_FocalLength=_SigmaImagePoints=NULL;
  _MaxDispOutliers=_SigmaCamCenterX=_OutlDirChange=_OutlLengthChange=NULL;
  _SigmaCamCenterY=_SigmaCamCenterZ=_SigmaCamRotX=_SigmaCamRotY=NULL;
  _SigmaCamRotZ=_MinCamDistance=_MaxCamDistance=_OverallMotionX=NULL;
  _OverallMotionY=_OverallMotionZ=_OverallRotX=_OverallRotY=_OverallRotZ=NULL;
  _SizeMovingObjX=_SizeMovingObjY=_SizeMovingObjZ=NULL;
  _UserGivenP=NULL;
  _MotionVecX=_MotionVecY=_MotionVecZ=_RotVecX=_RotVecY=_RotVecZ=NULL;
  _MovingObjMotionVecX=_MovingObjMotionVecY=_MovingObjMotionVecZ=NULL;
  _MovingObjRotVecX=_MovingObjRotVecY=_MovingObjRotVecZ=NULL;
  _AddParameter(*_para);
  Reset();
}

GenSynthMatches::~GenSynthMatches()
{}

int GenSynthMatches::CreateMatches()
{
  //  _para->ShowData();
  
  //cout << "DebugLevel is ";
  //PrintDebugLevel();
  Reset();

  _CheckParams();

  _CreateCamMovement();
  _CreateMovingTransforms();

  if (_CreateStatic3DPoints()!=0){
    BIASERR("user error: maybe no common viewing vaolume");
    return -2;
  }
  if (_CreateMoving3DPoints()!=0){
    BIASERR("user error: moving object not visible");
    return -1;
  }

  _CreateStatic2DPoints();
  _AddUniformDistributedOutliers();
  _CreateMoving2DPoints();
  return 0;
}

void GenSynthMatches::DrawTrue(Image<unsigned char>& im,
                               int minindex, int maxindex) const
{
  _Draw(im, minindex, maxindex, &_GT_Points);
} 

void GenSynthMatches::DrawNoisy(Image<unsigned char>& im,
                               int minindex, int maxindex) const
{
  _Draw(im, minindex, maxindex, &_Points);
} 


void GenSynthMatches::_Draw(Image<unsigned char>& im,
                            int minindex, int maxindex, 
                            const std::vector<std::vector<HomgPoint2D> >* p) const
{
  if (minindex<0 || minindex>=*_NumImages || 
      maxindex<=0 || maxindex>=*_NumImages){
    BIASERR("invalid image to draw");
    BIASABORT;
  }

  double mag=1.0;
  int imag=(int)rint(mag);
  if (!im.IsEmpty()) im.Release();
  im.Init(*_ImWidth*imag, *_ImHeight*imag, 3);
  im.FillImageWithConstValue((unsigned char)0);
  
  unsigned char red[3]={255, 0, 0};
  unsigned char green[3]={0, 255, 0};
  unsigned char hgreen[3]={0, 128, 0};
  unsigned char blue[3]={0, 0, 255};
  unsigned int start[2], end[2];
  unsigned int radius=1;
  for (int k=minindex+1; k<maxindex+1; k++){
    // draw inliers green
    for (int i=0; i<*_NumStaticPoints; i++){
      start[0]=(unsigned int)rint((*p)[k-1][i][0]*mag);
      start[1]=(unsigned int)rint((*p)[k-1][i][1]*mag);
      end[0]=(unsigned int)rint((*p)[k][i][0]*mag);
      end[1]=(unsigned int)rint((*p)[k][i][1]*mag);
      ImageDraw<unsigned char>::Line(im, start, end, hgreen);
      ImageDraw<unsigned char>::Line(im, start, start, green);
      if (k==minindex+1){
        if (start[0]>=radius && start[1]>=radius &&
            start[0]<=*_ImWidth*imag-radius-1 && 
            start[1]<=*_ImHeight*imag-radius-1) 
          ImageDraw<unsigned char>::CircleCenterFilled(im, start[0], start[1],
                                                       radius, green);
      }
    }
    // draw outliers blue
    for (int i=*_NumStaticPoints; i<*_NumStaticPoints+*_NumOutliers; i++){
      start[0]=(unsigned int)rint((*p)[k-1][i][0]*mag);
      start[1]=(unsigned int)rint((*p)[k-1][i][1]*mag);
      end[0]=(unsigned int)rint((*p)[k][i][0]*mag);
      end[1]=(unsigned int)rint((*p)[k][i][1]*mag);
      ImageDraw<unsigned char>::Line(im, start, end, blue);
      if (k==minindex+1){
        if (start[0]>=radius && start[1]>=radius &&
            start[0]<=*_ImWidth-radius-1 && start[1]<=*_ImHeight-radius-1)
          ImageDraw<unsigned char>::CircleCenterFilled(im, start[0], start[1],
                                                       radius, blue);
      }
    }
   // draw moving objects red
    for (int i=*_NumStaticPoints+*_NumOutliers; 
         i<*_NumStaticPoints+*_NumOutliers+*_NumMovingPoints * *_NumMovingObjects; i++){
      start[0]=(unsigned int)rint((*p)[k-1][i][0]*mag);
      start[1]=(unsigned int)rint((*p)[k-1][i][1]*mag);
      end[0]=(unsigned int)rint((*p)[k][i][0]*mag);
      end[1]=(unsigned int)rint((*p)[k][i][1]*mag);
      ImageDraw<unsigned char>::Line(im, start, end, red);
      if (k==minindex+1){
        if (start[0]>=radius && start[1]>=radius &&
            start[0]<=*_ImWidth-radius-1 && start[1]<=*_ImHeight-radius-1)
          ImageDraw<unsigned char>::CircleCenterFilled(im, start[0], start[1],
                                                       radius, red);
      }
    }
  }
}


int GenSynthMatches::WriteData(string file) const
{
  ofstream of(file.c_str());
  if (!of){
    BIASERR("error opening "<<file);
    return -1;
  }
  of << *this;
  if (!of){
    BIASERR("error writing to "<<file);
    return -1;
  }
  of.close();
  return 0;
}

// hack to disabled unreachabel warning
#if defined(WIN32)
#  pragma warning(push)
#  pragma warning(disable : 4702)
#endif
int GenSynthMatches::ReadData(string file)
{
  ifstream of(file.c_str());
  if (!of){
    BIASERR("error opening "<<file);
    return -1;
  }
  BIASERR("unfinished");

  ////  of >> *this;
  //if (!of){
  //  BIASERR("error reading from "<<file);
  //  return -1;
  //}
  //of.close();

  BIASABORT;
  return -1;
}
#if defined(WIN32)
#  pragma warning(pop)
#endif


void GenSynthMatches::Reset()
{
  MatchDataBase::Reset();
  _rand.Reset();
  
  // clear K, R, C and P
  //_K.SetZero();
  //_Ki.SetZero();
  _GT_R.clear();
  //_R.clear();
  _GT_C.clear();
  //_C.clear();
  _GT_P.clear();
  //_P.clear();

  // now clear the 3D data points
  _StaticPoints.clear();
  _MovingPoints.clear();
  
  // transformation for moving points
  _MovingTransform.clear();

  // now clear 2D points
  _GT_Points.clear();
  //_Points.clear();
  _GT_NormalizedPoints.clear();
  //_NormalizedPoints.clear();

}

void GenSynthMatches::GetGTR(int imNo, RMatrixBase& R) const
{ R = _GT_R[imNo]; }
void GenSynthMatches::GetGTR(std::vector<RMatrixBase>& R) const
{ R = _GT_R; }

void GenSynthMatches::GetGTC(int imNo, HomgPoint3D& C) const
{ C = _GT_C[imNo]; }
void GenSynthMatches::GetGTC(std::vector<HomgPoint3D>& C) const
{ C = _GT_C; }

void GenSynthMatches::GetGTP(int imNo, PMatrixBase& P) const
{ P = _GT_P[imNo]; }
void GenSynthMatches::GetGTP(std::vector<PMatrixBase>& P) const
{ P = _GT_P; }

void GenSynthMatches::GetGTPoints(int imNo, std::vector<HomgPoint2D>& p) const
{ p = _GT_Points[imNo]; }
void GenSynthMatches::GetGTPoints(std::vector<std::vector<HomgPoint2D> >& p) const
{ p = _GT_Points; }

void GenSynthMatches::GetGTNormalizedPoints(int imNo, std::vector<HomgPoint2D>& p) const
{ p = _GT_NormalizedPoints[imNo]; }
void GenSynthMatches::GetGTNormalizedPoints(std::vector<std::vector<HomgPoint2D> >& p) const
{ p = _GT_NormalizedPoints; }

void GenSynthMatches::Get3DPoints(std::vector<HomgPoint3D>& p) const
{
  p = _StaticPoints;
}

void GenSynthMatches::GetInliers(std::vector<bool>& inl) const
{
  inl.resize(*_NumPoints);
  for (int i=0; i<*_NumStaticPoints; i++) inl[i]=true;
  for (int i=*_NumStaticPoints; i<*_NumPoints; i++)  inl[i]=false;
}

/** A moving 3D point X_i is generated for image i from 3D Point X using the 
    euclidean transform E_i=[R_i | T_i]:
    X_i = E_i * X
    It projects into the image point x_i with the according projection matrix 
    P_i=[r_i'|-r_i'C_i] (where ' denotes transposition):
    x_i = P_i * X_i

    Finding a projection matrix P_x where each 3D point from image i-1 
    projects onto the same 2D points goes as follows:
    x_i = P_x X_{i-1}

    First take the 3D point and move it to ist new poition in image i. This is 
    done by first applying the inverse E_{i-1} and then E_i:
    X_i = E_i * E_{i-1}^{-1} * X_{i-1}
    Now use the old projection matrix P_i. 
    x_i = P_i * E_i * E_{i-1}^{-1} * X_{i-1}

    The new projection matrix P_x is composed as follows:
    P_x = P_i * E_i * E_{i-1}^{-1} 
        = [r_i'|-r_i'C_i] * [R_i | T_i] * [R_{i-1} | T_{i-1}]^{-1}
        = [r_i'|-r_i'C_i] * [R_i | T_i] * [R_{i-1}' | -R_{i-1}'T_{i-1}]
        = [r_i'|-r_i'C_i] * [R_i R_{i-1}' | T_i - R_i R_{i-1}'T_{i-1}]
        = [r_i' R_i R_{i-1}' | -r_i'C_i + r_i'(T_i - R_i R_{i-1}'T_{i-1})]
        = [r_i' R_i R_{i-1}' | -r_i'C_i + r_i'T_i - r_i'R_i R_{i-1}'T_{i-1}]
        = [r_i'R_iR_{i-1}' | -r_i'R_iR_{i-1}'R_{i-1}R_i'C_i + r_i'R_iR_{i-1}'R_{i-1}R_i'T_i - r_i'R_i R_{i-1}'T_{i-1}]
        = [r_i'R_iR_{i-1}' | -r_i'R_iR_{i-1}'(T_{i-1}+R_{i-1}R_i'(C_i-T_i))]
    setting R_x' = r_i'R_iR_{i-1}' is aequivalent to R_x = R_{i-1} R_i' r_i
    setting C_x = T_{i-1} + R_{i-1} R_i' (C_i - T_i)
    and leads to
        = [ R_x' | -R_x' C_x ]
    @author woelk 08/2004 */
void 
GenSynthMatches::GetGTNormalizedF(std::vector<std::vector<FMatrixBase> >& F) const
{
  Vector3<double> C1, C2, cc1, cc2;
  RMatrixBase R1, R2, R, r1, r2;
  int idx;

  //cerr << "resizing F with newsize "<<*_NumMovingObjects+1<<endl;
  F.resize(*_NumImages-1);
  for (int i=1; i<*_NumImages; i++){
    //cerr << "resizing F["<<i<<"] with newsize "<<*_NumMovingObjects+1<<endl;
    F[i-1].resize(*_NumMovingObjects+1);

    _GT_C[i-1].GetEuclidian(C1);
    _GT_C[i].GetEuclidian(C2);
    BIASCDOUT(GSM_FMATRIX, "\n------------------------------------------\n\n");
    BIASCDOUT(GSM_FMATRIX, "C1: "<<C1<<"R1: "<<_GT_R[i-1]<<endl);
    BIASCDOUT(GSM_FMATRIX, "C2: "<<C2<<"R2: "<<_GT_R[i]<<endl);
    F[i-1][0].Compose(_GT_R[i-1], C1, _GT_R[i], C2);
    BIASCDOUT(GSM_FMATRIX, "F from image "<<i-1<<" to "<<i
              <<" for static objects is "<<F[i-1][0]<<endl);
#ifdef BIAS_DEBUG
      if (DebugLevelIsSet(GSM_FMATRIX)){
        for (int k=0; k<*_NumStaticPoints; k++){
          BIASCDOUT(GSM_FMATRIX, setw(5)<<k<<"  p2^T * F * p1 = "
                    <<(F[i-1][0]*_GT_NormalizedPoints[i-1][k]).
                    ScalarProduct(_GT_NormalizedPoints[i][k])
                    <<"\t"<<_GT_NormalizedPoints[i-1][k]<<"  "
                    <<_GT_NormalizedPoints[i][k]<<endl);
        }
      }
#endif

    for (int l=0; l<*_NumMovingObjects; l++){
      _GT_C[i-1].GetEuclidian(C1);
      R1=_GT_R[i-1];
      BIASCDOUT(GSM_FMATRIX, "\n------\n\nC1: "<<C1<<"R1: "<<R1<<endl);

      idx=l * *_NumMovingObjects + i-1;
      cc1.Set((*_MovingObjMotionVecX)[idx], (*_MovingObjMotionVecY)[idx],
              (*_MovingObjMotionVecZ)[idx]);
      r1.SetXYZ((*_MovingObjRotVecX)[idx], (*_MovingObjRotVecY)[idx], 
               (*_MovingObjRotVecZ)[idx]);

      idx=l * *_NumMovingObjects + i;
      cc2.Set((*_MovingObjMotionVecX)[idx], (*_MovingObjMotionVecY)[idx],
              (*_MovingObjMotionVecZ)[idx]);
      _GT_C[i].GetEuclidian(C2);
      r2.SetXYZ((*_MovingObjRotVecX)[idx], (*_MovingObjRotVecY)[idx], 
                (*_MovingObjRotVecZ)[idx]);
      r1.Mult(r2.Transpose(), R);
      R.Mult((C2-cc2), C2);
      C2+=cc1;
      //      C2=cc1+r1*r2.Transpose()*(C2-cc2);
      R=_GT_R[i];
      R2=r1*r2.Transpose()*R;

      BIASCDOUT(GSM_FMATRIX, "C2: "<<C2<<"R2: "<<R2<<endl);

      F[i-1][l+1].Compose(R1, C1, R2, C2);
      BIASCDOUT(GSM_FMATRIX, "F from image "<<i-1<<" to "<<i<<" for "<<l
                <<"th objects is "<<F[i-1][l+1]<<endl);
#ifdef BIAS_DEBUG
      if (DebugLevelIsSet(GSM_FMATRIX)){
        for (int k=*_NumStaticPoints + *_NumOutliers + l * *_NumMovingPoints; 
             k<*_NumStaticPoints + *_NumOutliers + (l+1) * *_NumMovingPoints;
             k++){
          BIASCDOUT(GSM_FMATRIX, setw(5)<<k<<"  p2^T * F * p1 = "
                    <<(F[i-1][l+1]*_GT_NormalizedPoints[i-1][k]).
                    ScalarProduct(_GT_NormalizedPoints[i][k])
                    <<"\t"<<_GT_NormalizedPoints[i-1][k]<<"  "
                    <<_GT_NormalizedPoints[i][k]<<endl);
        }
      }
#endif
    }
  }
}

int GenSynthMatches::Write(const std::string& fname) const
{
  return MatchDataBase::Write(fname);
}

int GenSynthMatches::Write(ostream& os) const
{
  BIASCDOUT(D_MD_IO, "using GenSynthMatches::Write(ostream)\n");
  int res=MatchDataBase::Write(os);
  if (res==0){
    os.write((char*)_NumStaticPoints, sizeof(int));
    os.write((char*)_NumOutliers, sizeof(int));
    os.write((char*)_ImWidth, sizeof(int));
    os.write((char*)_ImHeight, sizeof(int));
    os.write((char*)_FocalLength, sizeof(double));
    os.write((char*)_PrinciplePointX, sizeof(double));
    os.write((char*)_PrinciplePointY, sizeof(double));
    os.write((char*)_SigmaImagePoints, sizeof(double));
    os.write((char*)_MaxDispOutliers, sizeof(double));
    os.write((char*)_OutlDirChange, sizeof(double));
    os.write((char*)_OutlLengthChange, sizeof(double));
    os.write((char*)_SigmaCamCenterX, sizeof(double));
    os.write((char*)_SigmaCamCenterY, sizeof(double));
    os.write((char*)_SigmaCamCenterZ, sizeof(double));
    os.write((char*)_SigmaCamRotX, sizeof(double));
    os.write((char*)_SigmaCamRotY, sizeof(double));
    os.write((char*)_SigmaCamRotZ, sizeof(double));
    os.write((char*)_MinCamDistance, sizeof(double));
    os.write((char*)_MaxCamDistance, sizeof(double));
    os.write((char*)_UserGivenP, sizeof(bool));
    os.write((char*)_OverallMotionX, sizeof(double));
    os.write((char*)_OverallMotionY, sizeof(double));
    os.write((char*)_OverallMotionZ, sizeof(double));
    os.write((char*)_OverallRotX, sizeof(double));
    os.write((char*)_OverallRotY, sizeof(double));
    os.write((char*)_OverallRotZ, sizeof(double));
    os.write((char*)_NumMovingPoints, sizeof(int));
    os.write((char*)_NumMovingObjects, sizeof(int));
    
    for (int i=0; i<*_NumMovingObjects; i++){
      os.write((char*)&(*_SizeMovingObjX)[i], sizeof(double));
      os.write((char*)&(*_SizeMovingObjY)[i], sizeof(double));
      os.write((char*)&(*_SizeMovingObjZ)[i], sizeof(double));
    }

    for (int i=0; i<*_NumStaticPoints; i++){
      os.write((char*)_StaticPoints[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
    }
    for (int i=0; i<*_NumMovingPoints * *_NumMovingObjects; i++){
      os.write((char*)_MovingPoints[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
    }

    for (int i=0; i<*_NumImages; i++){
      os.write((char*)&(*_MotionVecX)[i], sizeof(double));
      os.write((char*)&(*_MotionVecY)[i], sizeof(double));
      os.write((char*)&(*_MotionVecZ)[i], sizeof(double));
      os.write((char*)&(*_RotVecX)[i], sizeof(double));
      os.write((char*)&(*_RotVecY)[i], sizeof(double));
      os.write((char*)&(*_RotVecZ)[i], sizeof(double));
      for (int k=0; k<*_NumMovingObjects; k++){
        int idx=k * *_NumImages + i;
        os.write((char*)&(*_MovingObjMotionVecX)[idx], sizeof(double));
        os.write((char*)&(*_MovingObjMotionVecY)[idx], sizeof(double));
        os.write((char*)&(*_MovingObjMotionVecZ)[idx], sizeof(double));
        os.write((char*)&(*_MovingObjRotVecX)[idx], sizeof(double));
        os.write((char*)&(*_MovingObjRotVecY)[idx], sizeof(double));
        os.write((char*)&(*_MovingObjRotVecZ)[idx], sizeof(double)); 
      }
      os.write((char*)_GT_R[i].GetData(), sizeof(ROTATION_MATRIX_TYPE)*9);
      os.write((char*)_GT_C[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
      for (int k=0; k<*_NumMovingObjects; k++){
        int idx=k * *_NumImages + i;
        os.write((char*)_MovingTransform[idx].GetData(), sizeof(double)*16);
      }
      
    }
    // write outlier
    for (int i=0; i<*_NumImages; i++){
      for (int k=*_NumStaticPoints; k<*_NumStaticPoints+*_NumOutliers; k++){
        os.write((char*)_GT_Points[i][k].GetData(),sizeof(HOMGPOINT2D_TYPE)*3);
      }
    }

    if (!os) res=-3;
  }
  return res;
}

int GenSynthMatches::Read(const string& fname)
{
  return MatchDataBase::Read(fname);
}

int GenSynthMatches::Read(istream & is)
{
  BIASCDOUT(D_MD_IO, "using GenSynthMatches::Read(ostream)\n");
  int res = MatchDataBase::Read(is);
  if (res==0){
    is.read((char*)_NumStaticPoints, sizeof(int));
    is.read((char*)_NumOutliers, sizeof(int));
    is.read((char*)_ImWidth, sizeof(int));
    is.read((char*)_ImHeight, sizeof(int));
    is.read((char*)_FocalLength, sizeof(double));
    is.read((char*)_PrinciplePointX, sizeof(double));
    is.read((char*)_PrinciplePointY, sizeof(double));
    is.read((char*)_SigmaImagePoints, sizeof(double));
    is.read((char*)_MaxDispOutliers, sizeof(double));
    is.read((char*)_OutlDirChange, sizeof(double));
    is.read((char*)_OutlLengthChange, sizeof(double));
    is.read((char*)_SigmaCamCenterX, sizeof(double));
    is.read((char*)_SigmaCamCenterY, sizeof(double));
    is.read((char*)_SigmaCamCenterZ, sizeof(double));
    is.read((char*)_SigmaCamRotX, sizeof(double));
    is.read((char*)_SigmaCamRotY, sizeof(double));
    is.read((char*)_SigmaCamRotZ, sizeof(double));
    is.read((char*)_MinCamDistance, sizeof(double));
    is.read((char*)_MaxCamDistance, sizeof(double));
    is.read((char*)_UserGivenP, sizeof(bool));
    is.read((char*)_OverallMotionX, sizeof(double));
    is.read((char*)_OverallMotionY, sizeof(double));
    is.read((char*)_OverallMotionZ, sizeof(double));
    is.read((char*)_OverallRotX, sizeof(double));
    is.read((char*)_OverallRotY, sizeof(double));
    is.read((char*)_OverallRotZ, sizeof(double));
    is.read((char*)_NumMovingPoints, sizeof(int));
    is.read((char*)_NumMovingObjects, sizeof(int));
    _SizeMovingObjX->newsize(*_NumMovingObjects);
    _SizeMovingObjY->newsize(*_NumMovingObjects);
    _SizeMovingObjZ->newsize(*_NumMovingObjects);
    for (int i=0; i<*_NumMovingObjects; i++){
      is.read((char*)&(*_SizeMovingObjX)[i], sizeof(double));
      is.read((char*)&(*_SizeMovingObjY)[i], sizeof(double));
      is.read((char*)&(*_SizeMovingObjZ)[i], sizeof(double));
    }

    _StaticPoints.resize(*_NumStaticPoints);
    for (int i=0; i<*_NumStaticPoints; i++){
      is.read((char*)_StaticPoints[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
    }
    _MovingPoints.resize(*_NumMovingPoints * *_NumMovingObjects);
    for (int i=0; i<*_NumMovingPoints * *_NumMovingObjects; i++){
      is.read((char*)_MovingPoints[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
    }

    _MotionVecX->newsize(*_NumImages);
    _MotionVecY->newsize(*_NumImages);
    _MotionVecZ->newsize(*_NumImages);
    _RotVecX->newsize(*_NumImages);
    _RotVecY->newsize(*_NumImages);
    _RotVecZ->newsize(*_NumImages);
    _MovingObjMotionVecX->newsize(*_NumImages * *_NumMovingObjects);
    _MovingObjMotionVecY->newsize(*_NumImages * *_NumMovingObjects);
    _MovingObjMotionVecZ->newsize(*_NumImages * *_NumMovingObjects);
    _MovingObjRotVecX->newsize(*_NumImages * *_NumMovingObjects);
    _MovingObjRotVecY->newsize(*_NumImages * *_NumMovingObjects);
    _MovingObjRotVecZ->newsize(*_NumImages * *_NumMovingObjects);
    _GT_R.resize(*_NumImages);
    _GT_C.resize(*_NumImages);
    _GT_P.resize(*_NumImages);
    _GT_Points.resize(*_NumImages);
    _GT_NormalizedPoints.resize(*_NumImages);
    _MovingTransform.resize(*_NumImages * *_NumMovingObjects);

    for (int i=0; i<*_NumImages; i++){
      is.read((char*)&(*_MotionVecX)[i], sizeof(double));
      is.read((char*)&(*_MotionVecY)[i], sizeof(double));
      is.read((char*)&(*_MotionVecZ)[i], sizeof(double));
      is.read((char*)&(*_RotVecX)[i], sizeof(double));
      is.read((char*)&(*_RotVecY)[i], sizeof(double));
      is.read((char*)&(*_RotVecZ)[i], sizeof(double));
      for (int k=0; k<*_NumMovingObjects; k++){
        int idx=k * *_NumImages + i;
        is.read((char*)&(*_MovingObjMotionVecX)[idx], sizeof(double));
        is.read((char*)&(*_MovingObjMotionVecY)[idx], sizeof(double));
        is.read((char*)&(*_MovingObjMotionVecZ)[idx], sizeof(double));
        is.read((char*)&(*_MovingObjRotVecX)[idx], sizeof(double));
        is.read((char*)&(*_MovingObjRotVecY)[idx], sizeof(double));
        is.read((char*)&(*_MovingObjRotVecZ)[idx], sizeof(double)); 
      }

      is.read((char*)_GT_R[i].GetData(), sizeof(ROTATION_MATRIX_TYPE)*9);
      is.read((char*)_GT_C[i].GetData(), sizeof(HOMGPOINT3D_TYPE)*4);
      _GT_P[i].Compose(_K, _GT_R[i], _GT_C[i]);
      for (int k=0; k<*_NumMovingObjects; k++){
        int idx=k * *_NumImages + i;
        is.read((char*)_MovingTransform[idx].GetData(), sizeof(double)*16);
      }
    }
    _CreateStaticGT2DPoints();
    _CreateMovingGT2DPoints();
    // read outlier
    for (int i=0; i<*_NumImages; i++){
      for (int k=*_NumStaticPoints; k<*_NumStaticPoints+*_NumOutliers; k++){
        is.read((char*)_GT_Points[i][k].GetData(),sizeof(HOMGPOINT2D_TYPE)*3);
        _GT_NormalizedPoints[i][k]=_Ki*_GT_Points[i][k];
      }
    }
    if (!is) res=-3;
  } 
  return res;
}

void GenSynthMatches::_CreateCamMovement()
{
  _K[0][0]=_K[1][1]=*_FocalLength;
  _K[2][2]=1.0;
  _K[0][2]=*_PrinciplePointX;
  _K[1][2]=*_PrinciplePointY;

  _Ki=_K.Invert();

  _GT_R.resize(*_NumImages);
  _R.resize(*_NumImages);
  _GT_C.resize(*_NumImages);
  _C.resize(*_NumImages);
  _GT_P.resize(*_NumImages);
  _P.resize(*_NumImages);
  
  HomgPoint3D C, nC;
  RMatrixBase R, nR;
  for (int i=0; i<*_NumImages; i++){
     if (*_UserGivenP){
      BIASASSERT((int)_MotionVecX->Size()>=*_NumImages);
      BIASASSERT((int)_MotionVecY->Size()>=*_NumImages);
      BIASASSERT((int)_MotionVecZ->Size()>=*_NumImages);
      BIASASSERT((int)_RotVecX->Size()>=*_NumImages);
      BIASASSERT((int)_RotVecY->Size()>=*_NumImages);
      BIASASSERT((int)_RotVecZ->Size()>=*_NumImages);
      C.Set((*_MotionVecX)[i], (*_MotionVecY)[i], (*_MotionVecZ)[i], 1.0);
      nC.Set(C[0]+_rand.GetNormalDistributed(0.0, *_SigmaCamCenterX),
             C[1]+_rand.GetNormalDistributed(0.0, *_SigmaCamCenterY),
             C[2]+_rand.GetNormalDistributed(0.0, *_SigmaCamCenterZ),
             1.0);
      R.SetXYZ((*_RotVecX)[i], (*_RotVecY)[i], (*_RotVecZ)[i]);
      nR.SetXYZ((*_RotVecX)[i]+_rand.GetNormalDistributed(0.0,*_SigmaCamRotX),
                (*_RotVecY)[i]+_rand.GetNormalDistributed(0.0,*_SigmaCamRotY),
                (*_RotVecZ)[i]+_rand.GetNormalDistributed(0.0,*_SigmaCamRotZ));
    } else { // if (*_UserGivenP){
      BIASERR("unfinished");
    } // if (*_UserGivenP){
    
    _GT_R[i]=R;
    _R[i]=nR;
    _GT_C[i]=C;
    _C[i]=nC;
    _GT_P[i].Compose(_K, _GT_R[i], _GT_C[i]);
    _P[i].Compose(_K, _R[i], _C[i]);
#ifdef BIAS_DEBUG
    BIASCDOUT(GSM_CAM_MOTION, "_K: "<<_K<<endl);
    BIASCDOUT(GSM_CAM_MOTION, "_Ki: "<<_Ki<<endl);
    if (DebugLevelIsSet(GSM_CAM_MOTION)){
      Vector3<double> r, nr;
      _GT_R[i].GetRotationAnglesXYZ(r[0], r[1], r[2]);
      _R[i].GetRotationAnglesXYZ(nr[0], nr[1], nr[2]);
      BIASCDOUT(GSM_CAM_MOTION, "_GT_R["<<i<<"] = "<<r<<endl);
      BIASCDOUT(GSM_CAM_MOTION, "_R["<<i<<"] = "<<nr<<endl);
      BIASCDOUT(GSM_CAM_MOTION, "_GT_C["<<i<<"] = "<<_GT_C[i]<<endl);
      BIASCDOUT(GSM_CAM_MOTION, "_C["<<i<<"] = "<<_C[i]<<endl);
      BIASCDOUT(GSM_CAM_MOTION, "_GT_P["<<i<<"] = "<<_GT_P[i]<<endl);
      BIASCDOUT(GSM_CAM_MOTION, "_P["<<i<<"] = "<<_P[i]<<endl);
    }
#endif
  }
}

void GenSynthMatches::_CreateMovingTransforms()
{
  _MovingTransform.resize(*_NumImages * *_NumMovingObjects);
  if (*_NumMovingObjects>0 && *_NumMovingPoints>0){
    Vector3<double> r, t;
    const int ni=*_NumImages;
    int ofs, idx;
    for (int n=0; n<*_NumMovingObjects; n++){
      BIASCDOUT(GSM_MOVIN_TRANSF, n<<"th moving object\n");
      ofs = ni*n;
      for (int k=0; k<ni; k++){
        idx=ofs+k;
        t[0]=(*_MovingObjMotionVecX)[idx];
        t[1]=(*_MovingObjMotionVecY)[idx];
        t[2]=(*_MovingObjMotionVecZ)[idx];
        r[0]=(*_MovingObjRotVecX)[idx];
        r[1]=(*_MovingObjRotVecY)[idx];
        r[2]=(*_MovingObjRotVecZ)[idx];
        _MovingTransform[idx].SetXYZ(r, t);
        BIASCDOUT(GSM_MOVIN_TRANSF, idx<<"th moving transf: "
                  <<_MovingTransform[idx]<<endl);
      }
    }
  }
}


int GenSynthMatches::_CreateStatic3DPoints()
{
  double minx, maxx, miny, maxy, minz, maxz;
  // explanation of var names
  // 1. character: f/l - belonging first/last P Matrix 
  // 2. character: n/f - near/far clippingplane
  // 3. character: u/l - upper/lower image corner 
  // 4. character: r/l - left/right image corner
  // points in array are stored in this order
  // p3d[0]=fnul, p3d[1]=fnur, p3d[2]=fnll, p3d[3]=fnlr, p3d[4]=fful, 
  // p3d[5]=ffur, p3d[6]=ffll, p3d[7]=fflr, p3d[8]=lnul, p3d[9]=lnur, 
  // p3d[10]=lnll, p3d[11]=lnlr, p3d[12]=lful, p3d[13]=lfur, p3d[14]=lfll, 
  // p3d[15]=lflr;
  HomgPoint3D p3d[16];
  HomgPoint3D fC, lC;
  fC=_GT_C[0];
  lC=_GT_C[*_NumImages-1];
  PMatrix fP, lP;
  fP=_GT_P[0];
  lP=_GT_P[*_NumImages-1];
  
  // now calculate the volume seen by at minimum one camera
  // backproject the image corners into space and apply min/max depth
  Vector3<double> ray;
  HomgPoint2D ul(0.0, 0.0), ur(*_ImWidth-1.0, 0.0), 
    ll(0.0, *_ImHeight-1.0), lr(*_ImWidth-1.0, *_ImHeight-1.0);
  
  fP.BackprojectWorldCoo(ul, *_MinCamDistance, p3d[0]);
  fP.BackprojectWorldCoo(ur, *_MinCamDistance, p3d[1]);
  fP.BackprojectWorldCoo(ll, *_MinCamDistance, p3d[2]);
  fP.BackprojectWorldCoo(lr, *_MinCamDistance, p3d[3]);
  fP.BackprojectWorldCoo(ul, *_MaxCamDistance, p3d[4]);
  fP.BackprojectWorldCoo(ur, *_MaxCamDistance, p3d[5]);
  fP.BackprojectWorldCoo(ll, *_MaxCamDistance, p3d[6]);
  fP.BackprojectWorldCoo(lr, *_MaxCamDistance, p3d[7]);

  lP.BackprojectWorldCoo(ul, *_MinCamDistance, p3d[8]);
  lP.BackprojectWorldCoo(ur, *_MinCamDistance, p3d[9]);
  lP.BackprojectWorldCoo(ll, *_MinCamDistance, p3d[10]);
  lP.BackprojectWorldCoo(lr, *_MinCamDistance, p3d[11]);
  lP.BackprojectWorldCoo(ul, *_MaxCamDistance, p3d[12]);
  lP.BackprojectWorldCoo(ur, *_MaxCamDistance, p3d[13]);
  lP.BackprojectWorldCoo(ll, *_MaxCamDistance, p3d[14]);
  lP.BackprojectWorldCoo(lr, *_MaxCamDistance, p3d[15]);

  minx=miny=minz=DBL_MAX;
  maxx=maxy=maxz=-DBL_MAX;
  for (int j=0; j<16; j++){
    BIASCDOUT(GSM_VIEWINGVOL, "p3d["<<j<<"] = "<<p3d[j]<<endl);
    if (p3d[j][0]<minx) minx=p3d[j][0];
    if (p3d[j][0]>maxx) maxx=p3d[j][0];
    if (p3d[j][1]<miny) miny=p3d[j][1];
    if (p3d[j][1]>maxy) maxy=p3d[j][1];
    if (p3d[j][2]<minz) minz=p3d[j][2];
    if (p3d[j][2]>maxz) maxz=p3d[j][2];
  }
  BIASCDOUT(GSM_VIEWINGVOL, "Generating 3D point between ["<<minx<<", "
            <<miny<<", "<<minz<<"] and ["<<maxx<<", "<<maxy<<", "<<maxz
            <<"]\n");
  
  // generate static points
  HomgPoint3D p;
  HomgPoint2D p2d, np2d;
  int i=0, mtry;
  bool is_seen=true, seen_in_k=false;
  _StaticPoints.resize(*_NumStaticPoints);
  while(i<*_NumStaticPoints){
    mtry=0;
    do {
      is_seen=true;
      p.Set(_rand.GetUniformDistributed(minx, maxx),
            _rand.GetUniformDistributed(miny, maxy),
            _rand.GetUniformDistributed(minz, maxz));
      for (int k=0; k<*_NumImages; k++){
        p2d=_GT_P[k]*p; 
        np2d=_P[k]*p; 
        // 99 % of all noisy points lie with *_SigmaImagePoints * 2.6 distance 
        // of true image points
        seen_in_k = (_IsSeen(p2d, *_SigmaImagePoints * 2.6) && 
                     _IsSeen(np2d, *_SigmaImagePoints * 2.6));
        is_seen=is_seen && seen_in_k;
        if (!seen_in_k) {
          BIASCDOUT(GSM_3D_POINTS, p<<" not seen in P["<<k<<"] : "<<p2d
                    <<"/"<<np2d<<"\n");
          break;
        }
      }
      if (mtry>1000){
        for (int k=0; k<*_NumImages; k++){
          cerr << "_P["<<k<<"]: "<<_P[k]<<endl;
          cerr << "_GT_P["<<k<<"]: "<<_GT_P[k]<<endl;
        }
        cerr << "tried to generate 3D points between ["<<minx<<", "<<miny
             <<", "<<minz<<"] and ["<<maxx<<", "<<maxy<<", "<<maxz<<"]\n";
        BIASERR("more than 1000 tries for one 3D point, maybe no common "
                <<"viewing volume present");
        return -1;
      }
      mtry++;
    } while (!is_seen);
    _StaticPoints[i]=p;
    BIASCDOUT(GSM_3D_POINTS, i<<"th true 3D point: "<<_StaticPoints[i]<<endl);
    i++;
  }
  return 0;
}

int GenSynthMatches::_AreMovingObjectsVisible()
{
  if (*_NumMovingPoints>0 && *_NumMovingObjects>0){
    HomgPoint3D corner[8], p3d;
    HomgPoint2D p2d, np2d;
    bool seen_in_k=true, any_seen_in_k=false;
    double r1, r2, r3;
      
    for (int i=0; i<*_NumMovingObjects; i++){
      corner[0][3]=corner[1][3]=corner[2][3]=corner[3][3]=1.0;
      corner[4][3]=corner[5][3]=corner[6][3]=corner[7][3]=1.0;

      corner[0][2]=corner[1][2]=corner[2][2]=corner[3][2]=
        (*_SizeMovingObjZ)[i]/2.0;
      corner[4][2]=corner[5][2]=corner[6][2]=corner[7][2]=
        -(*_SizeMovingObjZ)[i]/2.0;

      corner[0][1]=corner[1][1]=corner[4][1]=corner[5][1]=
        (*_SizeMovingObjY)[i]/2.0;
      corner[2][1]=corner[3][1]=corner[6][1]=corner[7][1]=
        -(*_SizeMovingObjY)[i]/2.0;
    
      corner[0][0]=corner[3][0]=corner[4][0]=corner[7][0]=
        (*_SizeMovingObjX)[i]/2.0;
      corner[1][0]=corner[2][0]=corner[5][0]=corner[6][0]=
        -(*_SizeMovingObjX)[i]/2.0;

      for (int k=0; k<*_NumImages; k++){
        int idx = i * *_NumImages + k;
        // check if center of moving object is visible
        for (int l=0; l<8; l++){
          //cerr <<setw(3)<<l<<"  "<<corner[l]<<endl;
          p3d=_MovingTransform[idx]*corner[l];
          p2d=_GT_P[k]*p3d;
          np2d=_P[k]*p3d;
          seen_in_k = _IsSeen(p2d) && _IsSeen(np2d);
          any_seen_in_k = any_seen_in_k || seen_in_k;
        }
        if (!any_seen_in_k){
          cerr << "_MovingTransform["<<idx<<"]: "<<_MovingTransform[idx]
               <<endl;
          cerr << "_P["<<k<<"]: "<<_P[k]<<endl;
          cerr << "_GT_P["<<k<<"]: "<<_GT_P[k]<<endl;
          cerr << "_R["<<k<<"]: "<<_R[k]<<endl;
          _R[k].GetRotationAnglesXYZ(r1, r2, r3);
          cerr <<"rotation angles of _R["<<k<<"]: "<<r1<<"\t"<< r2<<"\t"
               <<r3<<endl;
          cerr << "_GT_R["<<k<<"]: "<<_GT_R[k]<<endl;
          cerr << "_C["<<k<<"]: "<<_C[k]<<endl;
          cerr << "_GT_C["<<k<<"]: "<<_GT_C[k]<<endl;
          cerr << "corners: "<<endl;
          for (int l=0; l<8; l++){
            p3d=_MovingTransform[idx]*corner[l];
            p2d=_GT_P[k]*p3d;
            np2d=_P[k]*p3d;
            cerr <<setw(3)<<l<<" corn: "<<corner[l]<<"\tp3d: "<<p3d<<"\tp2d: "
                 <<p2d<<"\tnp2d: "<<np2d<<endl;
          }
          BIASERR("no corner of moving object not seen in image "<<k);
          return -1;
        }
        // check if center is visible
        p3d.Set(0.0, 0.0, 0.0);
        p3d=_MovingTransform[idx]*p3d;
        p2d=_GT_P[k]*p3d;
        np2d=_P[k]*p3d;
        seen_in_k = _IsSeen(p2d) && _IsSeen(np2d);
        if (!seen_in_k){
          cerr << "_MovingTransform["<<idx<<"]: "<<_MovingTransform[idx]<<endl;
          cerr << "_P["<<k<<"]: "<<_P[k]<<endl;
          cerr << "_GT_P["<<k<<"]: "<<_GT_P[k]<<endl;
          cerr << "_R["<<k<<"]: "<<_R[k]<<endl;
          _R[k].GetRotationAnglesXYZ(r1, r2, r3);
          cerr <<"rotation angles of _R["<<k<<"]: "<<r1<<"\t"<< r2<<"\t"
               <<r3<<endl;
          cerr << "_GT_R["<<k<<"]: "<<_GT_R[k]<<endl;
          cerr << "_C["<<k<<"]: "<<_C[k]<<endl;
          cerr << "_GT_C["<<k<<"]: "<<_GT_C[k]<<endl;
          BIASERR("center of moving object not seen in image "<<k<<"  center "
                  <<p3d<<"  p2d: "<<p2d<<"  np2d: "<<np2d);
          return -2;
        }
      }
    }
  }
  return 0;
  
}


int GenSynthMatches::_CreateMoving3DPoints()
{
  _MovingPoints.resize(*_NumMovingPoints * *_NumMovingObjects);
  double x2, y2, z2;
  
  HomgPoint2D p2d, np2d;
  HomgPoint3D p3d;
  bool seen_in_all=true, seen_in_k=true;
  int mpi;

  if (_AreMovingObjectsVisible()!=0) return -1;

  for (int l=0; l<*_NumMovingObjects; l++){
    x2=(*_SizeMovingObjX)[l]/2.0;
    y2=(*_SizeMovingObjY)[l]/2.0;
    z2=(*_SizeMovingObjZ)[l]/2.0;
    for (int i=0; i<*_NumMovingPoints; i++){
      mpi=i + l* *_NumMovingPoints;
      do {
        seen_in_all=true;
        _MovingPoints[mpi].Set(_rand.GetUniformDistributed(-x2, x2),
                               _rand.GetUniformDistributed(-y2, y2),
                               _rand.GetUniformDistributed(-z2, z2), 1.0);
        for (int k=0; k<*_NumImages; k++){
          p3d=_MovingTransform[l* *_NumImages + k]*_MovingPoints[mpi];
          p2d=_GT_P[k]*p3d;
          np2d=_P[k]*p3d;
          seen_in_k = _IsSeen(p2d) && _IsSeen(np2d);
          if (!seen_in_k){
            BIASCDOUT(GSM_3D_POINTS, p3d<<" moving point not seen in k "
                      <<p2d<<"/"<<np2d<<endl);
            break;
          }
        }
        seen_in_all = seen_in_k && seen_in_all;
      } while (!seen_in_all);
      BIASCDOUT(GSM_3D_POINTS, mpi<<"th moving 3D point : "
                <<_MovingPoints[mpi]<<endl);
    }
  }
  return 0;
}

void GenSynthMatches::_CreateStatic2DPoints()
{
  _CreateStaticGT2DPoints();
  std::vector<HomgPoint2D> empty(*_NumStaticPoints + 
                                 *_NumMovingPoints * *_NumMovingObjects 
                                 + *_NumOutliers);
  _Points.assign(*_NumImages, empty);
  _NormalizedPoints.assign(*_NumImages, empty);

  BIASCDOUT(GSM_NOISE, "Static2D: "<<*_SigmaImagePoints<<endl);
  
  HomgPoint2D noise;
  for (int k=0; k<*_NumImages; k++){
    for (int i=0; i<*_NumStaticPoints; i++){
      _Points[k][i]=_P[k]*_StaticPoints[i];
      _Points[k][i].Homogenize();
      noise.Set(_rand.GetNormalDistributed(0.0, *_SigmaImagePoints),
                _rand.GetNormalDistributed(0.0, *_SigmaImagePoints), 0.0);
      BIASCDOUT(GSM_NOISE, "s2d: "<<noise << endl);
      _Points[k][i]+=noise;
      _NormalizedPoints[k][i]=_Ki*_Points[k][i];
      if (!_IsSeen(_Points[k][i])){
        BIASERR("static point "<<i<<"<< not seen in image "<<k<<" : "
                <<_Points[k][i]);
      }
    }
  }
#ifdef BIASDEBUG
  if (DebugLevelIsSet(GSM_2D_MATCHES)){
    for (int i=0; i<*_NumStaticPoints; i++){
      BIASCDOUT(GSM_2D_MATCHES, "match "<<i<<" :  ");
      for (int k=0; k<*_NumImages; k++){
        BIASCDOUT(GSM_2D_MATCHES, k<<":"<<_GT_Points[k][i]<<"/"
                  <<_Points[k][i]<<"/"<<_GT_NormalizedPoints[k][i]<<"/"
                  <<_NormalizedPoints[k][i]<<"    ");
      }
      BIASCDOUT(GSM_2D_MATCHES, endl);
    }
  }
#endif
}

void GenSynthMatches::_CreateStaticGT2DPoints()
{
  std::vector<HomgPoint2D> empty(*_NumStaticPoints 
                                 + *_NumMovingPoints * *_NumMovingObjects + 
                                 *_NumOutliers);
  _GT_Points.assign(*_NumImages, empty);
  _GT_NormalizedPoints.assign(*_NumImages, empty);

  for (int k=0; k<*_NumImages; k++){
    for (int i=0; i<*_NumStaticPoints; i++){
      _GT_Points[k][i]=_GT_P[k]*_StaticPoints[i];
      _GT_Points[k][i].Homogenize();
      _GT_NormalizedPoints[k][i]=_Ki*_GT_Points[k][i];
      if (!_IsSeen(_GT_Points[k][i])){
        BIASERR("static GT point "<<i<<"<< not seen in image "<<k<<" : "
                <<_GT_Points[k][i]);
      }
    }
  }
}

void GenSynthMatches::_AddUniformDistributedOutliers()
{
  BIASCDOUT(GSM_NOISE, "AddOutlier: "<<*_SigmaImagePoints<<endl);
  HomgPoint2D noise, outldisp;
  bool seen_from_all=true, seen_in_k=true;
  RMatrixBase R;
  for (int i=*_NumStaticPoints; i<*_NumStaticPoints+*_NumOutliers; i++){
    do {
      seen_from_all=true;
      _GT_Points[0][i].Set(_rand.GetUniformDistributed(0.0, *_ImWidth-1.0),
                        _rand.GetUniformDistributed(0.0, *_ImHeight-1.0), 1.0);
      noise.Set(_rand.GetNormalDistributed(0.0, *_SigmaImagePoints),
                _rand.GetNormalDistributed(0.0, *_SigmaImagePoints), 0.0);
      BIASCDOUT(GSM_NOISE, "outl: "<<noise << endl);
      _Points[0][i]=_GT_Points[0][i]+noise;
      _GT_NormalizedPoints[0][i]=_Ki*_GT_Points[0][i];
      _NormalizedPoints[0][i]=_Ki*_Points[0][i];
      seen_in_k = _IsSeen(_GT_Points[0][i]) && _IsSeen(_Points[0][i]);
      seen_from_all = seen_from_all && seen_in_k;
      // first match determines direction
      outldisp.Set(_rand.GetUniformDistributed(-*_MaxDispOutliers, 
                                               *_MaxDispOutliers), 
                   _rand.GetUniformDistributed(-*_MaxDispOutliers, 
                                               *_MaxDispOutliers), 0.0);
      _GT_Points[1][i]=_GT_Points[0][i]+outldisp;
      noise.Set(_rand.GetNormalDistributed(0.0, *_SigmaImagePoints),
                _rand.GetNormalDistributed(0.0, *_SigmaImagePoints), 0.0);
      BIASCDOUT(GSM_NOISE, "outl: "<<noise << endl);
      _Points[1][i]=_GT_Points[1][i]+noise;
      _GT_NormalizedPoints[1][i]=_Ki*_GT_Points[1][i];
      _NormalizedPoints[1][i]=_Ki*_Points[1][i];
      seen_in_k = _IsSeen(_GT_Points[1][i]) && _IsSeen(_Points[1][i]);
      seen_from_all = seen_from_all && seen_in_k;
      // direction is taken from last match
      for (int k=2; k<*_NumImages; k++){
        R.SetXYZ(0.0, 0.0, _rand.GetNormalDistributed(0.0,*_OutlDirChange));
        outldisp=R*outldisp;
        outldisp*=_rand.GetNormalDistributed(1.0,*_OutlLengthChange);
        _GT_Points[k][i]=_GT_Points[k-1][i]+outldisp;
        noise.Set(_rand.GetNormalDistributed(0.0, *_SigmaImagePoints),
                  _rand.GetNormalDistributed(0.0, *_SigmaImagePoints), 0.0);
        BIASCDOUT(GSM_NOISE, "outl: "<<noise << endl);
        _Points[k][i]=_GT_Points[k][i]+noise;
        _GT_NormalizedPoints[k][i]=_Ki*_GT_Points[k][i];
        _NormalizedPoints[k][i]=_Ki*_Points[k][i];
        seen_in_k = _IsSeen(_GT_Points[k][i]) && _IsSeen(_Points[k][i]);
        seen_from_all = seen_from_all && seen_in_k;
        if (!seen_from_all) break;
      }
    } while (!seen_from_all);
  }
  
#ifdef BIASDEBUG
  if (DebugLevelIsSet(GSM_2D_MATCHES)){
    for (int i=*_NumStaticPoints; i<*_NumStaticPoints+*_NumOutliers; i++){
      BIASCDOUT(GSM_2D_MATCHES, "match "<<i<<" :  ");
      for (int k=0; k<*_NumImages; k++){
        BIASCDOUT(GSM_2D_MATCHES, k<<":"<<_GT_Points[k][i]<<"/"
                  <<_Points[k][i]<<"/"<<_GT_NormalizedPoints[k][i]<<"/"
                  <<_NormalizedPoints[k][i]<<"    ");
      }
      BIASCDOUT(GSM_2D_MATCHES, endl);
    }
  }
#endif
}


void GenSynthMatches::_CreateMoving2DPoints()
{
  if ((int)_GT_Points.size()!=*_NumImages || 
      (int)_GT_Points[0].size()!=*_NumStaticPoints+*_NumOutliers+
      *_NumMovingPoints * *_NumMovingObjects){
    BIASERR("internal data structs not allocated call "
            <<"_CreateStatic2DPoints() first ");
    BIASABORT;
  }

  _CreateMovingGT2DPoints();

  BIASCDOUT(GSM_NOISE, "Moving2D: "<<*_SigmaImagePoints<<endl);

  HomgPoint2D noise;
  HomgPoint3D p3d;
  for (int l=0; l<*_NumMovingObjects; l++){
    int offset=*_NumStaticPoints+*_NumOutliers + l * *_NumMovingPoints;
    BIASCDOUT(GSM_MOVIN_3D_POINTS, "object "<<l<<endl);
    for (int k=0; k<*_NumImages; k++){
      int idx=l* *_NumImages + k;
      for (int i=offset; i<offset+*_NumMovingPoints; i++){
        p3d=_MovingTransform[idx]*_MovingPoints[i-offset+l* *_NumMovingPoints];
        BIASCDOUT(GSM_MOVIN_3D_POINTS, "using moving transf "<<idx
                  <<" and _MovingPoints["<<i-offset+l * *_NumMovingPoints
                  <<"] to create point in image "<<k<<"\n");
        _Points[k][i]=_P[k]*p3d;
        _Points[k][i].Homogenize();
        noise.Set(_rand.GetNormalDistributed(0.0, *_SigmaImagePoints),
                  _rand.GetNormalDistributed(0.0, *_SigmaImagePoints), 0.0);
        BIASCDOUT(GSM_NOISE, "m2d: "<<noise << endl);
        _Points[k][i]+=noise;
        BIASCDOUT(GSM_MOVIN_3D_POINTS, "moving 3d point "<<i<<" "<<p3d
                  <<"\t "<<_Points[k][i]<<endl);
        _NormalizedPoints[k][i]=_Ki*_Points[k][i];
        if (!_IsSeen(_Points[k][i])){
          BIASERR("moving point "<<i<<"<< not seen in image "<<k<<" : "
                  <<_Points[k][i]);
        }
      }
    }
  }
}

void GenSynthMatches::_CreateMovingGT2DPoints()
{
  if ((int)_GT_Points.size()!=*_NumImages || 
      (int)_GT_Points[0].size()!=*_NumStaticPoints+*_NumOutliers+
      *_NumMovingPoints * *_NumMovingObjects){
    BIASERR("internal data structs not allocated call "
            <<"_CreateStatic2DPoints() first ");
    BIASABORT;
  }

  HomgPoint3D p3d;
  for (int l=0; l<*_NumMovingObjects; l++){
    int offset=*_NumStaticPoints + *_NumOutliers + l * *_NumMovingPoints;
    BIASCDOUT(GSM_MOVIN_3D_POINTS, "object "<<l<<endl);
    for (int k=0; k<*_NumImages; k++){
      int idx=l * *_NumImages + k;
      for (int i=offset; i<offset+*_NumMovingPoints; i++){
        p3d=_MovingTransform[idx]*_MovingPoints[i-offset+l* *_NumMovingPoints];
        BIASCDOUT(GSM_MOVIN_3D_POINTS, "using moving transf "<<idx
                  <<" and _MovingPoints["<<i-offset+l * *_NumMovingPoints
                  <<"] to create point in image "<<k<<"\n");
        _GT_Points[k][i]=_GT_P[k]*p3d;
        _GT_Points[k][i].Homogenize();
        BIASCDOUT(GSM_MOVIN_3D_POINTS, "moving 3d point "<<i<<" "<<p3d
                  <<"\t "<<_GT_Points[k][i]<<endl);
        
        _GT_NormalizedPoints[k][i]=_Ki*_GT_Points[k][i];
        if (!_IsSeen(_GT_Points[k][i])){
          BIASERR("moving point "<<i<<"<< not seen in image "<<k<<" : "
                  <<_GT_Points[k][i]);
        }
      }
    }
  }
  BIASCDOUT(GSM_MOVIN_3D_POINTS, endl);
}


void GenSynthMatches::_AddParameter(Param &para)
{
  ////////////////////////////////////////////////////////////
  //         PGNum
  ///////////////////////////////////////////////////////////

  int grp;
  //  para.ShowData();
  if (para.CheckParam("NumImages")){
    grp=para.GetGroupID("NumImages");
  } else {
    grp=para.GetFreeGroupID();
  }
  //  cerr << "GenSynthMatches: using ID "<<grp<<" for group num\n";

  if (para.CheckParam("NumStaticPoints")){ 
    _NumStaticPoints=para.GetParamInt("NumStaticPoints");
  } else {
    _NumStaticPoints=para.AddParamInt("NumStaticPoints", "number of true matches", 10, 0, INT_MAX, 0, grp); 
  }

  if (para.CheckParam("NumOutliers")){ 
    _NumOutliers=para.GetParamInt("NumOutliers");
  } else {
    _NumOutliers=para.AddParamInt("NumOutliers", "number of outliers", 10, 0, INT_MAX, 0, grp); 
  }

  if (para.CheckParam("ImWidth")){ 
    _ImWidth=para.GetParamInt("ImWidth");
  } else {
    _ImWidth=para.AddParamInt("ImWidth", "image width", 640, 0, INT_MAX, 0, grp); 
  }

  if (para.CheckParam("ImHeight")){ 
    _ImHeight=para.GetParamInt("ImHeight");
  } else {
    _ImHeight=para.AddParamInt("ImHeight", "image height", 480, 0, INT_MAX, 0, grp); 
  }

  if (para.CheckParam("NumMovingPoints")){ 
    _NumMovingPoints=para.GetParamInt("NumMovingPoints");
  } else {
    _NumMovingPoints=para.AddParamInt("NumMovingPoints", "number of moving points", 10, 0, INT_MAX, 0, grp); 
  }

  if (para.CheckParam("NumMovingObjects")){ 
    _NumMovingObjects=para.GetParamInt("NumMovingObjects");
  } else {
    _NumMovingObjects=para.AddParamInt("NumMovingObjects", "number of moving objects", 1, 0, INT_MAX, 0, grp); 
  }

  if (para.IsUsedGroupID(grp))
    para.SetGroupName(grp, "numbers");

  ////////////////////////////////////////////////////////////
  //         PGMotion
  ///////////////////////////////////////////////////////////

  grp=para.GetFreeGroupID();
  //cerr << "using ID "<<grp<<" for group motion\n";

  if (para.CheckParam("FocalLength")){ 
    _FocalLength=para.GetParamDouble("FocalLength");
  } else {
    _FocalLength=para.AddParamDouble("FocalLength", "focal length in pixel", 840.0, 0.0, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("PrinciplePointX")){ 
    _PrinciplePointX=para.GetParamDouble("PrinciplePointX");
  } else {
    _PrinciplePointX=para.AddParamDouble("PrinciplePointX", "x component of principle point", 320.0, 0.0, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("PrinciplePointY")){ 
    _PrinciplePointY=para.GetParamDouble("PrinciplePointY");
  } else {
    _PrinciplePointY=para.AddParamDouble("PrinciplePointY", "y component of principle point", 240.0, 0.0, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallMotionX")){ 
    _OverallMotionX=para.GetParamDouble("OverallMotionX");
  } else {
    _OverallMotionX=para.AddParamDouble("OverallMotionX", "motion in x direction", -1.0, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallMotionY")){ 
    _OverallMotionY=para.GetParamDouble("OverallMotionY");
  } else {
    _OverallMotionY=para.AddParamDouble("OverallMotionY", "motion in y direction", 0.0, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallMotionZ")){ 
    _OverallMotionZ=para.GetParamDouble("OverallMotionZ");
  } else {
    _OverallMotionZ=para.AddParamDouble("OverallMotionZ", "motion in z direction", 10.0, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallRotX")){ 
    _OverallRotX=para.GetParamDouble("OverallRotX");
  } else {
    _OverallRotX=para.AddParamDouble("OverallRotX", "rotation around x axis", 0.0, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallRotY")){ 
    _OverallRotY=para.GetParamDouble("OverallRotY");
  } else {
    _OverallRotY=para.AddParamDouble("OverallRotY", "rotation around y axis", 0.5, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("OverallRotZ")){ 
    _OverallRotZ=para.GetParamDouble("OverallRotZ");
  } else {
    _OverallRotZ=para.AddParamDouble("OverallRotZ", "rotation around z axis", 0.0, -DBL_MAX, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("UserGivenP")){ 
    _UserGivenP=para.GetParamBool("UserGivenP");
  } else {
    _UserGivenP=para.AddParamBool("UserGivenP", "if true use MotionVec? and RotVec?, if false use Overall? ", true, 0, grp); 
  }

  Vector<double> dflt;
  dflt.newsize(2);
  dflt[0]=0.0; dflt[1]=-0.1;
  if (para.CheckParam("MotionVecX")){ 
    _MotionVecX=para.GetParamVecDbl("MotionVecX");
  } else {
    _MotionVecX=para.AddParamVecDbl("MotionVecX", "vector of x components of camera centers", dflt, 0, grp); 
  }

  dflt[0]=0.0; dflt[1]=0.0;
  if (para.CheckParam("MotionVecY")){ 
    _MotionVecY=para.GetParamVecDbl("MotionVecY");
  } else {
    _MotionVecY=para.AddParamVecDbl("MotionVecY", "vector of y components of camera centers", dflt, 0, grp); 
  }

  dflt[0]=0.0; dflt[1]=1.0;
  if (para.CheckParam("MotionVecZ")){ 
    _MotionVecZ=para.GetParamVecDbl("MotionVecZ");
  } else {
    _MotionVecZ=para.AddParamVecDbl("MotionVecZ", "vector of z components of camera centers", dflt, 0, grp); 
  }

  dflt[0]=0.0; dflt[1]=0.0;
  if (para.CheckParam("RotVecX")){ 
    _RotVecX=para.GetParamVecDbl("RotVecX");
  } else {
    _RotVecX=para.AddParamVecDbl("RotVecX", "vector of rotation around x-axis", dflt, 0, grp); 
  }
 
  dflt[0]=0.0; dflt[1]=-0.1;
  if (para.CheckParam("RotVecY")){ 
    _RotVecY=para.GetParamVecDbl("RotVecY");
  } else {
    _RotVecY=para.AddParamVecDbl("RotVecY", "vector of rotation around y-axis", dflt, 0, grp); 
  }
 
  dflt[0]=0.0; dflt[1]=0.0;
  if (para.CheckParam("RotVecZ")){ 
    _RotVecZ=para.GetParamVecDbl("RotVecZ");
  } else {
    _RotVecZ=para.AddParamVecDbl("RotVecZ", "vector of rotation around z-axis", dflt, 0, grp); 
  }
 
  if (para.IsUsedGroupID(grp))
    para.SetGroupName(grp, "cam motion");

  ////////////////////////////////////////////////////////////
  //         PGNoise
  ///////////////////////////////////////////////////////////

  grp=para.GetFreeGroupID();
  //cerr << "using ID "<<grp<<" for group noise\n";

  if (para.CheckParam("SigmaImagePoints")){ 
    _SigmaImagePoints=para.GetParamDouble("SigmaImagePoints");
  } else {
    _SigmaImagePoints=para.AddParamDouble("SigmaImagePoints", "sigma of normal distribution in image", 1.0, 0.0, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("MaxDispOutliers")){ 
    _MaxDispOutliers=para.GetParamDouble("MaxDispOutliers");
  } else {
    _MaxDispOutliers=para.AddParamDouble("MaxDispOutliers", "max allowed flow for outliers", 25.0, 0.0, DBL_MAX, 0, grp); 
  }

  if (para.CheckParam("SigmaCamCenterX")){ 
    _SigmaCamCenterX=para.GetParamDouble("SigmaCamCenterX");
  } else {
    _SigmaCamCenterX=para.AddParamDouble("SigmaCamCenterX", "sigma of normal distributed noise of cam centers", 0.1, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("SigmaCamCenterY")){ 
    _SigmaCamCenterY=para.GetParamDouble("SigmaCamCenterY");
  } else {
    _SigmaCamCenterY=para.AddParamDouble("SigmaCamCenterY", "sigma of normal distributed noise of cam centers", 0.1, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("SigmaCamCenterZ")){ 
    _SigmaCamCenterZ=para.GetParamDouble("SigmaCamCenterZ");
  } else {
    _SigmaCamCenterZ=para.AddParamDouble("SigmaCamCenterZ", "sigma of normal distributed noise of cam centers", 0.1, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("SigmaCamRotX")){ 
    _SigmaCamRotX=para.GetParamDouble("SigmaCamRotX");
  } else {
    _SigmaCamRotX=para.AddParamDouble("SigmaCamRotX", "sigma of normal distributed noise of cam rotation", 0.01, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("SigmaCamRotY")){ 
    _SigmaCamRotY=para.GetParamDouble("SigmaCamRotY");
  } else {
    _SigmaCamRotY=para.AddParamDouble("SigmaCamRotY", "sigma of normal distributed noise of cam rotation", 0.01, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("SigmaCamRotZ")){ 
    _SigmaCamRotZ=para.GetParamDouble("SigmaCamRotZ");
  } else {
    _SigmaCamRotZ=para.AddParamDouble("SigmaCamRotZ", "sigma of normal distributed noise of cam rotation", 0.01, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("MinCamDistance")){ 
    _MinCamDistance=para.GetParamDouble("MinCamDistance");
  } else {
    _MinCamDistance=para.AddParamDouble("MinCamDistance", "minimal distance a point must have to every camera", 1.0, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("MaxCamDistance")){ 
    _MaxCamDistance=para.GetParamDouble("MaxCamDistance");
  } else {
    _MaxCamDistance=para.AddParamDouble("MaxCamDistance", "maximal distance a point can have to every camera", 50.0, 0.0, DBL_MAX, 0, grp); 
  }
  
  if (para.CheckParam("OutlDirChange")){ 
    _OutlDirChange=para.GetParamDouble("OutlDirChange");
  } else {
    _OutlDirChange=para.AddParamDouble("OutlDirChange", "maximal change of direction for outlying matches (RAD)", 0.05, -M_PI, M_PI, 0, grp); 
  }
  
  if (para.CheckParam("OutlLengthChange")){ 
    _OutlLengthChange=para.GetParamDouble("OutlLengthChange");
  } else {
    _OutlLengthChange=para.AddParamDouble("OutlLengthChange", "maximal change of length for outlying matches", 0.05, -1.0, 1.0, 0, grp); 
  }

  if (para.IsUsedGroupID(grp))
    para.SetGroupName(grp, "noise");
  
  ////////////////////////////////////////////////////////////
  //         PGMovObj
  ///////////////////////////////////////////////////////////

  grp=para.GetFreeGroupID();
  //cerr << "using ID "<<grp<<" for group moving objects\n";
  Vector<double> sizedflt;
  sizedflt.newsize(2);
  sizedflt[0]=5.0; sizedflt[1]=5.0;
  if (para.CheckParam("SizeMovingObjX")){ 
    _SizeMovingObjX=para.GetParamVecDbl("SizeMovingObjX");
  } else {
    _SizeMovingObjX=para.AddParamVecDbl("SizeMovingObjX", "x size of moving object", sizedflt, 0, grp); 
  }
  sizedflt[0]=1.5; sizedflt[1]=1.5;
  if (para.CheckParam("SizeMovingObjY")){ 
    _SizeMovingObjY=para.GetParamVecDbl("SizeMovingObjY");
  } else {
    _SizeMovingObjY=para.AddParamVecDbl("SizeMovingObjY", "y size of moving object", sizedflt, 0, grp); 
  }
  sizedflt[0]=2.0; sizedflt[1]=2.0;
  if (para.CheckParam("SizeMovingObjZ")){ 
    _SizeMovingObjZ=para.GetParamVecDbl("SizeMovingObjZ");
  } else {
    _SizeMovingObjZ=para.AddParamVecDbl("SizeMovingObjZ", "z size of moving object", sizedflt, 0, grp); 
  }
  
  dflt.newsize(4);
  dflt[0]=10.0; dflt[1]=10.5; dflt[2]=0.0; dflt[3]=-1.0;
  if (para.CheckParam("MovingObjMotionVecX")){ 
    _MovingObjMotionVecX=para.GetParamVecDbl("MovingObjMotionVecX");
  } else {
    _MovingObjMotionVecX=para.AddParamVecDbl("MovingObjMotionVecX", "vector of x components of moving object centers", dflt, 0, grp); 
  }

  dflt[0]=0.0; dflt[1]=0.0; dflt[2]=1.5; dflt[3]=1.5;
  if (para.CheckParam("MovingObjMotionVecY")){ 
    _MovingObjMotionVecY=para.GetParamVecDbl("MovingObjMotionVecY");
  } else {
    _MovingObjMotionVecY=para.AddParamVecDbl("MovingObjMotionVecY", "vector of y components of moving object centers", dflt, 0, grp); 
  }

  dflt[0]=20.0; dflt[1]=20.0; dflt[2]=18.0; dflt[3]=18.0;
  if (para.CheckParam("MovingObjMotionVecZ")){ 
    _MovingObjMotionVecZ=para.GetParamVecDbl("MovingObjMotionVecZ");
  } else {
    _MovingObjMotionVecZ=para.AddParamVecDbl("MovingObjMotionVecZ", "vector of z components of moving object centers", dflt, 0, grp); 
  }

  dflt[0]=0.0; dflt[1]=0.0; dflt[2]=0.0; dflt[3]=0.0;
  if (para.CheckParam("MovingObjRotVecX")){ 
    _MovingObjRotVecX=para.GetParamVecDbl("MovingObjRotVecX");
  } else {
    _MovingObjRotVecX=para.AddParamVecDbl("MovingObjRotVecX", "vector of rotation of moving object around x-axis", dflt, 0, grp); 
  }
 
  dflt[0]=0.0; dflt[1]=-0.1; dflt[2]=0.0; dflt[3]=0.1;
  if (para.CheckParam("MovingObjRotVecY")){ 
    _MovingObjRotVecY=para.GetParamVecDbl("MovingObjRotVecY");
  } else {
    _MovingObjRotVecY=para.AddParamVecDbl("MovingObjRotVecY", "vector of rotation of moving object around y-axis", dflt, 0, grp); 
  }
 
  dflt[0]=0.0; dflt[1]=0.0; dflt[2]=0.0; dflt[3]=0.0;
  if (para.CheckParam("MovingObjRotVecZ")){ 
    _MovingObjRotVecZ=para.GetParamVecDbl("MovingObjRotVecZ");
  } else {
    _MovingObjRotVecZ=para.AddParamVecDbl("MovingObjRotVecZ", "vector of rotation of moving object around z-axis", dflt, 0, grp); 
  }

  if (para.IsUsedGroupID(grp))
    para.SetGroupName(grp, "moving objects");

}

bool GenSynthMatches::_CheckParams() const
{
  if (*_NumPoints!=*_NumStaticPoints+*_NumOutliers+
      *_NumMovingPoints * *_NumMovingObjects){
    BIASERR("inconsistent parameter (number of points), correcting");
    *_NumPoints = *_NumStaticPoints+*_NumOutliers+
      *_NumMovingPoints * *_NumMovingObjects;
    //    BIASABORT;
    return false;
  }
  if (*_NumMovingObjects>_SizeMovingObjX->size() ||
      *_NumMovingObjects>_SizeMovingObjY->size() ||
      *_NumMovingObjects>_SizeMovingObjZ->size()){
    BIASERR("inconsistent parameter (NumMovingObjects and SizeMovingObj?)"
            <<*_NumMovingObjects<<" "<<_SizeMovingObjX->size()
            <<" "<<_SizeMovingObjY->size()<<" "<<_SizeMovingObjZ->size());
    BIASABORT;
    //return false;
  }
  int n=*_NumMovingObjects**_NumImages;
  if (n>_MovingObjMotionVecX->size() ||
      n>_MovingObjMotionVecY->size() ||
      n>_MovingObjMotionVecZ->size() ||
      n>_MovingObjRotVecX->size() ||
      n>_MovingObjRotVecY->size() ||
      n>_MovingObjRotVecZ->size()){
    BIASERR("inconsistent parameter (NumMovingObjects, NumImages and MovingObj*Vec?)");
    BIASABORT;
    //return false;
  }
  if ((int)_MotionVecX->Size()<*_NumImages ||
      (int)_MotionVecY->Size()<*_NumImages ||
      (int)_MotionVecZ->Size()<*_NumImages ||
      (int)_RotVecX->Size()<*_NumImages ||
      (int)_RotVecY->Size()<*_NumImages ||
      (int)_RotVecZ->Size()<*_NumImages){
    BIASERR("MotionVec? and RotVec? must be of size NumImages");
    BIASABORT;
    //return false;
  }

  return true;
}


GenSynthMatches &GenSynthMatches::operator=(const MatchDataBase& mdb)
{
  ((MatchDataBase)(*this))=mdb;
  
  _GT_R=_R;
  _GT_C=_C;
  _GT_P=_P;
  _MovingTransform.resize(0);
  _StaticPoints.resize(0);
  _MovingPoints.resize(0);
  *_NumStaticPoints=*_NumOutliers=*_ImWidth=*_ImHeight=0;
  *_FocalLength=*_PrinciplePointX=*_PrinciplePointY=0.0;

  *_SigmaImagePoints=*_MaxDispOutliers=*_OutlDirChange=*_OutlLengthChange=*_SigmaCamCenterX=*_SigmaCamCenterY=*_SigmaCamCenterZ=*_SigmaCamRotX=*_SigmaCamRotY=*_SigmaCamRotZ=*_MinCamDistance=*_MaxCamDistance=0.0;

  *_UserGivenP=true;
  
  _MotionVecX->newsize(0);
  _MotionVecY->newsize(0);
  _MotionVecZ->newsize(0);
  _RotVecX->newsize(0);
  _RotVecY->newsize(0);
  _RotVecZ->newsize(0);
  _MovingObjMotionVecX->newsize(0);
  _MovingObjMotionVecY->newsize(0);
  _MovingObjMotionVecZ->newsize(0);
  _MovingObjRotVecX->newsize(0);
  _MovingObjRotVecY->newsize(0);
  _MovingObjRotVecZ->newsize(0);

  *_OverallMotionX=*_OverallMotionY=*_OverallMotionZ=*_OverallRotX=*_OverallRotY=*_OverallRotZ=0.0;

  
   *_NumMovingPoints=*_NumMovingObjects=0;
   _SizeMovingObjX->newsize(0);
   _SizeMovingObjY->newsize(0);
   _SizeMovingObjZ->newsize(0);

   _Points.resize(0);
   _NormalizedPoints.resize(0);

   return *this;
}
