/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be usefulen,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Utils/TriangleMesh.hh>
#include <Base/Math/Vector.hh>

// #include <iostream>

using namespace BIAS;
using namespace std;

void TriangleMesh::
GenerateDenseMesh(const BIAS::Image<float>& DenseDepthMap,
		  BIAS::PMatrix& P,
		  const BIAS::Image<unsigned char>& Texture)
{
  if( (DenseDepthMap.GetWidth()!=Texture.GetWidth()) ||
      (DenseDepthMap.GetHeight()!=Texture.GetHeight()) ) {
    BIASERR("Texturesize != DepthMapsize \n");
    return ;
  }    

  //init datastructs  
  unsigned int w=DenseDepthMap.GetWidth();
  unsigned int h=DenseDepthMap.GetHeight();
  vector<int> pointIndices;
  pointIndices.resize(w*h);

  meshVertices_.clear();
  vertexNormals_.clear();
  textureCoords_.clear();
  triangleIndices_.clear();

  meshVertices_.reserve(w*h);
  vertexNormals_.reserve(w*h);
  textureCoords_.reserve(w*h);
  triangleIndices_.reserve(w*h*2);

  texture_ = Texture;

  //build list of indices and list of Vertices
  float texCoordOffsetX = 1.0f / float(w-1);
  float texCoordOffsetY = 1.0f / float(h-1);
  Vector2<float> texCoord;
  texCoord.Set(0.0, 1.0);
  
  Vector3<double> vertex;
  HomgPoint3D vertexHom;
    
  float** depthValues = DenseDepthMap.GetImageDataArray();
  unsigned int currentIndex=0;
  unsigned int x,y;
  for( y=0; y<h; y++) {
    for( x=0; x<w; x++) {
      if(depthValues[y][x]<= 0.0) {
	pointIndices[y*w+x] = -1;
      }
      else {
	pointIndices[y*w+x] = currentIndex;
	P.BackprojectWorldCoo(HomgPoint2D(x,y), depthValues[y][x], vertexHom);
	vertexHom.GetEuclidian(vertex);
	meshVertices_.push_back(vertex);       
	textureCoords_.push_back(texCoord);
	currentIndex++;
      }
      texCoord[0] += texCoordOffsetX;
    } 
    texCoord[0] = 0.0;
    texCoord[1] -= texCoordOffsetY;
  }
  
  //meshing
  Vector3<int> triangle;
  int index0, index1, index2;
  for( y=0; y<h; y++) {
    for( x=0; x<w-1; x++) {
      /*         v1
       *      i0---i1
       *       |   /
       *       |  / v0
       *       i2
       */
      if(y<h-1) {
	index0 = pointIndices[y*w+x];
	index1 = pointIndices[y*w+x+1];
	index2 = pointIndices[(y+1)*w+x];
	if(index0 > (-1) && index1 > (-1) && index2 > (-1)){
	  triangle.Set(index2, index1, index0);
	  //	  triangleIndices_.push_back(Vector<unsigned int>(triangle));
	  triangleIndices_.push_back(triangle);
	}
      }    
      /*
       *         i0 
       *        / |
       *       /  |
       *      i2--i1
       */
      if(y>0) {
	index2 = pointIndices[y*w+x];
	index1 = pointIndices[y*w+x+1];
	index0 = pointIndices[(y-1)*w+x+1];
	if(index0 > (-1) && index1 > (-1) && index2 > (-1)) {
	  triangle.Set(index2, index1, index0);
	  //	  triangleIndices_.push_back(Vector<unsigned int>(triangle));
	  triangleIndices_.push_back(triangle);
	}
      }
    }
  }
  
//   cout<<"vertices "<<meshVertices_.size()<<endl;
//   cout<<"texCoords "<<textureCoords_.size()<<endl;
//   cout<<"triangles "<<triangleIndices_.size()<<endl;
    
}
